<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: EditgeomController.php 3209 2012-12-13 11:03:33Z vlad.bruma $
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Default
 * @copyright  Copyright (c) 2012 VEC
 */

class EditgeomController extends Giscuit_Controller_Ows
{
    
    protected $_table;

    protected $_form;

    protected $_id;
    
    protected $_editgeom;
    
    protected $_editattr;

    public function init ()
    {
        parent::init();
        
        $this->fixUploadedFileNames();
        
        $translate = Zend_Registry::get('Zend_Translate');
        $layersTable = new Giscuit_Db_Table_Layers();
        $result = $layersTable->getEditLayer();
        
        if(
            ! isset($result['settings']) ||
            (! $result['editgeom'] && ! $result['editattr']) 
            ) {
            throw new Giscuit_Exception('No editable layer found');
        }
        
        $this->_editgeom = $result['editgeom'];
        $this->_editattr = $result['editattr'];
        $settings = $result['settings'];
        
        //Table
        $tableName = 'Giscuit_Db_Table_Layer_' . $settings['class'];
        $this->_table = new $tableName(array(
            'name' => $settings['table']
        ));
        $this->_table->setSettings($settings);
        
        //Form
        $this->_form = $this->getForm($settings['class']);
        
        //Id
        $request = $this->getRequest();
        $this->_id = intval($request->getParam('id', 0));
        
        //Masking
        $maskingPlugin = Zend_Controller_Front::getInstance()->getPlugin('Giscuit_Controller_Plugin_Masking');
        if ($maskingPlugin != NULL && $this->_id) {
            if (! $maskingPlugin->isAllowedByGid($this->_table, $this->_id)) {
                throw new Giscuit_Exception($translate->_('geometryOutsideAllowedZone'));
            }
        }
    
    }

    protected function _addLayer ()
    {            
        if ($this->_table == NULL || $this->_map->getScale() > $this->_table->editScale) {
            return;
        }
        
        $settings = $this->_table->getSettings();
        $settings['applyClassesStylesLabels'] = false; 
        
        $layerName = 'Giscuit_Map_Layer_' . $settings['class'];  
        $layer = new $layerName($this->_map, $settings);
  
        $layer->set('dump', MS_TRUE);
        $layer->setMetaData('wfs_title', $layer->name);
        //FIXME: Static gid name 
        $layer->setMetaData('gml_featureid', 'gid');
        $layer->setMetaData('gml_include_items', 'all');
        
        //Get select
        $select = $this->_table->getSelect('gid_geometry');

        
        //Mask layer
        $maskingPlugin = Zend_Controller_Front::getInstance()->getPlugin('Giscuit_Controller_Plugin_Masking');
        if ($maskingPlugin != NULL) {
            $maskingPlugin->whereIntersections($select, 'role_edit', $settings['table']);
        }
        
        $layer->setData($select);
        
        //FIXME:
        $layer->getMsObj()->set('name', 'giscuit_edit');

    }
    
    public function getAction ()
    {
        if($this->_editgeom == false && $this->_editattr == false) {
            $this->triggerError("Editing is forbidden");    
        }
        
        $result['replyCode'] = 200;

        $request = $this->getRequest();
        $version = intval($request->getParam('version', 0));
        
        if( empty($this->_id)) {

        } else {
            $editingPlugin = Zend_Controller_Front::getInstance()->getPlugin('Giscuit_Controller_Plugin_Editing');

            $settings = $this->_table->getSettings();
            
            if($this->_table instanceof Giscuit_Db_Table_Layer_History) {
                $historyTableName = 'Giscuit_Db_Table_Layer_' . $settings['class'];
                    
                $historyTableOptions = array(
                    'name' => $settings['table'] ,
                    'withHistory' => true
                );
                                
                $historyTable = new $historyTableName($historyTableOptions);
                $historyTable->setSettings($settings);

                if(! empty($version)) {
                    $data = $editingPlugin->getItemForEdit($this->_id, $historyTable, $this->_form, $version);
                } else {
                    $data = $editingPlugin->getItemForEdit($this->_id, $this->_table, $this->_form);
                }
                
                if(! empty($version)) {
                    $result['geometry'] = $data['the_geom'];
                }

                $versions = $historyTable->getHistoryVersions($this->_id);
                $this->view->versions = $versions;
                    
                $this->view->version = $version;
                
            } else {
                $data = $editingPlugin->getItemForEdit($this->_id, $this->_table, $this->_form);
            }    

            $this->_form->populate($data->toArray());

        } 
        
        $this->view->form = $this->_form;

        $result['form'] = $this->view->render('editgeom' . DIR_SEP . 'get' . GT_SCRIPT_EXT);
        
        $this->_helper->json($result);
    }
    
    public function addAction ()
    {
        if($this->_editgeom == false) {
            $this->triggerError("Editing is forbidden");    
        }
        
        $result['replyCode'] = 200;
        
        $translate = Zend_Registry::get('Zend_Translate');
        $request = $this->getRequest();
        $geometry = $request->getParam('geometry');
        
        $validateGeometry = new Giscuit_Validate_Geometry();
        $validateResult = $validateGeometry->isValid($geometry);
        if ($validateResult !== true) {
            foreach ($validateGeometry->getMessages() as $key => $value) {
                $result['errors'][] = $translate->_('invalidGeometry') . ": " . $key;
            }
        }
        
        $maskingPlugin = Zend_Controller_Front::getInstance()->getPlugin('Giscuit_Controller_Plugin_Masking');
        if ($maskingPlugin != NULL && $validateResult) {
            if (! $maskingPlugin->isAllowedByGeometry($geometry)) {
                $result['errors'][] = $translate->_('geometryOutsideAllowedZone');
            }
        }
        /*
        if (! $validateGeometry->isValid($geometry)) {
            throw new Giscuit_Exception("Invalid geometry $geometry");
            
        }
        */
        //Remove file elements if $_FILES is empty
        if(empty($_FILES)) {    
            foreach ($this->_form->getElements() as $element) {
                if($element instanceof Zend_Form_Element_File) {
                    $this->_form->removeElement($element->getName());
                }
            }
        }
        
        $this->_form->populate($request->getParams());
        
        if (! $this->_form->isValid($request->getParams()) || isset($result['errors'])) {
            
            $result['success'] = false;
            $this->view->form = $this->_form;
            $result['form'] = $this->view->render('editgeom' . DIR_SEP . 'get' . GT_SCRIPT_EXT);

        } else {
            $result['success'] = true;
            
            $editingPlugin = Zend_Controller_Front::getInstance()->getPlugin('Giscuit_Controller_Plugin_Editing');
            $result['gid'] = $editingPlugin->addItemForEdit($this->_table, $geometry, $this->_form);
        }
        
        $layout = Zend_Layout::getMvcInstance();
        if ($layout instanceof Zend_Layout) {
            $layout->disableLayout();
        }
        $this->_helper->viewRenderer->setNoRender();
        
        if(GT_TESTING == false) {
            //FIXME: Avoid converting to html entities by the browser
            header("Content-Type: text/plain");
        }
        
        //FIXME: Json helper does not work file uploads
        echo Zend_Json::encode($result);
        
    }

    public function deleteAction ()
    {
        if($this->_editgeom == false) {
            $this->triggerError("Editing is forbidden");    
        }
        
        $translate = Zend_Registry::get('Zend_Translate');
        
        $result['replyCode'] = 200;
        
        $request = $this->getRequest();
        $id = intval($request->getParam('id', 0));
        
        $editingPlugin = Zend_Controller_Front::getInstance()->getPlugin('Giscuit_Controller_Plugin_Editing');
        $editingPlugin->deleteItemForEdit($id, $this->_table, $this->_form);
        
        $this->_helper->json($result);
    }
    
    public function editAction ()
    {
        if($this->_editgeom == false && $this->_editattr == false) {
            $this->triggerError("Editing is forbidden");    
        }
        
        $translate = Zend_Registry::get('Zend_Translate');
        
        $result['replyCode'] = 200;
        $request = $this->getRequest();
        
        if($this->_editgeom == false) {
            $geometry = NULL;
        } else {
            $geometry = $request->getParam('geometry');
        
            //Validate geometry
            if(! empty($geometry)) {
                $validateGeometry = new Giscuit_Validate_Geometry();
                $validateResult = $validateGeometry->isValid($geometry);
                if ($validateResult !== true) {
                    foreach ($validateGeometry->getMessages() as $key => $value) {
                        $result['errors'][] = $translate->_('invalidGeometry') . ": " . $key;
                    }
                }
            } else {
                $geometry = NULL;
            }
        }
        
        //Remove file elements if $_FILES is empty
        if(empty($_FILES)) {    
            foreach ($this->_form->getElements() as $element) {
                if($element instanceof Zend_Form_Element_File) {
                    $this->_form->removeElement($element->getName());
                }
            }
        }
        
        $this->_form->populate($request->getParams());
        
        if (! $this->_form->isValid($request->getParams()) || isset($result['errors'])) {
            
            $result['success'] = false;
            $this->view->form = $this->_form;
            $result['form'] = $this->view->render('editgeom' . DIR_SEP . 'get' . GT_SCRIPT_EXT);
            
        } else {
            $result['success'] = true;
            
            $editingPlugin = Zend_Controller_Front::getInstance()->getPlugin('Giscuit_Controller_Plugin_Editing');
            $editingPlugin->updateItemForEdit($this->_id, $this->_table, $geometry, $this->_form);
        }
        
        $layout = Zend_Layout::getMvcInstance();
        if ($layout instanceof Zend_Layout) {
            $layout->disableLayout();
        }
        $this->_helper->viewRenderer->setNoRender();
        
        if(GT_TESTING == false) {
            //FIXME: Avoid converting to html entities by the browser
            header("Content-Type: text/plain");
        }
        
        //FIXME: Json helper does not work file uploads
        echo Zend_Json::encode($result);
        
    }
    
    public function backAction ()
    {
        if($this->_editgeom == false) {
            $this->triggerError("Editing is forbidden");    
        }
        
        $result['replyCode'] = 200;
        
        $request = $this->getRequest();

        $result['changesCount'] = count(Zend_Registry::get('settings')->editingHistory);
        
        //Get last action
        if($result['changesCount'] > 0) {
        
            $lastAction = Zend_Registry::get('settings')->editingHistory[count(Zend_Registry::get('settings')->editingHistory) - 1];
            
            //History Table
            $settings = $this->_table->getSettings();
            $historyTableName = 'Giscuit_Db_Table_Layer_' . $settings['class'];
            $this->_history = new $historyTableName(array(
                'withHistory' => true ,
                'name' => $settings['table']
            ));
            $this->_history->setSettings($settings);
            
            //Get fields
            $fields = $this->_table->info('cols');
            
            if($lastAction['hstatus'] == 'U') {
                
                //Update
                $historyRow = $this->_history->find($lastAction['prev'])->current();
                
                $row = $this->_table->find($historyRow->gid)->current();
                foreach($fields as $field) {
                    $row->$field = $historyRow->$field;
                }
                $row->save();
                
                //Save feature history
                $editingPlugin = Zend_Controller_Front::getInstance()->getPlugin('Giscuit_Controller_Plugin_Editing');
                $editingPlugin->setItemHistory($historyRow->gid, $this->_table, array(
                    'hstatus' => 'U'
                ));
                
            } else if($lastAction['hstatus'] == 'C') {
                
                //Create
                $historyRow = $this->_history->find($lastAction['curr'])->current();
                
                $row = $this->_table->find($historyRow->gid)->current();
                
                //Save feature history
                $editingPlugin = Zend_Controller_Front::getInstance()->getPlugin('Giscuit_Controller_Plugin_Editing');
                $editingPlugin->setItemHistory($historyRow->gid, $this->_table, array(
                    'hstatus' => 'D'
                ));
                
                $row->delete();
                
            } else if($lastAction['hstatus'] == 'D') {
                
                //Delete
                $historyRow = $this->_history->find($lastAction['prev'])->current();
                
                $row = $this->_table->createRow();
                foreach($fields as $field) {
                    $row->$field = $historyRow->$field;
                }
                
                $newGid = $row->save();
                
                //Save feature history
                $editingPlugin = Zend_Controller_Front::getInstance()->getPlugin('Giscuit_Controller_Plugin_Editing');
                $editingPlugin->setItemHistory($newGid, $this->_table, array(
                    'hstatus' => 'C'
                ));
            }
            
            //Delete last action
            unset(Zend_Registry::get('settings')->editingHistory[count(Zend_Registry::get('settings')->editingHistory) - 1]);
            
            //
            $result['changesCount'] = count(Zend_Registry::get('settings')->editingHistory);
        } 
                        
        $this->_helper->json($result);
    }
    
    public function indexAction ()
    {
        $layout = Zend_Layout::getMvcInstance();
        if ($layout instanceof Zend_Layout) {
            $layout->disableLayout();
        }       
        $this->_helper->viewRenderer->setNoRender();
        
        $this->_addMap();
        $this->_addLayer();
        
        $this->_draw(NULL);
    }

    protected function _parseRequest ($request)
    {
        $request->setParameter('SERVICE', 'WFS');
    }

    public function privateAction ()
    {

    }

    public function publicAction ()
    {

    }
    
    public function getForm($class) {
        $formName = 'Giscuit_Form_Layer_' . $class;
        
        $configPath = GT_LIBRARY_PATH . DIR_SEP . 'Giscuit' . DIR_SEP . 'Form' . DIR_SEP . 'Layer' . DIR_SEP . $class . '.xml';
        if (file_exists($configPath)) {
            $configContent = new Zend_Config_Xml($configPath);
            $form = new $formName($configContent);
        } else {
            $form = new $formName();
        }
        
        $form->setMethod('post');
        $form->addAttribs(array(
            'id' => 'gtEditattrForm'
        ));
        
        return $form;
    }
    
    public function fixUploadedFileNames()
    {
        if(! count($_FILES)) {
            return;
        }
        
        //
        $uniqueNames = array();
        foreach ($_FILES as $key => $value) {
            if(! $value['name']) {
                continue;
            }
            
            if(! isset($uniqueNames[$value['name']])) {
                $uniqueNames[$value['name']] = 1;
            } else {
                $uniqueNames[$value['name']] ++;
                $_FILES[$key]['name'] = $uniqueNames[$value['name']] . '-' . $_FILES[$key]['name'];
            }
        }

    }

}