<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: SnapController.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Default
 * @copyright  Copyright (c) 2012 VEC
 */

class SnapController extends Giscuit_Controller_Ows
{

    protected function _addLayer ()
    {
        $config = Zend_Registry::get('config');
       
        //Get extent    
        $extent = $this->_map->getExtent();
        
        //Get layers
        $layersTable = new Giscuit_Db_Table_Layers();
        
        $mapOptions = array(
            'addEditLayer' => false , 
            'whereView' => 't' , 
            'whereSelect' => 't' , 
            'whereActive' => 't' , 
            'withLayerSettings' => true , 
            'withAcl' => true
        );
        $result = $layersTable->getItems($mapOptions);
        
        $selects = array();
        
        //Add layers
        foreach ($result as $layerSettings) {
            
            $className = 'Giscuit_Map_Layer_'. $layerSettings['class'] ;
            $tableName = 'Giscuit_Db_Table_Layer_' . $layerSettings['class'];
            
            if(! (
                class_exists($className) && 
                is_subclass_of($className, 'Giscuit_Map_Layer_Vector')
                )) {
                    
                continue;
            }
            
            $tableOptions = array(
                'name' => $layerSettings['table']
            );
            if(! empty(Zend_Registry::get('settings')->historyDateEnd)) {
                $tableOptions['withHistory'] = true;
            }
            
            $table = new $tableName($tableOptions);
            $table->setSettings($layerSettings);
            
            //To add vertexes to editing layer
            if ($layerSettings['edit'] == 't' && 
                Zend_Registry::get('settings')->editingActive == true &&
                $this->_map->getScale() <= $table->editScale) {
                
                continue;
            }
            
            if ($this->_map->getScale() <= $table->snapScale) {
                $select = $table->getSelect('gid_geometry');
                
                if (! empty($select)) {
                    $selects[] = '(' . $select . ')';
                }
               
            }
        
        }
        
        if (! empty($selects)) {
            $selects = implode("\nUNION\n", $selects);
            
            $layer = new Giscuit_Map_Layer_Snap($this->_map, array(
                'class' => 'Snap' ,
                'table' => 'virtual' ,
                'applyClassesStylesLabels' => false
            ));
            
            $layer->set('dump', MS_TRUE);
            $layer->setMetaData('wfs_title', $layer->name);
            //FIXME: Static gid name 
            $layer->setMetaData('gml_featureid', 'gid');
            $layer->setMetaData('gml_include_items', 'gid');
            
            $layer->setData($selects);
            
            //FIXME:
            $layer->getMsObj()->set('name', 'giscuit_snap');
            
        }
    }

    public function indexAction ()
    {
        $layout = Zend_Layout::getMvcInstance();
        if ($layout instanceof Zend_Layout) {
            $layout->disableLayout();
        }
        $this->_helper->viewRenderer->setNoRender();
        
        $this->_addMap();
        $this->_addLayer();
        
        $this->_draw(NULL);
    }

    protected function _parseRequest ($request)
    {
        $request->setParameter('SERVICE', 'WFS');
    }

    public function privateAction ()
    {

    }

    public function publicAction ()
    {

    }

}
