<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: GeocodeController.php 2993 2012-09-12 14:43:49Z vlad.bruma $
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Tool
 * @copyright  Copyright (c) 2012 VEC
 */

class Tool_GeocodeController extends Zend_Controller_Action
{
    /**
     * Maximum number of results returned 
     *
     * @var integer
     */
    public $limit = 10;
    
    /**
     * Similarity threshold
     * 
     * @var float
     */
    public $trgmLimit = 0.2;

    public function indexAction ()
    {
        $config = Zend_Registry::get('config');
                
        $request = $this->getRequest();
        
        //Text
        $text = $request->getParam('text', '');
        $query = $request->getParam('query', '');
        if($text === '' && $query !== '') {
            $text = $query;
        }
        
        //Location
        $location = $request->getParam('location', '');
        /*
        //FIXME:
        $location = $query;
        $text = '';
        */
        
        $location = array_map('floatval', explode(',', $location));
        if(count($location) != 2) {
            $location = '';
        }
        
        //Extent
        $extent = $request->getParam('extent', '');
        $extent = array_map('floatval', explode(',', $extent));
        if(count($extent) != 4) {
            $extent = '';
        }
        
        //Centroid
        $centroid = $request->getParam('centroid', 'f') == 't' ? true : false;
        /*
        //FIXME:
        $centroid = true;
        */
        
        //Limit
        $limit = intval($request->getParam('limit', 0));
        if(! $limit || ($limit < 1 && $limit > 15)) {
            $limit = $this->limit;
        }
        
        //Source and scale
        $sourceParam = $request->getParam('source');
        if(empty($sourceParam)) {
            $source = $config->geocode->default;
            foreach ($config->geocode->sources as $value) {
                if($value->id == $source) {
                    $scale = $value->scale;
                    break;
                }
            }
        } else {
            foreach ($config->geocode->sources as $value) {
                if($value->id == $sourceParam) {
                    $source = $value->id;
                    $scale = $value->scale;
                    break;
                }
            }
        }   
        if (empty($source)) {
            $result = array(
                'records' => array(),
                'replyCode' => 500
            );
            $this->_helper->json($result);
            return;
        }
        
        //
        if($text !== '') {
            $result = $this->_gtForward($source, $text, $extent, $centroid, $limit);
        } else if ($location !== '') {
            $result = $this->_gtReverse($source, $location, $extent, $centroid, $limit);
        } else {
            $result = array(
                'records' => array(),
                'replyCode' => 501
            );
            $this->_helper->json($result);
            return;
        }
        
        $result['replyCode'] = 200;
        $result['scale'] = $scale;
        $this->_helper->json($result);
    }
    
    protected function _gtForward($source, $text, $extent = '', $centroid = false, $limit = 10) {
        $config = Zend_Registry::get('config');
        $db = Zend_Registry::get('db');
        
        $result['records'] = array();
        
        if ($source == 'google') {
            $extent = "{$extent[1]},{$extent[0]}|{$extent[3]},{$extent[2]}";
                
            //Google geocode
            $url = "http://maps.googleapis.com/maps/api/geocode/json?address=" . urlencode($text)             
                . "&sensor=false"
                . "&bounds=" . $extent
                . "&language=" . Zend_Registry::get('Zend_Translate')->getLocale();
                    
            $data = Zend_Json::decode(file_get_contents($url));
                
            if($data["status"] == "OK") {
                foreach($data["results"] as $feature) {
                        
                    $viewport = array(
                       $feature['geometry']['viewport']['southwest']['lng'] ,
                       $feature['geometry']['viewport']['southwest']['lat'] ,
                       $feature['geometry']['viewport']['northeast']['lng'] ,
                       $feature['geometry']['viewport']['northeast']['lat']
                    ); 
                        
                    $result['records'][] = array(
                        'text' => $feature['formatted_address'] ,
                        'the_geom' => "POINT({$feature['geometry']['location']['lng']}, {$feature['geometry']['location']['lat']})" ,
                        'viewport' => $viewport
                    );
                 }
            }
             
        } else {
            
            $tableAndField = explode('.', $source);
            
            //Centroid
            if($centroid) {
                $theGeomQuery = "ST_Centroid({$tableAndField[0]}.the_geom)";
                $theGeomLineCentroidQuery = "CASE  
                    WHEN GeometryType({$tableAndField[0]}.the_geom) = 'MULTILINESTRING' THEN ST_AsText(ST_Line_Interpolate_Point(ST_GeometryN({$tableAndField[0]}.the_geom, 1), 0.5))
                    WHEN GeometryType({$tableAndField[0]}.the_geom) = 'LINESTRING' THEN ST_AsText(ST_Line_Interpolate_Point({$tableAndField[0]}.the_geom, 0.5))
                    ELSE NULL END";
            } else {
                $theGeomQuery = "{$tableAndField[0]}.the_geom";
                $theGeomLineCentroidQuery = 'NULL';
            }
            
            //Select
            $select = $db->select();
            $select->from("{$config->database->schema}_layers.{$tableAndField[0]}", array(
                //'gid',
                $tableAndField[1],
                'the_geom' => new Zend_Db_Expr("ST_AsText({$theGeomQuery})"),
                'the_geom_line_centroid' => new Zend_Db_Expr($theGeomLineCentroidQuery)
            ));

            //Extent
            if(count($extent) == 4) {
                $extent = "ST_SetSRID('BOX3D({$extent[0]} {$extent[1]},{$extent[2]} {$extent[3]})'::box3d, {$config->map->srid})";
                $select->where("\"$tableAndField[0]\".\"the_geom\" && {$extent}");
            }
            
            //Text
            $db->query("SELECT set_limit({$this->trgmLimit})");
            $select->where("\"$tableAndField[0]\".\"$tableAndField[1]\" % ?", $text);
            $textQuoted = $db->quote($text);
            $select->order("similarity(\"$tableAndField[0]\".\"$tableAndField[1]\", {$textQuoted}) DESC");
            
            //Limit
            $select->limit($limit);
            
            //Result
            $matches = $db->fetchAll($select);
            foreach ($matches as $match) {
                if(empty($match->the_geom_line_centroid)) {
                    $theGeomResult = $match->the_geom;
                } else {
                    $theGeomResult = $match->the_geom_line_centroid;
                }
                $result['records'][] = array(
                    //'gid' => $match->gid,
                    'text' => $match->$tableAndField[1] ,
                    'the_geom' => $theGeomResult
                );
            }
        }
        
        return $result;
    }
    
    protected function _gtReverse($source, $location, $extent = '', $centroid = false, $limit = 10) {
                
        $config = Zend_Registry::get('config');
        $db = Zend_Registry::get('db');
                
        $result['records'] = array();
        
        if ($source == 'google') {
            
        } else {
            $tableAndField = explode('.', $source);
            $locationGeom = "ST_SetSrid(ST_GeometryFromText('POINT({$location[0]} {$location[1]})'), {$config->map->srid})";
            
            //Centroid
            if($centroid) {
                $theGeomQuery = "ST_Centroid({$tableAndField[0]}.the_geom)";
                $theGeomLineCentroidQuery = "CASE  
                    WHEN GeometryType({$tableAndField[0]}.the_geom) = 'MULTILINESTRING' 
                        THEN ST_AsText(ST_Line_Interpolate_Point(
                            ST_GeometryN({$tableAndField[0]}.the_geom, 1),
                            ST_line_locate_point(ST_GeometryN({$tableAndField[0]}.the_geom, 1), $locationGeom)
                        ))
                    WHEN GeometryType({$tableAndField[0]}.the_geom) = 'LINESTRING' 
                        THEN ST_AsText(ST_Line_Interpolate_Point(
                            {$tableAndField[0]}.the_geom,
                            ST_line_locate_point({$tableAndField[0]}.the_geom, $locationGeom)
                        ))
                    ELSE 
                        NULL 
                    END";
            } else {
                $theGeomQuery = "{$tableAndField[0]}.the_geom";
                $theGeomLineCentroidQuery = 'NULL';
            }
            
            //Select
            $select = $db->select();
            $select->from("{$config->database->schema}_layers.{$tableAndField[0]}", array(
                //'gid',
                $tableAndField[1],
                'the_geom' => new Zend_Db_Expr("ST_AsText({$theGeomQuery})"),
                'the_geom_line_centroid' => new Zend_Db_Expr($theGeomLineCentroidQuery)
            ));
            
            //Extent
            if(count($extent) == 4) {
                $extent = "ST_SetSRID('BOX3D({$extent[0]} {$extent[1]},{$extent[2]} {$extent[3]})'::box3d, {$config->map->srid})";
                $select->where("\"$tableAndField[0]\".\"the_geom\" && {$extent}");
            } else {
                if($config->map->units == 'dd') {
                    $withInDistance = 0.01;
                } else {
                    $withInDistance = 1110;
                }
                
                $select->where("ST_DWithin(\"$tableAndField[0]\".\"the_geom\", {$locationGeom}, {$withInDistance})");
            }
            
            //Order
            $select->order("ST_Distance({$tableAndField[0]}.the_geom, $locationGeom) ASC");
            
            //Limit
            $select->limit($limit);
            
            //Result
            $matches = $db->fetchAll($select);
            foreach ($matches as $match) {
                if(empty($match->the_geom_line_centroid)) {
                    $theGeomResult = $match->the_geom;
                } else {
                    $theGeomResult = $match->the_geom_line_centroid;
                }
                $result['records'][] = array(
                    //'gid' => $match->gid,
                    'text' => $match->$tableAndField[1] ,
                    'the_geom' => $theGeomResult
                );
            }
        }
        
        return $result;
        
    }
    
}