<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * @see Zend_Controller_Plugin_Abstract
 */
require_once 'Zend/Controller/Plugin/Abstract.php';

/**
 * Plugin for quick zone access
 *  
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Controller_Plugin_Quickaccess extends Giscuit_Controller_Plugin_Abstract
{

    protected $_translations = array(
        'en' => array(
            'quickaccess' => 'Quick access',
            'published' => 'Published'
        ),
        'ro' => array(
            'quickaccess' => 'Acces rapid',
            'published' => 'Publicat'
        ),
        'ru' => array(
            'quickaccess' => 'Быстрый доступ',
            'published' => 'Опубликованный'
        )
    );
    
    public $layerClass = 'Quickaccess';
    
    
    /**
     * List of the javascript files to include
     * 
     * @return array
     */
    public function getJsFilesList ()
    {
        return array(
            'Control/Quickaccess.js'
        );
    }
    
    /**
     * List of the javascript file to include
     * 
     * @return array
     */
    public function getMobileJsFilesList() 
    {
        return array(
            'Mobile/Control/Quickaccess.js'
        );
    }
    
    /**
     * Install plugin
     * 
     * @return boolean
     */
    public function install($newConfig) 
    {
        $db = Zend_Registry::get('db');
        $rulesTable = new Giscuit_Db_Table_Rules();
        $layersTable = new Giscuit_Db_Table_Layers();

        //Insert rules and roles for layer access
        $data = array('role' => 'registered', 'resource' => 'quickaccess_layer', 'privilege' => 'view', 'group' => 'layer');
        $rulesTable->insert($data);
        
        $data = array('role' => 'registered', 'resource' => 'quickaccess_layer', 'privilege' => 'edit', 'group' => 'layer');
        $rulesTable->insert($data);
        
        //Create mask table
        $schema = $newConfig->production->database->schema;
        $layersSchema = $schema . '_layers';
        $db->beginTransaction();
        try {
            $db->query("
            CREATE TABLE {$layersSchema}.quickaccess (
                gid serial,
                name_id integer,
                user_id integer,
                published character(1) DEFAULT 'f'::bpchar,
                the_geom public.geometry,
                CONSTRAINT enforce_dims_the_geom CHECK ((public.st_ndims(the_geom) = 2)),
                CONSTRAINT enforce_geotype_the_geom CHECK (((public.geometrytype(the_geom) = 'MULTIPOLYGON'::text) OR (the_geom IS NULL)))
            );");

            $db->query("ALTER TABLE ONLY {$layersSchema}.quickaccess ADD CONSTRAINT quickaccess_pkey PRIMARY KEY (gid);");

            $db->commit();
        } catch (Exception $e) {
            $db->rollBack();
            throw new Giscuit_Db_Table_Exception($e->getMessage());
        }
        
        //Create layer
        $form = new Giscuit_Form_Admin_LayersAdd();
        $form->getElement('class')->setValue($this->layerClass);
        $form->getElement('name_en')->setValue($this->_translations['en']['quickaccess']);
        $form->getElement('table')->setValue('quickaccess');
        $layerId = $layersTable->add($form, array(
            'type' => 'vector' , 
            'layerType' => 'Polygon' ,
            'parentId' => ''
        ));
        
        
        $db->beginTransaction();
        try {
            $db->query("UPDATE {$schema}.layers SET active = 't', edit_maxscale = 50000, resource = 'quickaccess_layer', priv_view = 'view', priv_editgeom = 'edit' WHERE id = {$layerId};");
            $db->commit();
        } catch (Exception $e) {
            $db->rollBack();
            throw new Giscuit_Db_Table_Exception($e->getMessage());
        }
        
        //Add default section translations
        foreach ($this->_translations as $languageKey => $languageValue) {
            $languageFile = GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx';
            if(! file_exists($languageFile)) {
                continue;       
            }
            
            $translate = new Zend_Translate('tmx', $languageFile, $languageKey);
        
            $newMessages = array_merge($translate->getMessages(), $languageValue);
            $newTmx = Giscuit_Controller_Plugin_Abstract::arrayToTmx($languageKey, $newMessages);
            
            file_put_contents(GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx', $newTmx);
        }
    }
    
    /**
     * Uninstall plugin
     *  
     * @return boolean
     */
    public function uninstall($newConfig) 
    {
        $db = Zend_Registry::get('db');
        $rulesTable = new Giscuit_Db_Table_Rules();
        
        //Delete rules and roles
        $where = $rulesTable->getAdapter()->quoteInto('resource = ?', 'quickaccess_layer');
        $rulesTable->delete($where);
        
        //Drop tables
        $schema = $newConfig->production->database->schema;
        $db->beginTransaction();
        try {
            $db->query("DROP TABLE IF EXISTS {$schema}_layers.quickaccess CASCADE;");
            
            $db->commit();
        } catch (Exception $e) {
            $db->rollBack();
            throw new Giscuit_Db_Table_Exception($e->getMessage());
        }
        
        //Delete layer and it's files
        $layersTable = new Giscuit_Db_Table_Layers();
        $layerData = $layersTable->getItems(array('whereClass' => $this->layerClass));
        
        reset($layerData);
        $layerId = key($layerData);
        
        if ($layerId != NULL) {
            $layersTable->delete('id = ' . $layerId);
        }
        
        //Delete translations
        foreach ($this->_translations as $languageKey => $languageValue) {
            $languageFile = GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx';
            if(! file_exists($languageFile)) {
                continue;       
            }
            
            $translate = new Zend_Translate('tmx', $languageFile, $languageKey);
            
            $translations = $translate->getMessages();
            foreach ($languageValue as $key => $value) {
                unset($translations[$key]);
            }

            $newTmx = Giscuit_Controller_Plugin_Abstract::arrayToTmx($languageKey, $translations);
            file_put_contents(GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx', $newTmx);
        }
    }
    
    /**
     * Get parameters
     * 
     * @return array
     */
    public function getParams()
    {
        $config = Zend_Registry::get('config');
        $translate = Zend_Registry::get('Zend_Translate');
        $locale = $translate->getLocale();
        $request = $this->getRequest();
                
        $layerTable = new Giscuit_Db_Table_Layer_Quickaccess(array(
            'name' => 'quickaccess'
        ));
        $select = $layerTable->getSelect('all_table', array(
            'geometryAsWkt' => true,
            'forMenu' => true
        ));
        
        $select->joinLeft(array(
            'text_name' => $config->database->schema . '.text'
        ), "text_name.id = quickaccess.name_id", array(
            'name' => $locale
        ));
        
        $rows = $layerTable->fetchAll($select)->toArray();
        
        return array(
            'quickaccessRecords' => $rows
        );    
    }
}