<?php
/**
 * Giscuit
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @subpackage Layer
 * @copyright  Copyright (c) 2011 Giscuit
 */

/**
 * @see Giscuit_Db_Table_Layer_Abstract
 */
require_once 'Giscuit/Db/Table/Layer/Abstract.php';

/**
 * World boundaries class for table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @subpackage Layer
 * @copyright  Copyright (c) 2011 Giscuit
 */
class Giscuit_Db_Table_Layer_Worldboundaries extends Giscuit_Db_Table_Layer_Abstract
{

    /**
     * Get neighbours
     * 
     * @param float $lon
     * @param float $lat
     * @param array $options <ul>Available options:
     * <li>limit integer - number of results to be fetched</li></ul>
     * @return Zend_Db_Table_Rowset
     */
    public function getNeighbours($lon, $lat, $options = array()) {
        $config = Zend_Registry::get('config');
        
        $select = $this->select();
        $select->setIntegrityCheck(false);
        
        $select->from(
            array(
                'a' => $this->_schema . '.' . $this->_name,
            ),
            'name'
        );
        
        //Find all the polygons that touch
        $select->joinInner(array(
            'b' => $this->_schema . '.' . $this->_name
        ), "st_touches(a.the_geom, b.the_geom)", array(
            'neighbour' => 'name'
        ));

        //Find to which polygon this coordinates belong
        $select->where("st_intersects('SRID={$config->map->srid};POINT($lon $lat)'::geometry, a.the_geom)");
        
        //Set the result limit from the plugin's parameters
        if (isset($options['limit'])) {
            $select->limit($options['limit']);
        }
                
        //Fetch the results from the database
        $result = $this->fetchAll($select);
        
        return $result;
    }

}