<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Rastercut.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Form
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Form_Abstract
 */
require_once 'Giscuit/Form/Abstract.php';

/**
 * Download raster form class
 * 
 * @category   Giscuit
 * @package    Giscuit_Form
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Form_Rastercut extends Giscuit_Form_Abstract
{
    /**
     * Initialize
     *
     * @return void
     */
    public function init ()
    {
        //Config
        $config = Zend_Registry::get('config');
        
        $isAllowedHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('isAllowed');
        
        //Error
        $this->addElement('hidden', 'geometry', array(
            'required' => true , 
            'id' => 'gtRastercutGeometry' , 
            'validators' => array(
                'Geometry'
            ) , 
            'decorators' => array(
                'ViewHelper' , 
                'Errors'
            )
        ));
        
        //Comment
        if ($isAllowedHelper->direct('rastercut_watermark')) {
            $date = new Zend_Date();
            $auth = Zend_Auth::getInstance();
            
            $commentValue = $auth->getIdentity()->fname . ' ' . $auth->getIdentity()->lname . ' ' . $date->get(GT_DATE_STYLE_PHP);
            $commentRequired = false;
        
        } else {
            $commentValue = '';
            $commentRequired = true;
        }
        $this->addElement('textarea', 'comment', array(
            /*
            'filters' => array(
                'StringTrim' , 
                'HtmlSpecialChars'
            ) , 
            'validators' => array() , 
            */
            'value' => $commentValue , 
            'required' => $commentRequired , 
            'label' => 'comment' , 
            'rows' => 3 , 
            'cols' => 30
        )//'id' => 'rastercutComment'
        );
        
        //Resolution
        $resolutionArr = array(
            72 => 72 , 
            200 => 200 , 
            300 => 300
        );
        $this->addElement('select', 'resolution', array(
            'filters' => array(
                'Digits'
            ) , 
            /*
            'validators' => array(
                array(
                    'InArray' , 
                    false , 
                    array(
                        $resolutionArr
                    )
                )
            ) ,
            */ 
            'required' => true , 
            'label' => 'resolution' , 
            //'id' => 'rastercutResolution' , 
            'multiOptions' => $resolutionArr
        ));
        
        //Scale
        $scales = explode(',', $config->map->resolutions);
        foreach ($scales as $value) {
            $value = round(Giscuit_Map_Map::getScaleFromResolution($value));
            $scaleArr[$value] = '1:' . $value;
        }
        
        $this->addElement('select', 'scale', array(
            'filters' => array(
                'Digits'
            ) , 
            /*
            'validators' => array(
                array(
                    'InArray' , 
                    false , 
                    array(
                        array_keys($scaleArr)
                    )
                )
            ) ,
            */ 
            'required' => true , 
            'label' => 'scale' , 
            'id' => 'gtRastercutScale' , 
            'multiOptions' => $scaleArr
        ));
        
        if ($isAllowedHelper->direct('rastercut_tiled')) {
            $this->addElement('checkbox', 'tiled', array(
                'validators' => array() , 
                'label' => 'tiled'
            ));
        }
        
        $this->addElement('button', 'submit', array(
            'required' => false , 
            'ignore' => true , 
            'label' => 'submit' , 
            'attribs' => array(
                'id' => 'gtRastercutSubmit'
            ) ,
            /* 
            'decorators' => array(
                'ViewHelper'
            )
            */
        ));
    
    }
}