<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Wms.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Map
 * @subpackage Layer
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * @see Giscuit_Map_Layer_Abstract
 */
require_once 'Giscuit/Map/Layer/Abstract.php';

/**
 * Class for abstract mapserver wms layer object
 *
 * @category   Giscuit
 * @package    Giscuit_Map
 * @subpackage Layer
 * @copyright  Copyright (c) 2012 VEC
 */
abstract class Giscuit_Map_Layer_Wms extends Giscuit_Map_Layer_Abstract
{
    /**
     * Request timeout
     * 
     * @var string
     */
    public $timeout = 10;
    
    /**
     * Class constructor
     *
     * @param mixed $map Map object
     * @param array $settings Settings
     * @return void
     */
    public function __construct ($map, $settings)
    {
        parent::__construct($map, $settings);
        
        $config = Zend_Registry::get('config');
        
        $this->setConnectionType(MS_WMS);    
        $this->setMetaData("wms_srs", "EPSG:" . $config->map->srid);
                
        $this->set('connection',  $this->_settings['owsclient_uri']);
        $this->setMetaData("wms_name", $this->_settings['owsclient_layers']);
        
        //Wms format
        if ($this->_settings['owsclient_image_format'] == 'jpeg') {
            $this->setMetaData("wms_format", 'image/jpeg');    
            $this->setMetaData("wms_transparent", MS_FALSE);
        } elseif($this->_settings['owsclient_image_format'] == 'png') {
            $this->setMetaData("wms_format", 'image/png');
        } elseif($this->_settings['owsclient_image_format'] == 'png24') {
            $this->setMetaData("wms_format", 'image/png; mode=24bit');
        }      

        //Authentication
        if(! empty($this->_settings['owsclient_auth_type'])) {
            $this->setMetaData("wms_auth_type", $this->_settings['owsclient_auth_type']);
            $this->setMetaData("wms_auth_username", $this->_settings['owsclient_auth_username']);
            $this->setMetaData("wms_auth_password", $this->_settings['owsclient_auth_password']);
        }
        
        //Proxy
        if(! empty($config->proxy->host) && ! empty($config->proxy->port)) {
            $this->setMetaData("wms_proxy_type", "http");
            $this->setMetaData("wms_proxy_host", $config->proxy->host);
            $this->setMetaData("wms_proxy_port", $config->proxy->port);
            
            if (! empty($config->proxy->username) && ! empty($config->proxy->password)) {
                $this->setMetaData("wms_proxy_auth_type", "anysafe");
                $this->setMetaData("wms_proxy_username", $config->proxy->username);
                $this->setMetaData("wms_proxy_password", $config->proxy->password);
            }
        }
        
        $this->setMetaData("wms_connectiontimeout", $this->timeout);
        $this->setMetaData("wms_server_version", $this->_settings['owsclient_version']);
    
    }
    
    /**
     * Get data by point coordinates
     * 
     * @param float $lon
     * @param float $lat
     * @param array $options <ul>Available options:
     * <li>featureCount integer - Number of results to return</li>
     * <li>infoFormat string - The format the format in which the result should be requested. Depends on remote server’s capabilities.</li></ul>
     * @return array
     */
    public function getByPoint($x, $y, array $options = array())
    {
        
        if (empty($options['featureCount'])) {
            $options['featureCount'] = 1;
        }
        
        if(empty($options['infoFormat'])) {
            $options['infoFormat'] = 'text/html';
        }
        
        $ctx = stream_context_create(array(
            'http' => array(
                'timeout' => $this->timeout
            )
        ));
        
        $uri = $this->getWMSFeatureInfoURL($x, $y, $options['featureCount'], $options['infoFormat']);
        $contents = file_get_contents($uri, 0, $ctx);
        
        if($options['infoFormat'] == 'text/plain') {
            $contents = '<pre>' . $contents . '</pre>';
        }

        return $contents;
    }
    
}