<?php

/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Navigation.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Menu
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * Class for hierarchic navigation menu
 * 
 * @category   Giscuit
 * @package    Giscuit_Menu
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Menu_Navigation extends Giscuit_Menu
{
    /**
     * Build a menu using Yahoo User Interface javascript template
     * 
     * @param array $data
     * @return string
     */
    public function buildMenu (array &$data)
    {
        $js = '[';
        
        foreach ($data as $row) {
            $js .= $this->_buildRow($row);
            if (! empty($row['child'])) {
                $js .= sprintf(',submenu:{id:YAHOO.util.Dom.generateId(), itemdata: %s}', $this->buildMenu($row['child']));
            }
            $js .= "},";
        }
        
        if(strlen($js) > 1) {
            $js = substr($js, 0, - 1);
        }
        $js .= ']';
        
        return $js;
    }
    
    /**
     * Build menu element
     * 
     * @param array $row Element
     * @return string
     */
    protected function _buildRow ($row)
    {
        $text = "{";
        $text .= "text: '{$row['title']}' ";
        if (! empty($row['url'])) {
            $text .= ", url: '{$row['url']}' ";
        }
        if (! empty($row['target']) && $row['target'] == 't' && $row['type'] != 1) {
            $text .= ", target: '_blank' ";
        }
        
        return $text;
    }
    
    
    
    /**
     * Formats menu elements
     * 
     * @param array $child Elnewsement of the menu
     * @return array
     */
    protected function _parse ($child)
    {
        $locale = Zend_Registry::get('Zend_Translate')->getLocale();
        
        $url = Zend_Controller_Action_HelperBroker::getStaticHelper('url');
        $baseUrl = $url->url(array(
            'language' =>  $locale,
            'module' => 'default'
        ), null, true);
        
        if (! empty($child['type'])) {
            $voidUrl = "javascript:void(0);";
            
            if ($child['type'] == 1) {
                $child['url'] = $voidUrl;
            } elseif ($child['type'] == 2) {
                $child['url'] = $url->url(array(
                    'language' => $locale , 
                    'module' => 'default' ,
                    'controller' => 'menu' , 
                    'action' => 'browse' , 
                    'id' => $child['id']
                ), null, true);
            } else {
                $child['url'] = str_replace('<language>', $locale, $child['url']);
                $child['url'] = str_replace('<front>', $baseUrl, $child['url']);
            }
        }
        
        if (! empty($child['url'])) {
            $child['url'] = htmlspecialchars($child['url'], ENT_QUOTES);
        }
        $child['title'] = htmlspecialchars($child['title']);
        
        return $child;
    }
    
    /**
     * Initialize the creation of a nested menu from 2d array data
     * 
     * @param array $data
     * @return array
     */
    public function buildNestedArray (array $data)
    {
        $translate = Zend_Registry::get('Zend_Translate');
        $locale = Zend_Registry::get('Zend_Translate')->getLocale();
         
        $url = Zend_Controller_Action_HelperBroker::getStaticHelper('url');
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        
        
        if (! empty($userId)) {
            array_push($data, array(
                'id' => 65535 , 
                'parent_id' => 0 , 
                'type' => 3 , 
                'url' => $url->url(array(
                    'language' => $locale ,
                    'module' => 'default' ,
                    'controller' => 'auth' , 
                    'action' => 'logout'
                ), null, true) , 
                'title' => $translate->_('logout')
            ));
        } else {
            array_push($data, array(
                'id' => 65534 , 
                'parent_id' => 0 , 
                'type' => 3 , 
                'url' => $url->url(array(
                    'language' => $locale ,
                    'module' => 'default' ,
                    'controller' => 'auth'
                ), null, true) , 
                'title' => $translate->_('login')
            ));
            array_push($data, array(
                'id' => 65535 , 
                'parent_id' => 0 , 
                'type' => 3 , 
                'url' => $url->url(array(
                    'language' => $locale ,
                    'module' => 'default' ,
                    'controller' => 'auth',
                    'action' => 'signup'
                ), null, true) , 
                'title' => $translate->_('signUp')
            ));
        }
        
        $this->_data = $data;
        $this->_buildNestedArray($this->_data, $this->_arr, 0);
        
        
        return $this->_arr; 
    }

}