<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: DbTable.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Session
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Zend_Session_SaveHandler_DbTable
 */
require_once 'Zend/Session/SaveHandler/DbTable.php';

/**
 * Giscuit_Session_SaveHandler_DbTable
 *  
 * @category   Giscuit
 * @package    Giscuit_Session
 * @subpackage SaveHandler
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Session_SaveHandler_DbTable extends Zend_Session_SaveHandler_DbTable
{

    /**
     * Garbage Collection
     *
     * @param int $maxlifetime
     * @return true
     */
    public function gc ($maxlifetime)
    {
        return true;
    }

    /**
     * Write session data
     *
     * @param string $id
     * @param string $data
     * @return boolean
     */
    public function write ($id, $data)
    {
        if (empty($data)) {
            return true;
        }
        
        $return = false;
        
        $data = array(
            $this->_modifiedColumn => time() , 
            $this->_dataColumn => (string) $data
        );
        
        $rows = call_user_func_array(array(
            &$this , 
            'find'
        ), $this->_getPrimary($id));

        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        if (empty($userId)) {
            $data['user_id'] = NULL;
        } else {
            $data['user_id'] = $userId;
        }

        if (count($rows)) {
            if($rows->current()->data == $data['data']) {
                unset($data['data']);
            }
                        
            $data[$this->_lifetimeColumn] = $this->_getLifetime($rows->current());
            
            if ($this->update($data, $this->_getPrimary($id, self::PRIMARY_TYPE_WHERECLAUSE))) {
                $return = true;
            }
        } else {
            $data[$this->_lifetimeColumn] = $this->_lifetime;
            
            if ($this->insert(array_merge($this->_getPrimary($id, self::PRIMARY_TYPE_ASSOC), $data))) {
                $return = true;
            }
        }
        
        return $return;
    
    }

    /**
     * Close session
     *
     * @return boolean
     */
    public function close ()
    {
        $chance = 1;
        $outOf = 100;
        
        $random = mt_rand(1, $outOf);
        $hit = $random <= $chance;

        if ($hit == true) {
            if(! $this->getAdapter()->isConnected()) {
                return false;
            }
            
            $lifetime = (int) ini_get('session.gc_maxlifetime');
            
            $this->delete($this->getAdapter()->quoteIdentifier($this->_modifiedColumn) . ' + ' . $this->getAdapter()->quoteIdentifier($this->_lifetimeColumn) . ' < ' . $this->getAdapter()->quote(time()));
            
            if ($handle = opendir(GT_TMP_PATH)) {
                while (false !== ($file = readdir($handle))) {
                    
                    //Delete all zip files
                    if (substr($file, -4) == '.zip') {
                        $mtime = filemtime(GT_TMP_PATH . DIR_SEP . $file);
                        if ($mtime + $lifetime < time()) {
                            @unlink(GT_TMP_PATH . DIR_SEP . $file);
                        }
                    }
                }
                closedir($handle);
            }
        
        }
        
        return true;
    }

}
