<?php

//Patch information
$currentVersion = '1.3.1';
$updateVersion = '1.3.2';
$downloadsUrl = 'http://downloads.giscuit.com';

//Defines
require_once '..' . DIRECTORY_SEPARATOR . 'Defines.php';
if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
    define('GT_MS_PATH', realpath(GT_APP_PATH . DIR_SEP . '..' . DIR_SEP . '..' . DIR_SEP . '..'));    
    } else {
    define('GT_MS_PATH', NULL);
}

//Errors
error_reporting(E_ALL | E_STRICT);
ini_set('display_startup_errors', 1);
ini_set('display_errors', 1);
ini_set('log_errors', 0);

//Include_path
$paths = array(
    realpath(GT_LIBRARY_PATH) , 
    get_include_path()
);
set_include_path(implode(PATH_SEPARATOR, $paths));

//Loader
require_once 'Zend/Loader/Autoloader.php';
$loader = Zend_Loader_Autoloader::getInstance();
$loader->registerNamespace('Giscuit_');

//Translate
$translate = new Zend_Translate('tmx', GT_LANGUAGES_PATH . DIR_SEP . 'admin.en.tmx', 'en');
Zend_Registry::set('Zend_Translate', $translate);

//Config
$config = new Zend_Config_Xml(GT_CONFIG_PATH . DIR_SEP . 'config.xml', null, array(
    'skipExtends' => true , 
    'allowModifications' => true
));

//
$view = new Zend_View();
$view->type = 'patch';
$view->httpPath = $config->production->httpPath;
$view->licenseType = $config->production->licenses->type;


//
$view->params = '';

//Install 
if (isset($_GET['install']) && $_GET['install'] == 't') {
	$view->install = 't';
	
	if ($config->production->version == $updateVersion) {
    	$view->bd = "System was already updated to this version";
    	
	} else if ($config->production->version != $currentVersion) {
    	$view->bd = "This patch is for version {$currentVersion}";
	    
	} else {
	    //Initialize database
	    Giscuit_Controller_Plugin_Init::parseDbParams($config->production->database);
	    $db = Zend_Db::factory($config->production->database);
	    $db->getConnection();
	    Zend_Db_Table_Abstract::setDefaultAdapter($db);
	    
	    try {
	    	//Patch process BEGIN
	    	
	        //Update languages
	        updateLanguages(array(
	            'en' => array(
	                'downloading' => 'Downloading' ,
	                'notAvailableInOffline' => 'Not available in offline' ,
	                'geometryOutsideAllowedZone' => 'Geometry outside of allowed zone' ,
	                'hide' => 'Hide',
	                'remove' => 'Remove',
	        		'synchronizing' => 'Synchronizing' ,
	        		'userData' => 'User data' ,
	                'gpsAccuracyExceedsLimit' => 'Gps accuracy is more than %1$sm. Are you sure you want to add?' ,
	                'moveVertexToCurrentPosition' => 'Move vertex to current geolocation' ,
	                'editAttributes' => 'Edit feature attributes' ,
	                'editGeometry' => 'Edit feature geometry' ,
	                'finalizeSketch' => 'Finalize feature' ,
	                'removeVertex' => 'Delete vertex' ,
	                'addFeature' => 'Add feature' ,
	                'deleteFeature' => 'Delete feature' ,
	                'undoFeature' => 'Undo feature' ,
	                'editFeature' => 'Edit feature' ,
	                'saveFeature' => 'Save feature' ,
	                'undoVertex' => 'Undo vertex'
	            ),
	            'ro' => array(
	                'downloading' => 'Descărcarea' ,
	                'notAvailableInOffline' => 'Nu este disponibil în regim autonom' ,
	                'geometryOutsideAllowedZone' => 'Geometria în afară zonei permise' ,
	                'hide' => 'Ascunde' ,
	                'remove' => 'Şterge' ,
	        		'synchronizing' => 'Sincronizarea' ,
	        		'userData' => 'Datele utilizatorului' ,
	                'gpsAccuracyExceedsLimit' => 'Gps precizia este mai mare decît %1$sm. Sunteţi siguri că doriţi să adăugaţi?' ,
	                'moveVertexToCurrentPosition' => 'Deplasarea nodului la poziţia curentă' ,
	                'editAttributes' => 'Editarea atributelor obiectului' ,
	                'editGeometry' => 'Editarea geometriei obiectului' ,
	                'finalizeSketch' => 'Finalizează obiect' ,
	                'removeVertex' => 'Şterge nodul' ,
	                'addFeature' => 'Adaugă obiect' ,
	                'deleteFeature' => 'Şterge obiect' ,
	                'undoFeature' => 'Anulează obiect' ,
	                'editFeature' => 'Editează obiect' ,
	                'saveFeature' => 'Salvaează obiect' ,
	                'undoVertex' => 'Anulează nodul'
	            ),
	            'ru' => array(
	                'downloading' => 'Загрузка' ,
	                'notAvailableInOffline' => 'Не доступно в автономном режиме' ,
	                'geometryOutsideAllowedZone' => 'Геометрия за пределами разрешенной зоны' ,
	                'hide' => 'Спрятать' ,
	                'remove' => 'Удалить' ,
	        		'synchronizing' => 'Синхронизация' ,
	        		'userData' => 'Данные пользователя' ,
	                'gpsAccuracyExceedsLimit' => 'Точность GPS больше чем %1$sм. Вы уверены, что хотите добавить?' ,
	                'moveVertexToCurrentPosition' => 'Переместить точку к текущей позиции' ,
	                'editAttributes' => 'Измененить атрибуты объекта' ,
	                'editGeometry' => 'Измененить геометрию объекта' ,
	                'finalizeSketch' => 'Завершить объект' ,
	                'removeVertex' => 'Удалить точку' ,
	                'addFeature' => 'Добавить объект' ,
	                'deleteFeature' => 'Удалить объект' ,
	                'undoFeature' => 'Анулировать объект' ,
	                'editFeature' => 'Изменить объект' ,
	                'saveFeature' => 'Сохранить объект' ,
	                'undoVertex' => 'Анулировать точку'
	            )
	        ));
	        
	        //Update rules
	        $defaultSchema = $config->production->database->schema;
	        
	        $patchSql = array(
	            "ALTER TABLE {$defaultSchema}.layers ADD COLUMN max_gps_accuracy integer"
	        );
	        
	        foreach ($patchSql as $value) {
	            $db->query($value);
	        }
	        
	        //Update config
	    	
	        //Patch process END
	        $config->production->version = $updateVersion;
	        
	        //
	        require_once GT_MODULES_PATH . DIR_SEP . 'admin' . DIR_SEP . 'controllers' . DIR_SEP . 'GenconfController.php';
	        Admin_GenconfController::escapeConfig($config->production);
	        
	        //
	        $writer = new Zend_Config_Writer_Xml(array(
	            'config' => $config , 
	            'filename' => GT_CONFIG_PATH . DIR_SEP . 'config.xml'
	        ));
	        $writer->write();
	        
	        //License
	        if(isset($_POST['agreement_text']) && $_POST['agreement_text']) {
	            file_put_contents(GT_APP_PATH . DIR_SEP . "LICENSE.txt", $_POST['agreement_text']);
	        }
	        
	        //Clear cache
	        $frontendOptions = array(
	            'lifetime' => 3600 , 
	            'automatic_serialization' => true
	        );
	        if(extension_loaded('apc')) {
	            $backendOptions = array(
	            );
	            $backend = 'Apc';            
	        } else {
	            $backendOptions = array(
	                'cache_dir' => GT_TMP_PATH
	            );
	            $backend = 'File';
	        }
	        $cache = Zend_Cache::factory('Core', $backend, $frontendOptions, $backendOptions);
	        $cache->clean(Zend_Cache::CLEANING_MODE_ALL);
	        
	        //Update cache
	        file_put_contents(GT_TMP_PATH . DIR_SEP . 'offline_timestamp', time());
	    
	        $view->bd = "Giscuit was successfully patched to version " . $updateVersion;
	    
	    } catch (Exception $e) {
	        $db->rollBack();
	        $view->bd = $e->getMessage();
		}
	}
} else {
	$view->install = 'f';
	
    $contentView = new Zend_View();
    $contentView->type = 'patch';
    $contentView->addScriptPath(GT_MODULES_PATH . DIR_SEP . 'default' . DIR_SEP . 
        'views' . DIR_SEP . 'scripts' . DIR_SEP . 'generic');
    
    $view->bd = $contentView->render('installLicenseAgreement' . GT_SCRIPT_EXT);
    
}

$view->addScriptPath(GT_LAYOUTS_PATH);
echo $view->render('install' . GT_SCRIPT_EXT);

function updateLanguages(array $messages = array()) {
    foreach ($messages as $languageKey => $languageValue) {
        
        $languageFile = GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx';
        if(! file_exists($languageFile)) {
            continue;       
        }
        
        $translate = new Zend_Translate('tmx', $languageFile, $languageKey);
    
        $newMessages = array_merge($translate->getMessages(), $languageValue);
        $newTmx = Giscuit_Controller_Plugin_Abstract::arrayToTmx($languageKey, $newMessages);
        
        file_put_contents(GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx', $newTmx);
    }    
}