/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Historymanage.js 3110 2012-11-08 08:02:59Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * Class: Giscuit.Control.Historymanage
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Control.Historymanage = OpenLayers.Class(Giscuit.Control, {

    panel: null,
    
    submitButton: null,
    
    resetButton: null,
    
    revertButton: null,
    
    cal: null,
    
    ds: null,
    
    dt: null,
    
    dtHeightCoef: 280,
    
    selected: null,

    activate: function () {
        var activated = Giscuit.Control.prototype.activate.call(this);

        if (activated == true) {
            
            gt.control.editgeom.deactivate();
            
            //Check button
            gt.control.editgeom.button.set("label", Giscuit.Util.getMenuItemById(
                gt.control.editgeom.button, 
                'gtDatamanageHistoryControl'
            ).cfg.getProperty('text'));
            YAHOO.util.Dom.removeClass(
                gt.control.editgeom.button.get('element'), 
                'yui-checkbox-button-checked'
            );
            
            this.create();
        }
  
        return activated;
    },

    deactivate: function () {
        var deactivated = Giscuit.Control.prototype.deactivate.call(this);
        
        //Destroy features
        if(this.selected != null) {
            gt.layer.vector.destroyFeatures([this.selected]);
            this.selected = null;
        }
        
        return deactivated;
    },
    
    clear: function() {
        this.deactivate();
        
        gt.accord.unsubscribe('afterPanelOpen', this.updateDtHeight);
        
        var elements = [
            'dt' ,
            'cal' ,    
            'submitButton' ,
            'resetButton' ,
            'revertButton'
         ];
         for(var i=0; i < elements.length; i++) {
             if(this[elements[i]] != null) {
                 this[elements[i]].destroy();
                 this[elements[i]] = null;
             }
         }        
        
         YAHOO.util.Event.purgeElement('gtHistorymanageLayers', true);
        
         Giscuit.Control.prototype.clear.call(this);
    },

    createSuccess: function (o) {

        gt.accord.setBody(this.getPanelIndex(), o.responseText);

        var inTxt = YAHOO.util.Dom.get("gtHistorymanageDateBeginId");
        var outTxt = YAHOO.util.Dom.get("gtHistorymanageDateEndId");
                
        this.cal = new Giscuit.Calendar.Interval("gtHistorymanageCalContainer", {
            pages: 1,
            selected: initInterval,
            START_WEEKDAY: 1
        });

        //Select calendar dates from form text fields
        var initInterval = null;
        if(outTxt.value) {
            var initInterval = Giscuit.Util.parseDate(outTxt.value);
            
            if(inTxt.value) {
                var initInterval = new Array(
                    initInterval,
                    Giscuit.Util.parseDate(inTxt.value)
                );
            }
        }
        if(initInterval != null) {
            this.cal.select(initInterval);
        }
        
        this.cal.selectEvent.subscribe(function () {
            var interval = this.getInterval();

            if (interval.length == 2) {

                if (interval[0].getTime() != interval[1].getTime()) {
                    inTxt.value = YAHOO.util.Date.format(interval[0], {
                        format: gt.config.yuiDatetimeFormat
                    });
                    
                    outTxt.value = YAHOO.util.Date.format(interval[1], {
                        format: gt.config.yuiDatetimeFormat
                    });
                } else {
                    inTxt.value = '';

                    outTxt.value = YAHOO.util.Date.format(interval[1], {
                        format: gt.config.yuiDatetimeFormat
                    });
                }
            }
        },
        this.cal, true);

        this.cal.render();
        
        //Submit button
        this.submitButton = new YAHOO.widget.Button('gtHistorymanageSubmit');
        this.submitButton.on("click", this.submit, null, this);
        
        //Reset button
        this.resetButton = new YAHOO.widget.Button('gtHistorymanageReset');
        this.resetButton.on("click", this.reset, null, this);
        
        //Revert button
        this.revertButton = new YAHOO.widget.Button('gtHistorymanageRevert');
        this.revertButton.set('disabled', true);
        this.revertButton.on("click", this.revert, null, this);
        
        //
        YAHOO.util.Event.addListener('gtHistorymanageLayers', "change", function (e, obj) {
            if(this.value != 0) {
                obj.createDatatable(this.value);
            } else {
                if(this.dt != null) {
                    this.dt.destroy();
                    this.dt = null;
                }
                YAHOO.util.Dom.get("gtHistorymanageDt").innerHTML = '';
            }
        }, this);

    },

    create: function () {
        
        if(this.panel == null) {
            gt.accord.addPanel({
                label: gt.translate._('datahistory') ,
                content: this.getLoadingImage()
            });
            this.panel = gt.accord.getPanel(gt.accord.getPanels().length - 1);
            this.addHelpButton('http://docs.giscuit.com/En/tutorial_refguide.user.desktop.edit.pkg.html#history');
        } else {
            gt.accord.setBody(this.getPanelIndex(), this.getLoadingImage());
        }
        gt.accord.getPanel(this.getPanelIndex()).children[1].style.overflow = 'hidden';
        gt.accord.openPanel(this.getPanelIndex());
        
        gt.accord.on('afterPanelOpen', this.updateDtHeight, null, this);

        var createCallback = {
            success: this.createSuccess,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };

        var request = YAHOO.util.Connect.asyncRequest('GET', gt.config.httpToolPath + "/historymanage", createCallback);
    },
    
    createDatatable: function(id) {

        var createDatatableCallback = {
            success: this.createDatatableSuccess,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };
        
        var request = YAHOO.util.Connect.asyncRequest('GET', gt.config.httpToolPath + "/historymanage/get/id/" + id, createDatatableCallback);
    },
    
    createDatatableSuccess: function(o) {
        try {
            var responseText = YAHOO.lang.JSON.parse(o.responseText);
        } catch(e) {
            Giscuit.Util.parseFailure(o, e);
            return;
        }
        
        if (responseText.replyCode != 200) {
            Giscuit.Util.invalidReplyCode(responseText);
            return;
        }
        
        if(this.dt != null) {
            this.dt.destroy();
            this.dt = null;
        }
        
        //Create data source
        this.ds = new YAHOO.util.DataSource(responseText['records']);
        this.ds.responseType = YAHOO.util.DataSource.TYPE_JSARRAY;
        this.ds.responseSchema = {
            fields: responseText['fields']
        };
        
        //Limit column display number
        if (responseText['columns'].length > gt.config.maxFieldsAmount) {
            responseText['columns'] = responseText['columns'].slice(0, gt.config.maxFieldsAmount);
            
            //Show column limitation tooltip
            var firstNFieldsDisplayed = gt.translate._('firstNFieldsDisplayed').replace("%1$s", gt.config.maxFieldsAmount);
            gt.tooltip.cfg.setProperty('text', firstNFieldsDisplayed);
            gt.tooltip.showTooltip();
        }
        
        //Create data table
        this.dt = new YAHOO.widget.DataTable('gtHistorymanageDt', responseText['columns'], this.ds, {
            scrollable: true,
            width: '99.5%',
            selectionMode: "single",
            MSG_EMPTY: gt.translate._('emptyResult'),
            MSG_LOADING: gt.translate._('loading') + "..."
        });
        
        this.setOptimalDatatableHeight(this.dt, this.dtHeightCoef);
        
        //Select row
        this.dt.subscribe("cellClickEvent", function (oArgs) {
            this.dt.onEventSelectRow(oArgs);
            
            this.revertButton.set('disabled', false);
            
            var row = this.dt.getSelectedRows();
            
            var geomtryWkt = this.dt.getRecord(row[0]).getData('the_geom');
                        
            if(geomtryWkt instanceof Array) {
                geomtryWkt = geomtryWkt[1];
            }
            
            var geometry = OpenLayers.Geometry.fromWKT(geomtryWkt);
            
            if(this.selected != null) {
                gt.layer.vector.destroyFeatures([this.selected]);
                this.selected = null;
            }
            
            this.selected = new OpenLayers.Feature.Vector(geometry);
            
            gt.layer.vector.addFeatures([this.selected]);
            
            gt.map.zoomToExtent(geometry.getBounds());
            
        }, null, this);
    },

    submitSuccess: function () {
        gt.map.refresh();
        this.submitButton.set('disabled', false);
    },

    submit: function () {
        //Disable editing
        gt.control.editgeom.deactivate(false);
        
        //Disable submit button
        this.submitButton.set('disabled', true);
        //Destroy data table
        if(this.dt != null) {
            this.dt.destroy();
            this.dt = null;
        }
        YAHOO.util.Dom.get("gtHistorymanageLayers").selectedIndex = 0;
        YAHOO.util.Dom.get("gtHistorymanageDtPH").style.display = 'block';
        
        var submitCallback = {
            success: this.submitSuccess,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };

        YAHOO.util.Connect.setForm('gtHistorymanageForm');
        YAHOO.util.Connect.asyncRequest('POST', gt.config.httpToolPath + "/historymanage/submit", submitCallback);
    },
    
    reset: function() {
        //Disable editing
        gt.control.editgeom.deactivate(false);
        
        YAHOO.util.Dom.get("gtHistorymanageDateBeginId").value = '';
        YAHOO.util.Dom.get("gtHistorymanageDateEndId").value = '';
          
        this.cal.clear();
        this.submit();
          
        YAHOO.util.Dom.get("gtHistorymanageDtPH").style.display = 'none';
    },
    
    revertSuccess: function(o) {
        this.revertButton.set('disabled', false);
        
        try {
            var responseText = YAHOO.lang.JSON.parse(o.responseText);
        } catch(e) {
            Giscuit.Util.parseFailure(o, e);
            return;
        }
        
        if (responseText.replyCode != 200) {
            Giscuit.Util.invalidReplyCode(responseText);
            return;
        }
        
        //Destroy features
        if(this.selected != null) {
            gt.layer.vector.destroyFeatures([this.selected]);
            this.selected = null;
        }
        
        //Update data table
        var layers = YAHOO.util.Dom.get('gtHistorymanageLayers');
        var currentLayer = layers[layers.selectedIndex].value;
        this.createDatatable(currentLayer);
        
        //Update map
        gt.map.refresh();
    },

    revert: function() {
        var row = this.dt.getSelectedRows();
        var hgid = this.dt.getRecord(row[0]).getData('hgid');
        
        var layers = YAHOO.util.Dom.get('gtHistorymanageLayers');
        var layerId = layers[layers.selectedIndex].value;
                
        var revertCallback = {
            success: this.revertSuccess,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };
        
        this.revertButton.set('disabled', true);

        var request = YAHOO.util.Connect.asyncRequest('GET', gt.config.httpToolPath + "/historymanage/revert/hgid/" + hgid + "/layer_id/" + layerId, revertCallback);
    },
    
    updateDtHeight: function(args) {
        if(this.panel == args.panel && this.dt) {
            this.setOptimalDatatableHeight(this.dt, this.dtHeightCoef);
        }
    },
    
    CLASS_NAME: "Giscuit.Control.Historymanage"

});