/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Control.js 1132 2011-03-11 09:34:36Z ue $
 * @copyright  Copyright (c) 2012 VEC
 */
Giscuit.Map.prototype.gtEventListeners.push({
    'draw': function() {
        gt.control.link = new Giscuit.Control.Link();
    }
});

/**
 * Class: Giscuit.Control.Link
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Control.Link = OpenLayers.Class(Giscuit.Control, {
    
    panel: null,
    
    button: null,
    
    drawControl: null,
    
    features: [],
    
    markerExists: false,
    
    tooltipShown: false,
    
    keyPressTimer: null,
    
    mailButton: null,
    
    initialize: function (options) {
        Giscuit.Control.prototype.initialize.apply(this, [options]);
        
        var self = this;
        
        this.initializeLayer();
        
        //Events
        gt.map.globalSelectControl.setLayer(
            Giscuit.Util.unique(gt.map.globalSelectControl.layers.concat([gt.layer.marker]))
        );
        
        gt.layer.marker.events.register("featureselected", this, this.onFeatureSelect);
        gt.map.events.register("moveend", this, this.change);
        gt.control.layers.events.register("changelayer", this, this.change);
        gt.layer.marker.events.register("featureadded", this, this.change);
        gt.layer.marker.events.register("featureremoved", this, this.change);
        
        this.button = new YAHOO.widget.Button({ 
            label: gt.map.getControlIcon(12), 
            id: "gtLinkControl", 
            onclick: {fn: function() {
                if(self.active) {
                    self.deactivate();
                } else {
                    self.activate();
                }
            }},
            title: gt.translate._('markAndShare'), 
            container: "gtMapControls",
            type: 'checkbox'
        });
        
        this.populate();
    },
    
    populate: function () {
        var hash = Giscuit.Util.getHash();
        for (var key in hash) {
            if ( key == 'markers' ) {
                this.markerExists = true;
            }
        }

        if (this.markerExists == false) {
            return;
        } 
        
        var markers = decodeURIComponent(hash.markers);
        markers = markers.split('|');

        for (var i = 3; i < markers.length; i+=4) {
            this.createMarker({
                type: parseInt(markers[i-3]),
                x: parseFloat(markers[i-2]),
                y: parseFloat(markers[i-1]),
                text: markers[i]
            });
        }

    },
    
    initializeLayer: function() {
        //
        debug.info('Initializing marker layer');
        
        var markerStyle = {
            fillOpacity: 1,
            pointRadius: 17,
            graphicYOffset: -34
        };
        
        var defaultStyle = OpenLayers.Util.applyDefaults(markerStyle, OpenLayers.Feature.Vector.style["default"]);
        var selectStyle = OpenLayers.Util.applyDefaults(markerStyle, OpenLayers.Feature.Vector.style["select"]);
        var style = new OpenLayers.StyleMap({"default": defaultStyle, "select": selectStyle});
     
        gt.layer.marker = new OpenLayers.Layer.Vector("marker", {
            styleMap: style
        });
        
        gt.layer.marker.styleMap.addUniqueValueRules("default", "type", {
            1: {externalGraphic: gt.config.httpImagePath + '/marker/1.png'},
            2: {externalGraphic: gt.config.httpImagePath + '/marker/2.png'}
        });
                   
        gt.map.addLayer(gt.layer.marker);
        
        //
        var control = gt.map.globalSelectControl;
        
        for(var i=0; i < control.layers.length; i++) {
            if(! control.layers[i] || ! control.layers[i].map) {
                control.layers.splice(i, 1);
            }
        }
        
        control.layers.push(gt.layer.marker);
        control.setLayer(control.layers);
        
        gt.layer.marker.events.on({
            "featureselected": this.onFeatureSelect ,
            scope: this
        });
    },
    
    activate: function() {
        gt.map.deactivateAll();
        
        var activated = Giscuit.Control.prototype.activate.call(this);

        if (activated == true) {
            //Check button
            this.button.set('checked', true);
            
            var html = '<p>';
            
            html += '<button type="button" id="gtLinkEmailButton">' + gt.translate._('sendToEmail') + '</button>';
            
            html += '<fieldset class="gtLinkInputs"><legend>' + gt.translate._('desktopLayout') + '</legend><p>' + gt.translate._('pasteUrl') + 
                ': <br><input type="text" id="gtLinkUrl" style="width: 95%;">';
                
            if(gt.config.guest == 't') {    
                html += '<p style="margin-bottom: 0px;">' + gt.translate._('pasteHtml') + 
                    ': <br><input type="text" id="gtLinkHtml" style="width: 95%;"></p>';
            }
            html += '</fieldset>';
            
            html += '<fieldset class="gtLinkInputs"><legend>' + gt.translate._('mobileLayout') + '</legend><p>' + gt.translate._('pasteUrl') + 
                ': <br><input type="text" id="gtLinkUrlMobile" style="width: 95%;">';
            
            if(gt.config.guest == 't') {    
                html += '<p style="margin-bottom: 0px;">' + gt.translate._('pasteHtml') + 
                    ': <br><input type="text" id="gtLinkHtmlMobile" style="width: 95%;"></p>';
            }
            html += '</fieldset></p>';
            
            if(this.panel == null) {
                gt.accord.addPanel({
                    label: gt.translate._('markAndShare'),
                    content: html
                });
                this.panel = gt.accord.getPanel(gt.accord.getPanels().length - 1);
                this.addHelpButton('http://docs.giscuit.com/En/tutorial_refguide.user.desktop.link.pkg.html');
            } 

            gt.accord.setBody(this.getPanelIndex(), html);
            gt.accord.openPanel(this.getPanelIndex());
            
            //Button
            this.emailButton = new YAHOO.widget.Button('gtLinkEmailButton');
            this.emailButton.setStyle('margin-bottom', '-4px');
            this.emailButton.on('click', function (evt) {
                var subj = encodeURIComponent(gt.translate._('mapLink'));
                
                var desktopUrl = encodeURI(decodeURIComponent(YAHOO.util.Dom.get('gtLinkUrl').value));
                var mobileUrl = encodeURI(decodeURIComponent(YAHOO.util.Dom.get('gtLinkUrlMobile').value));
                var body = encodeURIComponent(
                    gt.translate._('emailLinkText')
                        .replace("%1$s", desktopUrl)
                        .replace("%2$s", mobileUrl)
                );

                window.location='mailto:?subject=' + subj + '&body=' + body;
            });
            
            this.change();
                        
            //Tooltip
            if (! this.tooltipShown && gt.config.tooltip == 't') {
                gt.tooltip.cfg.setProperty('text', gt.translate._('drawMarkerHelp'));
                gt.tooltip.showTooltip();
                this.tooltipShown = true;
            }
            
            this.drawControl = new OpenLayers.Control.DrawFeature(gt.layer.marker, OpenLayers.Handler.Point, {
                callbacks: {
                    "done": OpenLayers.Function.bind(this.done, this)
                }
            });
            gt.map.addControl(this.drawControl);
            this.drawControl.activate();
            
            gt.map.globalSelectControl.activate();

        }
        
        return activated;
    },
    
    deactivate: function () {
        var deactivated = Giscuit.Control.prototype.deactivate.call(this);

        if (deactivated == true) {
            this.drawControl.deactivate();
            this.drawControl.destroy();
            this.drawControl = null;
            
            //Uncheck button
            this.button.set('checked', false);
        }
        
        return deactivated;
    },
    
    clear: function() {
        this.deactivate();
        
        this.emailButton.destroy();
        this.emailButton = null;
        
        Giscuit.Control.prototype.clear.call(this);
    },
    
    change: function() {
        if(this.panel != null) {
            var hash = Giscuit.Util.getHash();
            var hashString = Giscuit.Util.setHash(hash);

            //Desktop
            var urlValue = window.location.protocol + '//' + window.location.host + gt.config.httpPath + '/map#' + hashString;
            YAHOO.util.Dom.get('gtLinkUrl').value = urlValue;
            
            //Mobile
            var httpPathMobile = gt.config.httpPath.substring(0, gt.config.httpPath.length - 7) + 'mobile';
            var urlValueMobile = window.location.protocol + '//' + window.location.host + 
                httpPathMobile 
                + '/map/index/lat/' + hash.lat + '/lon/' + hash.lon + '/zoom/' + hash.zoom;
            
            if(hash.layers) {
                urlValueMobile += '/layers/' + hash.layers;
            }
            if(hash.markers) {
                urlValueMobile += '/markers/' + hash.markers;
            }
            
            YAHOO.util.Dom.get('gtLinkUrlMobile').value = urlValueMobile;
            
            if(gt.config.guest == 't') {   
                //Desktop
                var htmlValue = '<iframe width="425" height="350" frameborder="0" scrolling="no" marginheight="0" marginwidth="0" src="' +
                    window.location.protocol + '//' + window.location.host + gt.config.httpPath + '/map/index/type/link#' + hashString +
                    '"></iframe>';
                YAHOO.util.Dom.get('gtLinkHtml').value = htmlValue;
                
                //Mobile
                var htmlValueMobile = '<iframe width="425" height="350" frameborder="0" scrolling="no" marginheight="0" marginwidth="0" src="' +
                    urlValueMobile + '/type/link' +
                    '"></iframe>';
                YAHOO.util.Dom.get('gtLinkHtmlMobile').value = htmlValueMobile;
            }
        }
    },
    
    closePopup: function(e, feature) {
        gt.map.globalSelectControl.unselect(feature);
    },
        
    createMarker: function (params) {
        var geom = new OpenLayers.Geometry.Point(params.x, params.y);
        var feature = new OpenLayers.Feature.Vector(geom, {type: params.type});
        var text = params.text;
        
        var popup = new OpenLayers.Popup.FramedCloud(
            "marker_" + this.features.length, 
            new OpenLayers.LonLat(geom.x,geom.y),
            new OpenLayers.Size(200, 50),
            '<textarea class="gtPopupInput">' + text + '</textarea>',
            null, true, null);
        popup.autoSize = false;
        
        popup.closeDiv.className = "gtMarkerCloseBox";
        popup.closeDiv.title = gt.translate._('hide');
        
        popup.addDeleteBox = function(callback) {
            this.deleteDiv = OpenLayers.Util.createDiv(
                this.id + "_delete", null, new OpenLayers.Size(17, 17)
            );
            this.deleteDiv.className = "olPopupDeleteBox";
            this.deleteDiv.style.zIndex = 1;
            this.deleteDiv.title = gt.translate._('remove');
       
            var contentDivPadding = this.getContentDivPadding();
            this.deleteDiv.style.right = contentDivPadding.right + "px";
            this.deleteDiv.style.top = contentDivPadding.top + 20 + "px";
            
            this.groupDiv.appendChild(this.deleteDiv);
        };
        
        popup.updateRelativePosition = function() {
            //update the padding
            this.padding = this.positionBlocks[this.relativePosition].padding;

            var contentDivPadding = this.getContentDivPadding();
            
            //update the position of our close box to new padding
            if (this.closeDiv) {
                this.closeDiv.style.right = contentDivPadding.right + this.padding.right + "px";
                this.closeDiv.style.top = contentDivPadding.top + this.padding.top + "px";
            }

            if (this.deleteDiv) {
                this.deleteDiv.style.right = contentDivPadding.right + this.padding.right + "px";
                this.deleteDiv.style.top = contentDivPadding.top + this.padding.top + 20 + "px";
            }
            
            this.updateBlocks();
            
        };
        
        popup.addDeleteBox();
        
        YAHOO.util.Event.addListener(popup.deleteDiv, "click", this.deletePopup, feature, this);
        YAHOO.util.Event.addListener(popup.closeDiv, "click", this.closePopup, feature, this);

        feature.popup = popup;
        
        this.features.push(feature);
        gt.layer.marker.addFeatures([feature]);
        gt.map.addPopup(popup);
        
        var textarea = YAHOO.util.Dom.getElementsByClassName('gtPopupInput', 'textarea', popup.contentDiv)[0];
        textarea.style.webkitUserSelect = 'text';
        YAHOO.util.Event.addListener(textarea, "keydown", this.changePopup, popup,  this);
        
        gt.map.globalSelectControl.select(feature);
    },
    
    changePopup: function(e, popup) {
        var textarea = YAHOO.util.Dom.getElementsByClassName('gtPopupInput', 'textarea', popup.contentDiv)[0];

        //Resize
        var heightOffset = 10;
        
        //FIXME: Height hack for firefox and ie
        if(YAHOO.env.ua.gecko || YAHOO.env.ua.ie) {
            heightOffset = 20;
        }
        
        //FIXME: Hack for text staying visibile in ie 
        if(YAHOO.env.ua.ie) {
            textarea.focus();
        }
        
        textarea.style.height = "1px";
        var textHeight = textarea.scrollHeight;
        if(textHeight < 30) {
            textHeight = 30;
        }
        textarea.style.height = textHeight + "px";
        
        popup.setSize(new OpenLayers.Size(200, textHeight + heightOffset));
        
        if ( this.keyPressTimer != null ) {
            this.keyPressTimer.cancel();
        }
        
        this.keyPressTimer = YAHOO.lang.later (250, this , function() {
            this.updateHash();
            this.keyPressTimer = null;
        });
    },
    
    done: function (geom) {
        this.createMarker({
            type: 1,
            x: geom.x,
            y: geom.y,
            text: '' 
        });
    },

    onFeatureSelect: function(e) {
        e.feature.popup.show();
        this.changePopup(null, e.feature.popup);
    },
    
    deletePopup: function(e, feature) {
        gt.map.removePopup(feature.popup);
        gt.layer.marker.removeFeatures([feature]);
        
        YAHOO.util.Event.purgeElement(feature.popup.contentDiv, true);
        YAHOO.util.Event.purgeElement(feature.popup.closeDiv);
        YAHOO.util.Event.purgeElement(feature.popup.deleteDiv);
        feature.destroy();

        this.updateHash();
    },

    updateHash: function(e) {
        
        var hash = Giscuit.Util.getHash();
        
        var init = false;
        
        var params = '';
        
        for (var i = 0; i < this.features.length; i++) {
            if ( this.features[i].data != null) {
                var textarea = YAHOO.util.Dom.getElementsByClassName('gtPopupInput', 'textarea', this.features[i].popup.contentDiv)[0];
                
                init = true;
                params += this.features[i].data.type + '|' +
                    this.features[i].geometry.x + '|' +
                    this.features[i].geometry.y + '|' +
                    this.filter(textarea.value) + '|';
            }
        }
        
        if ( init == true ) {
            hash.markers = encodeURIComponent(params).replace(/%0A/g, '%0D').replace(/%0D%0D/g, '%0D');
            Giscuit.Util.setHash(hash);
        } else {
            delete hash.markers;
            Giscuit.Util.setHash(hash);
        }

        this.change();
    },
    
    filter: function(str) {
        str = str.replace('|', ' ');
        
        return str;
    },
        
    CLASS_NAME: "Giscuit.Control.Link"
});