/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Privatelayers.js 3264 2013-01-11 16:05:22Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * Class: Giscuit.Control.Privatelayers
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Control.Privatelayers = OpenLayers.Class(Giscuit.Control, {

    labelDiv: null,
    
    dt: null,
    
    ds: null,
    
    dtHeightCoef: 50,
      
    labelSubmitButton: null,
    
    labelBackButton: null,
    
    submitButton: null,
    
    labelLayerId: null,
    
    labelColorPicker: null,
        
    records: [],

    menu: null,
    
    tooltip: null,
        
    initialize: function (layers, categories, options) {
        //Deprecated in 2.12
        //this.EVENT_TYPES.push('changelayer');
        
        Giscuit.Control.prototype.initialize.apply(this, [options]);
        
        YAHOO.widget.DataTable.Formatter.lsCheckboxFormatter = this.lsCheckboxFormatter;
        
        this.records = layers;
        this.add();
        
        this.setAttributions();
        
        this.initInput();
        
        //Datatable
        this.createDatatable();
        
        //Menu
        this.menu = new YAHOO.widget.MenuBar('gtPrivatelayersCategories', {
            autosubmenudisplay: true,
            hidedelay: 500,
            keepopen: true
        });
       
        this.menu.subscribe("click", function(e, oArgs) {
            var item = oArgs[1];
            var text = item.cfg.getProperty('text');
            var path = item.value.path;
            
            YAHOO.util.Dom.get('gtPrivatelayersCategoryText').children[0].innerHTML = text;

            this.ds.sendRequest(path, {
                success : this.dt.onDataReturnInitializeTable,
                failure : this.dt.onDataReturnInitializeTable,
                scope   : this.dt,
                argument: this.dt.getState()
            });

            //FIXME:
            this.menu.hide();
            this.menu.show();
            this.menu.element.style.visibility = 'inherit';
            
            //
            this.dt.unsubscribe('cellMousedownEvent', Giscuit.Datatable.prototype.orderCellMousedownEvent);
            if(path == '/') {
                this.dt.subscribe('cellMousedownEvent', Giscuit.Datatable.prototype.orderCellMousedownEvent);
            }
                        
            this.addMetadataTooltip();
            this.styleLayerNameOnZoom();
        }, null, this); 

        var baseCategories = [
            {text: gt.translate._('allLayers'), value: {'path': '/'}},
            {text: gt.translate._('visibleLayers'), value: {'path': '__visibleLayers'}},
            {id: 'gtLayerFilter', text: gt.translate._('filter')}
        ];
        
        var allCategories = [baseCategories, categories];
        categories = [{
            id: 'gtPrivatelayersCategoryText' ,
            text: gt.translate._('allLayers'), submenu: {
                id: YAHOO.util.Dom.generateId(), itemdata:allCategories
            }
        }];
       
        this.menu.addItems(categories);
        this.menu.render();
        this.menu.element.style.visibility = 'inherit';

        //Insert filter input in menu
        var filterHtml = gt.translate._('filter') + '<input type="text" id="gtLayerFilterInput" ' +
            'style="margin: 3px 0px 2px 3px; width: 100px;">';
        YAHOO.util.Dom.get('gtLayerFilter').firstChild.innerHTML = filterHtml;
        
        var self = this;
        var filterLayers = function (evt) {
            var categoryText = YAHOO.util.Dom.get('gtPrivatelayersCategoryText').children[0];
            categoryText.innerHTML = YAHOO.util.Dom.get('gtLayerFilterInput').value;
            if(! categoryText.innerHTML) {
                categoryText.innerHTML = gt.translate._('allLayers');
            }
            
            self.ds.sendRequest(null, {
                success : self.dt.onDataReturnInitializeTable,
                failure : self.dt.onDataReturnInitializeTable,
                scope   : self.dt,
                argument: self.dt.getState()
            });

            self.dt.unsubscribe('cellMousedownEvent', Giscuit.Datatable.prototype.orderCellMousedownEvent);
            if(YAHOO.util.Dom.get("gtLayerFilterInput").value == '') {
                self.dt.subscribe('cellMousedownEvent', Giscuit.Datatable.prototype.orderCellMousedownEvent);
            }
        };
        var filterFixes = function() {
            //
            this.keyDownEvent.unsubscribe(this._onKeyDown);
            this.keyPressEvent.unsubscribe(this._onKeyPress);
                                    
            //
            var items = this.getItems();
            
            //
            for(var i=0; i < items.length; i++) {
                items[i].focus = function() {
                    this.focusEvent.fire();
                };
                items[i].blur = function() {
                    this.blurEvent.fire();
                };
            }
            
            //
            var filterSpan = document.createElement('span');
            filterSpan.className = 'yuimenuitemlabel';
            filterSpan.innerHTML = items[1].element.firstChild.innerHTML;
            items[1].element.removeChild(items[1].element.firstChild);
            items[1].element.appendChild(filterSpan);
                        
            //
               YAHOO.util.Dom.get("gtLayerFilterInput").focus();
            YAHOO.util.Event.addListener("gtLayerFilterInput", "keyup", filterLayers);
            
            this.unsubscribe("show", filterFixes);
            this.subscribe("show", function() {
                YAHOO.util.Dom.get("gtLayerFilterInput").focus();
            });
        };
        this.menu.subscribe("show", filterFixes);
        
        //
        this.menu.getItem(0).element.style.borderRight = 0;
    },  
    
    
    lsCheckboxFormatter: function (elCell, oRecord, oColumn, oData) {
        if (oData != 'none') {
            YAHOO.widget.DataTable.formatCheckbox(elCell, oRecord, oColumn, oData);
            
            if(oColumn.key == 'label') {
                
                if(! oRecord.getData('classlabel_active')) {
                
                    var image = document.createElement('img');
                    image.src = gt.config.httpImagePath + '/gear.png';
                    image.style.cursor = 'pointer';
                    image.style.marginLeft = '3px';
                    image.style.width = '12px';
                    image.style.height = '12px';
                    image.title = gt.translate._('labelSettings');
    
                    //
                    YAHOO.util.Event.addListener(image, "click", function(oArgs) {
                        gt.control.layers.createLabel(this.getData('id'));
                    }, oRecord, true);
    
                    elCell.appendChild(image);
                }
            }

        } else {
            elCell.innerHTML = '';
        }
    },

    getLabelSuccess: function (o) {
        try {
            var responseText = YAHOO.lang.JSON.parse(o.responseText);
        } catch(e) {
            Giscuit.Util.parseFailure(o, e);
            return;
        }
        
        if (responseText.replyCode != 200) {
            Giscuit.Util.invalidReplyCode(responseText);
            return;
        }
        
        //
        this.labelDiv.innerHTML = responseText.form;
        
        //
        if (responseText.showWarning) {
            var firstNFieldsDisplayed = gt.translate._('firstNFieldsDisplayed').replace("%1$s", gt.config.maxFieldsAmount);
            gt.tooltip.cfg.setProperty('text', firstNFieldsDisplayed);
            gt.tooltip.showTooltip();
        }
        
        //
        this.labelSubmitButton = new YAHOO.widget.Button('gtPrivatelayersLabelSubmit');
        this.labelSubmitButton.on("click", this.submitLabel, null, this);
        
        this.labelBackButton = new YAHOO.widget.Button('gtPrivatelayersLabelBack');
        this.labelBackButton.on("click", this.labelUpdateSuccess, null, this);

        this.labelColorPicker = new Giscuit.Color.Picker('gtPrivatelayersLabel');
        this.labelColorPicker.render();
    },

    //Populate label Panel
    createLabel: function (id) {
        this.labelLayerId = id;
        
        YAHOO.util.Dom.get('gtPrivatelayersCategories').style.display = 'none';
        YAHOO.util.Dom.get('gtPrivatelayersDt').style.display = 'none';
        
        this.labelDiv.innerHTML = this.getLoadingImage();

        var getLabelCallback = {
            success: this.getLabelSuccess,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };

        var request = YAHOO.util.Connect.asyncRequest('GET', gt.config.httpToolPath + "/privatelayers/settings/id/" + this.labelLayerId, getLabelCallback);

    },

    labelUpdateSuccess: function (o) {
        
        YAHOO.util.Dom.get('gtPrivatelayersCategories').style.display = 'block';
        YAHOO.util.Dom.get('gtPrivatelayersDt').style.display = 'block';
        
        this.labelSubmitButton.destroy();
        this.labelSubmitButton = null;
        
        this.labelBackButton.destroy();
        this.labelBackButton = null;
        
        this.labelColorPicker.destroy();
        this.labelColorPicker = null;
        
        this.labelDiv.innerHTML = '';
        this.labelLayerId = null;
        
        
        if(typeof(o.responseText) != 'undefined') {
            try {
                var responseText = YAHOO.lang.JSON.parse(o.responseText);
            } catch(e) {
                Giscuit.Util.parseFailure(o, e);
                return;
            } 

            if (responseText.replyCode != 200) {
                Giscuit.Util.invalidReplyCode(responseText);
                return;
            }
        
            gt.map.refresh();
        }

    },

    //Send label form data
    submitLabel: function () {
        var labelUpdateCallback = {
            success: this.labelUpdateSuccess,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };

        YAHOO.util.Connect.setForm('gtPrivatelayersSettingsForm');
        YAHOO.util.Connect.asyncRequest('POST', gt.config.httpToolPath + "/privatelayers/settings/id/" + this.labelLayerId, labelUpdateCallback);
    },

    //Initialize input 
    initInput: function () {
        this.labelDiv = YAHOO.util.Dom.get('gtPrivatelayersLabelContainer');
    },
    
    doBeforeCallback: function (path, raw, res) {
        var data = res.results || [];
        var filtered = [];
        var filterInput = YAHOO.util.Dom.get("gtLayerFilterInput");
        if (path && path != '__visibleLayers') {
            for (var i=0; i < data.length; i++) {
                if (data[i].path.indexOf(path) === 0) {
                    filtered.push(data[i]);
                }
            }
            res.results = filtered;
            filterInput.value = '';
            
        } else if (path && path == '__visibleLayers') {
            for (var i=0; i < data.length; i++) {
                if (data[i].view) {
                    filtered.push(data[i]);
                }
            }
            res.results = filtered;
            filterInput.value = '';
        } else if (filterInput != null && filterInput.value != '') {
            var valueArr = filterInput.value.split(' ');
            var regexStr = '';
            for (var i = 0; i < valueArr.length; i++) {
                regexStr += valueArr[i] + '.*';
            }
            var regex = new RegExp(regexStr, "i");

            for (var i=0; i < data.length; i++) {
                if (data[i].name.match(regex) != null) {
                    filtered.push(data[i]);
                }
            }
            res.results = filtered;
        }
        
        this.initInput();

        //Order the array by order column
        function byOrder(a, b) {
            var x = a.order;
            var y = b.order;
            
            return ((x < y) ? -1 : ((x > y) ? 1 : 0));
        }
        res.results.sort(byOrder);
        
        return res;
    },

    //Populate datatable
    createDatatable: function () {
        //Define columns
        this.columnDefs = [{
            key: "name",
            resizeable: true,
            label: "",
            width: 175
        },
        {
            key: "view",
            formatter: "lsCheckboxFormatter",
            label: gt.map.getControlIcon(23, gt.translate._('view')),
            width: 15
        },
        {
            key: "select",
            formatter: "lsCheckboxFormatter",
            label: gt.map.getControlIcon(10, gt.translate._('information')),
            width: 15
        },
        {
            key: "label",
            formatter: "lsCheckboxFormatter",
            label: gt.map.getControlIcon(11, gt.translate._('label')),
            width: 35
        },
        {
            key: "edit",
            formatter: "lsCheckboxFormatter",
            label: gt.map.getControlIcon(6, gt.translate._('edit')),
            width: 15
        }];

        //Create data source
        this.ds = new YAHOO.util.DataSource(this.records);
        this.ds.responseType = YAHOO.util.DataSource.TYPE_JSARRAY;
                
        this.ds.responseSchema = {
            resultsList: "records",
            fields: ["id", "name", "view", "select", "label", "edit", "class", "path", "order", "js_layer", "metadata", "minscaledenom", "maxscaledenom", "js_isbase", "classlabel_active", "sendRequest"]
        };
        
        this.ds.doBeforeCallback = OpenLayers.Function.bind(this.doBeforeCallback, this);

        var dtOptions = {
            width: '99.5%',
            MSG_EMPTY: gt.translate._('emptyResult'),
            MSG_LOADING: gt.translate._('loading') + "..."
        };

        this.dt = new YAHOO.widget.ScrollingDataTable("gtPrivatelayersDt", this.columnDefs, this.ds, dtOptions);
        
        //
        gt.accord.on('afterPanelOpen', function(args) {
            if(this.getPanelIndex(args.panel) == 0 && this.dt) {
                this.setOptimalDatatableHeight(this.dt, this.dtHeightCoef);
            }
        }, null, this);
                
        //Order
        this.dt.orderTargets = {};
        this.dt.orderIsDragging = false;
        
        this.dt.orderDragEndEvent = new YAHOO.util.CustomEvent("orderDragEndEvent", this);
        var orderEnd = function(type, oArgs) {
            var orderArr = [];
            
            this.dt.disable();
            
            var records = this.dt.getRecordSet().getRecords();
            for(a=0; a < records.length; a++) {
                var id = records[a].getData('id');
                orderArr.push(id);
                
                //Update order in this.records
                for(var b=0; b < this.records.length; b++) {
                    if(id == this.records[b].id) {
                        this.records[b].order = a + 1;
                    }
                }
            }
            
            //
            this.addMetadataTooltip();
            this.styleLayerNameOnZoom();
            
            orderArr = orderArr.join(',');
            
            var orderCallback = {
                success: function(o) {
                    this.dt.undisable();
                    
                    try {
                        var responseText = YAHOO.lang.JSON.parse(o.responseText);
                    } catch(e) {
                        Giscuit.Util.parseFailure(o, e);
                    }
                    
                    if (responseText.replyCode != 200) {
                        Giscuit.Util.invalidReplyCode(responseText);
                        return;
                    }
                    
                    gt.map.refresh();
                },
                scope: this,
                timeout: this.callbackTimeout,
                failure: this.callbackFailure
            };
            
            var request = YAHOO.util.Connect.asyncRequest('GET', gt.config.httpToolPath + "/privatelayers/order/ids/" + orderArr, orderCallback);
        };
        this.dt.orderDragEndEvent.subscribe(orderEnd);
        
        this.dt.subscribe('cellMousedownEvent', Giscuit.Datatable.prototype.orderCellMousedownEvent);
        this.dt.subscribe('initEvent', Giscuit.Datatable.prototype.orderInitEvent);
        this.dt.subscribe('rowAddEvent', Giscuit.Datatable.prototype.orderRowAddEvent);
                   
        //Hide edit checkbox
        var hideEdit = true;
        for(var i=0; i < this.records.length; i++) {
            if(this.records[i].edit != 'none') {
                hideEdit = false;
            }
        }
        if(hideEdit) {
            this.dt.hideColumn('edit');
        }

        //
        this.addMetadataTooltip();
        
        //
        gt.map.events.register("zoomend", this, this.styleLayerNameOnZoom);
        
        //Checkbox click
        this.dt.subscribe('checkboxClickEvent', function (oArgs) {

            var elCheckbox = oArgs.target;
            var newValue = elCheckbox.checked;
            var record = this.dt.getRecord(elCheckbox);
            var column = this.dt.getColumn(elCheckbox);
            var recordIndex = this.dt.getRecordIndex(record);
            var recordKey = record.getData('id');
            
            //Debug
            var debugMsg = 'Layer ' + record.getData('class') + ' ';
            if (column.key == 'view') {
                if(newValue == true) {
                    debugMsg += 'has visibility enabled';
                } else {
                    debugMsg += 'has visibility disabled';
                }
                
                
            } else if (column.key == 'select') {
                if(newValue == true) {
                    debugMsg += 'has information enabled';
                } else {
                    debugMsg += 'has information disabled';
                }
            } else if (column.key == 'label') {
                if(newValue == true) {
                    debugMsg += 'has labels enabled';
                } else {
                    debugMsg += 'has labels disabled';
                }
            } else if (column.key == 'edit') {
                if(newValue == true) {
                    debugMsg += 'has editing enabled';
                } else {
                    debugMsg += 'has editing disabled';
                }
            }
            debug.info(debugMsg);
            
            //Make checkbox function as a radio button
            if (column.key == 'edit') {
                var current = 0;
                var recs = this.dt.getRecordSet().getRecords();

                while (current < recs.length) {
                    if (recs[current].getData('edit') != 'none' && current != recordIndex) {
                        this.dt.updateCell(recs[current], column, false);
                    }
                    current++;
                }
            }

            //Disable checkbox
            elCheckbox.disabled = true;

            //Enable checkbox
            updateSuccess = function (o) {
                elCheckbox.disabled = false;
                
                try {
                    var responseText = YAHOO.lang.JSON.parse(o.responseText);
                } catch(e) {
                    Giscuit.Util.parseFailure(o, e);
                    /*
                    //FIXME: Opera bug
                    if (YAHOO.env.ua.opera &&
                        e.message == "'OpenLayers.Layer.prototype.redraw.apply' is not a function") {
    
                        window.location.reload();
                    } else {
                        debug.group(e.message);
                        debug.warn(o.responseText);
                        debug.groupEnd();
                    }
                    */
                }
                
                var refreshEditgeom = false;

                if (responseText.replyCode != 200) {
                    Giscuit.Util.invalidReplyCode(responseText);
                    return;
                }

                var mapChanged = false;
                
                
                if (column.key == 'view') {
                    //Activate deactivate javascript layer visiblity
                    if(record.getData('js_layer')) {
                        var layer = gt.layer[record.getData('class')];
                        if(typeof(layer) != 'undefined') {
                            layer.setVisibility(newValue);
                        }
                    }
                    mapChanged = true;
                } else if (column.key == 'select') {
                    //Activate deactivate javascript layer snap
                    if(record.getData('js_layer')) {
                        var layer = gt.layer[record.getData('class')];
                        
                        if(typeof(layer) != 'undefined' && typeof(gt.snap.vector) != 'undefined') {
                            var newTargets = [];
                            for(var i=0; i < gt.snap.vector.targets.length; i++) {
                                var target = gt.snap.vector.targets[i];
                                
                                if(target.layer != layer) {
                                    newTargets.push(target);
                                }
                            }
                            
                            gt.snap.vector.targets = newTargets;
                            gt.snap.edit.targets = newTargets;
                            if (newValue) {
                                var defaults = gt.snap.vector.defaults;
                                var newTarget = {
                                    'edge': defaults.edge,
                                    'edgeTolerance': defaults.tolerance,
                                    'node': defaults.node,
                                    'nodeTolerance': defaults.tolerance,
                                    'tolerance': defaults.tolerance,
                                    'vertex': defaults.vertex,
                                    'vertexTolerance': defaults.tolerance,
                                    'layer': layer
                                };
                                                                
                                if(typeof(layer.features) != 'undefined') {
                                    gt.snap.vector.targets.push(newTarget);
                                    gt.snap.edit.targets.push(newTarget);
                                }
                            }
                        }
                    }
                    
                } else if (column.key == 'label') {
                    mapChanged = true;
                } else if (column.key == 'edit') {
                    mapChanged = true;
                    
                    //Reset edit records
                    for(var i=0; i < this.records.length; i++) {
                        this.records[i]['edit'] = false;
                    }
                }
                
                //Update data source
                for(var i=0; i < this.records.length; i++) {
                    if(this.records[i].id == recordKey) {
                        this.records[i][column.key] = newValue;
                    }
                }
                
                //Update data table
                record.setData(column.key, newValue);
                
                //
                if (column.key == 'view') {
                    this.styleLayerNameOnZoom();
                }
                
                //Set attributions
                this.setAttributions();
                
                if(mapChanged == true) {
                    gt.map.refresh();
                }
                
                //Edit
                if (column.key == 'edit') {
                    if (newValue == true) {
                        if(gt.control.editgeom.active == true) {
                            gt.control.editgeom.deactivate();
                            gt.control.editgeom.activate();
                        }
                    } else {
                        if(gt.control.editgeom.active == true) {
                            gt.control.editgeom.deactivate();
                        }
                    }
                }
                
                //
                if (column.key == 'view') {
                    this.events.triggerEvent('changelayer');
                }

            };
            
            var sendRequest = true;
            if(record.getData('sendRequest') === false) {
                sendRequest = false;
            }

            //Send/don't send request to server
            if (! sendRequest) {
                //Create response object
                var responseObj = {
                    replyCode: 200
                };

                //Send response
                updateSuccess({responseText: YAHOO.lang.JSON.stringify(responseObj)});
            } else {
                //Update checkbox callback
                var updateCallback = {
                    success: updateSuccess,
                    timeout: this.callbackTimeout,
                    failure: this.callbackFailure,
                    scope: this
                };
    
                //Send request
                var request = YAHOO.util.Connect.asyncRequest('GET', gt.config.httpToolPath + "/privatelayers/edit/id/" + recordKey + "/column/" + column.key + "/value/" + newValue, updateCallback);

            }
        },
        null, this);

    },

    add: function () {
        var snapLayers = [];
        
        //Order the array by order column
        function byOrder(a, b) {
            var x = a.order;
            var y = b.order;
            
            return ((x < y) ? -1 : ((x > y) ? 1 : 0));
        }
        this.records = this.records.sort(byOrder).reverse();
        
        //Add javascript layers
        var overlays = [];
        for(var i=0; i < this.records.length; i++) {
            if(this.records[i]['js_layer'] == true) {
                var layer = this.records[i]['class'];
                
                gt.layer[layer] = new Giscuit.Layer[layer]();
                gt.layer[layer].order = this.records[i]['order'];
                
                if(this.records[i]['js_isbase']) {
                    gt.map.addLayer(gt.layer[layer]);
                    
                } else {
                    overlays.push(gt.layer[layer]);
                }
                
                gt.layer[layer].setVisibility(this.records[i]['view']);
                
                //
                if(this.records[i]['select'] === true && 
                    typeof(gt.layer[layer].features) != 'undefined') {
                    
                    snapLayers.push(gt.layer[layer]);
                }
            }
        }
        
        //Main
        gt.layer.main = new OpenLayers.Layer.MapServer("main", gt.config.httpPath + "/draw", {
            layers: "basic"
        }, {
            transitionEffect: "resize",
            ratio: 1.1,
            singleTile: true
        });
        gt.map.addLayer(gt.layer.main);
        
        //
        gt.map.addLayers(overlays);
        
        //Edit
        gt.map.addEditLayer();
        
        //Vector
        gt.map.addVectorLayer();
        
        //Snap
        if (gt.config.snap == 't') {
            snapLayers.push(gt.layer.vector);
            snapLayers.push(gt.layer.edit);
            gt.map.addSnapLayer(snapLayers, true);
        }
    },
    
    addLayer: function (layer, record, type) {
        var maxId = 0;
        for (var i = 0; i < this.records.length; i++) {
            if (this.records[i].id > maxId) {
                maxId = this.records[i].id;
            }
        }
        maxId++;
        
        record.id = maxId;
        record.layerClass = layer.name;
        record.js_layer = true;
        record.sendRequest = false;
        record['class'] = layer.name;
        record.edit = 'none';
        record.label = 'none';
        
        if (typeof(record.metadata) == 'undefined') {
            record.metadata = {};
        }
        
        if (layer.visibility) {
            record.view = true;
        }
        
        this.records.push(record);
        gt.map.addLayer(layer);
        gt.layer[record.layerClass] = layer;
        
        if (type == 'base') {
            gt.map.setLayerIndex(layer, 0);
        }
        
        this.ds.sendRequest('', {
            success : this.dt.onDataReturnInitializeTable,
            failure : this.dt.onDataReturnInitializeTable,
            scope   : this.dt,
            argument: this.dt.getState()
        });
    },
    
    removeLayer: function(className) {
        
        for (var key in gt.layer) {
            if (key == className) {
                var layer = gt.layer[className];
                
                //Delete from gt.layer and map
                delete gt.layer[className];
                gt.map.removeLayer(layer);
                
                //Delete from records
                for (var i in this.records) {
                    if (this.records[i].layerClass == className) {
                        this.records.splice(i, 1);
                        break;
                    }
                }

                //Delete from datatable
                var records = this.dt.getRecordSet().getRecords();
                for(var i = 0; i < records.length; i++) {
                    if (records[i].getData('class') == className) {
                        this.dt.deleteRow(records[i]);
                        break;
                    }
                }
                
                return;
            }
        }
    },
    
    setAttributions: function() {
        var div = YAHOO.util.Dom.get('gtAttribution');
        var attributions = [gt.config.copyRight];
        
        for (var i=0; i < this.records.length; i++) { 
            var record = this.records[i];
            
            if(! record.metadata.ows_attribution_title) {
                continue;
            } 
            
            var attribution = record.metadata.ows_attribution_title;
            if(record.metadata.ows_attribution_onlineresource) {
                attribution = "<a target='_blank' href='" + record.metadata.ows_attribution_onlineresource +  "'>" + attribution + "</a>";
            }
            
            if(record.view == true && attribution 
                && ! Giscuit.Util.inArray(attributions, attribution)) {
                
                attributions.push(attribution);
            }
        }
        
        div.innerHTML = attributions.join(', ');
    },
    
    getEditLayer: function() {
        for(var i = 0; i < this.records.length; i++) {
            if(this.records[i]['edit'] == true && this.records[i]['view'] == true) {
                return this.records[i];
            }
        }
        
        return false;
    },
    
    setSettings: function(settingsName, layerId, action) {
        //
        var pos = {
            'view': 1,
            'select': 2,
            'label': 3,
            'edit': 4
        };
        if(! pos[settingsName]) {
            return false;
        }
        
        //
        var record = null;
        var records = this.dt.getRecordSet().getRecords();
        for(i=0; i < records.length; i++) {
            if(records[i].getData('id') == layerId) {
                record = records[i];
                break;
            }
        }
        if(! record) {
            return false;
        }
        
        //
        var checkbox = YAHOO.util.Dom.get(record._sId).children[pos[settingsName]].firstChild.firstChild;
        if(! checkbox) {
            return false;
        }
        
        //
        if(checkbox.checked != action) {
            if (! (YAHOO.env.ua.opera || YAHOO.env.ua.ie >= 9)) {
                checkbox.checked = action;
            }
            
            Giscuit.Util.fireEvent(checkbox, 'click');
        }
        
        return true;
    },
    
    getSettings: function(settingsName, layerId) {
        var records = this.dt.getRecordSet().getRecords();
        for(i=0; i < records.length; i++) {
            if(records[i].getData('id') == layerId) {
                return records[i].getData(settingsName);
            }
        }
        
        return undefined;
    },
    
    getLayerIdByClassName: function(val) {
        var records = this.dt.getRecordSet().getRecords();
        for(i=0; i < records.length; i++) {
            if(records[i].getData('class') == val) {
                return records[i].getData('id');
            }
        }
        
        return false;
    },
    
    addMetadataTooltip: function() {
        //Collect datatable elements for tooltip initialization
        var rows = this.dt.getTbodyEl().rows;
        var metadataEls = [];
        for (var i = 0; i < rows.length; i++) {
            metadataEls.push(rows[i].children[0]);
        }
        metadataEls.push('gtMetadataTooltip');
        
        //Create tooltip for datatable
        if(this.tooltip != null) {
            this.tooltip.destroy();
        }
        this.tooltip = new YAHOO.widget.Tooltip("gtMetadataTooltip", { 
            context: metadataEls,
            hidedelay: 500 ,
            showdelay: 1500 ,
            autodismissdelay: 1000 * 60
        });

        //Set the tooltip on top of the datatable
        this.tooltip.bringToTop();
        
        this.tooltip.doShow = Giscuit.Util.tooltipDoShow;
        
        //Set text for the tooltip before display
        this.tooltip.contextTriggerEvent.subscribe(function(type, args, dt) {
            var el = dt.getTdEl(args[0]);
            var data = dt.getRecord(el).getData();
            var metadata = data.metadata;
                        
            var text = '<ul>' + '<b>' + metadata.ows_title + '</b>';
            
            //Scale
            if(data.maxscaledenom) {
                text += '<li>' + gt.translate._('maxscaledenom') + ': 1:' + data.maxscaledenom + '</li>';
            }
            if(data.minscaledenom) {
                text += '<li>' + gt.translate._('minscaledenom') + ': 1:' + data.minscaledenom + '</li>';
            }
            
            for (var key in metadata) {
                if (metadata.hasOwnProperty(key) && key != 'ows_title') {
                    text += '<li>' + gt.translate._(key) + ': ' + metadata[key] + '</li>';
                }
            }
            text += '</ul>';
            this.cfg.setProperty("text", text);
        }, this.dt);

    },
    
    styleLayerNameOnZoom: function() {
        var scale = gt.map.getScale();
        
        var rows = this.dt.getTbodyEl().rows;
        for (var i = 0; i < rows.length; i++) {
            var data = this.dt.getRecord(rows[i]).getData();
            var label = rows[i].children[0].children[0];

            if(data.minscaledenom || data.maxscaledenom) { 
                label.style.color = '#000000';
            } 
                 
            if(data.view == true &&
                (data.minscaledenom && data.minscaledenom > scale ||
                data.maxscaledenom && data.maxscaledenom < scale)) {
                        
                label.style.color = '#DDDDDD';
            } 
            
        }
    },
            
    CLASS_NAME: "Giscuit.Control.Privatelayers"

});
