/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 *
 *
 * @version    $Id: Control.js 1132 2011-03-11 09:34:36Z ue $
 * @copyright  Copyright (c) 2012 VEC
 *
 */
Giscuit.Map.prototype.gtEventListeners.push({
    'draw': function() {
        gt.control.routing = new Giscuit.Control.Routing();
    }
});

/**
 * Class: Giscuit.Control.Routing
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Control.Routing = OpenLayers.Class(Giscuit.Control, {

    panel: null,
    
    button: null,
    
    dt: null,
    
    addButton: null,
    
    getButton: null,
    
    layersButton: null,
    
    algorithmsButton: null,
    
    costsButton: null,
    
    layers: [],
    
    selectedLayer: null,
    
    selectedAlgorithm: null,
    
    selectedCost: null,
    
    layerSourceDs: null,
    
    oAc: {},

    pathLines: [],
    
    pathLineStyle: null,

    pathPoints: [],
    
    pathPointStyle: null,
    
    clickControl: null,

    initialize: function (options) {
        Giscuit.Control.prototype.initialize.apply(this, [options]);
        
        var self = this;
        
        this.pathLineStyle = {
            'strokeWidth': 6,
            'strokeColor': '#007FFF',
            'strokeOpacity': 0.5
        };
        
        var pathPointRadius = 20;
        this.pathPointStyle = {
            'pointRadius': pathPointRadius,
            'externalGraphic': gt.config.httpImagePath + '/marker/blank.png',
            'graphicXOffset': pathPointRadius * -1,
            'graphicYOffset': pathPointRadius * -2,
            
            'labelYOffset': pathPointRadius * 2 - 12,
            'fontFamily': 'sans-serif',
            'fontSize': '10pt',
            'fontColor': '#000000',
            'fontWeight': 'bold'
        };
        
        this.button = new YAHOO.widget.Button({ 
            label: gt.map.getControlIcon(30), 
            id: "gtRoutingControl", 
            onclick: {fn: function() {
                if(self.active) {
                    self.deactivate();
                } else {
                    self.activate();
                }
            }},
            title: gt.translate._('routing'), 
            container: "gtMapControls",
            type: 'checkbox'
        });
    },
    
    activate: function () {
        //Deactivate all active controls
        gt.map.deactivateAll();

        var activated = Giscuit.Control.prototype.activate.call(this);
        
        if (activated == true) {
            //Check button
            this.button.set('checked', true);
            
            if (this.panel == null) {
                var html = this.getLoadingImage();
                
                gt.accord.addPanel({
                    label: gt.translate._('routing'),
                    content: html
                });
                this.panel = gt.accord.getPanel(gt.accord.getPanels().length - 1);
                this.addHelpButton('http://docs.giscuit.com/En/tutorial_refguide.user.desktop.routing.pkg.html');
                
                gt.accord.openPanel(this.getPanelIndex());
                
                YAHOO.util.Connect.asyncRequest('POST', gt.config.httpToolPath + '/routing/index', {
                    success: function (o) {
                        try {
                            var responseText = YAHOO.lang.JSON.parse(o.responseText);
                        } catch(e) {
                            Giscuit.Util.parseFailure(o, e);
                            return;
                        }
                        
                        if (responseText.replyCode != 200) {
                            Giscuit.Util.invalidReplyCode(responseText);
                            return;
                        }
                        
                        this.layers = responseText.layers;
                        this.create();
                        
                    },
                    failure: Giscuit.Util.callbackFailure,
                    scope: this
                });
            } else {
                //Select first row
                if(this.dt) {
                    this.dt.unselectAllRows();
                    this.dt.selectRow(this.dt.getTrEl(0));
                }
                
                //
                gt.layer.vector.addFeatures(this.pathPoints);
                
                gt.accord.openPanel(this.getPanelIndex());
                
                if(this.clickControl) {
                    this.clickControl.activate();
                }
            }
        } else {
            gt.accord.openPanel(this.getPanelIndex());
        }

        return activated;
    },

    deactivate: function () {

        var deactivated = Giscuit.Control.prototype.deactivate.call(this);
        if (deactivated == true) {
            
            //Destroy control
            if(this.clickControl && this.clickControl.active) {
                this.clickControl.deactivate();
            }
            
            gt.layer.vector.removeFeatures(this.pathLines);
            gt.layer.vector.removeFeatures(this.pathPoints);
            
            //Uncheck button
            this.button.set('checked', false);
        }
        
        return deactivated;
    },
    
    create: function() {
        //
        if(this.layers.length == 0) {
            gt.accord.setBody(this.getPanelIndex(), '<p>' + gt.translate._('noLayersAvailableForRouting') + '</p>');
            return;
        }
        
        //Click control
        this.clickControl = new Giscuit.Control.Click({
            'clickCallback': Giscuit.Util.bind(this.trigger, this)
        });
        gt.map.addControl(this.clickControl);
        this.clickControl.activate();
        
        //
        var selectedCostId = null,
            selectedAlgorithmId = null,
            selectedLayerId = null;
        
        //Selected layer
        if(selectedLayerId) {
            for(var i=0; i < this.layers.length; i++) {
                if(this.layers[i].id == selectedLayerId) {
                    this.selectedLayer = this.layers[i];
                    break;
                }
            }
        } else {
            this.selectedLayer = this.layers[0];
        }
        
        //Selected algorithm
        for (var i in this.selectedLayer.algorithms) {
            if(selectedAlgorithmId) {
                if(i == selectedAlgorithmId) {
                    this.selectedAlgorithm = i;
                    break;
                }
            } else {
                this.selectedAlgorithm = i;
                break;
            }
        }
        
        //Selected cost
        for (var i in this.selectedLayer.costs) {
            if(selectedCostId) {
                if(i == selectedCostId) {
                    this.selectedCost = i;
                    break;
                }
            } else {
                this.selectedCost = i;
                break;
            }
        }
        
        //Layers options html
        var html = '';
        html += '<input type="button" id="gtRoutingLayersButton">';
        html += '<select id="gtRoutingLayersSelect"></select>';

        //Algorithms options html
        html += '<input type="button" id="gtRoutingAlgorithms">';
        html += '<select id="gtRoutingAlgorithmsSelect"></select>';
        
        //Costs options html
        html += '<input type="button" id="gtRoutingCosts">';
        html += '<select id="gtRoutingCostsSelect"></select>';
        
        //
        html += '<p><div id="gtRoutingDatatable"></div></p>';
        html += '<button type="button" id="gtRoutingAddButton">' + gt.translate._('addDestination') + '</button>';
        html += '<button type="button" id="gtRoutingGetButton">' + gt.translate._('getDirections') + '</button>';
        html += '<div id="gtRoutingAcPH"></div>';
        
        gt.accord.setBody(this.getPanelIndex(), html);
        
        //Layers
        this.layersButton = new YAHOO.widget.Button("gtRoutingLayersButton", {  
            type: "menu",  
            title: gt.translate._('layers'),
            menu: "gtRoutingLayersSelect",
            lazyloadmenu: false
        }); 
        YAHOO.util.Dom.addClass(this.layersButton.get('element'), 'gt-fixed-width-menu-btn'); 
        
        var layersData = [];
        for(var i=0; i < this.layers.length; i++) {
            layersData.push({
                'value': this.layers[i].id,
                'text': this.layers[i].name
            });
        }
        Giscuit.Util.populateYuiMenu(this.layersButton, layersData, this.selectedLayer.id);
        this.layersButton.set(
            "label",
            '<em style="width:80px;">' + this.layersButton.getMenu().activeItem.cfg.getProperty('text') + '</em>'
        );
        
        if(this.layersButton.getMenu().getItems().length <= 1) {
            this.layersButton.get('element').style.display = 'none';
        } else {
            this.layersButton.get('element').style.display = '';
        }
        
        this.layersButton.getMenu().subscribe("click", function (p_sType, p_aArgs) {
            if (! p_aArgs[1]) {
                return;
            }
            
            //Layers
            this.layersButton.set("label", '<em style="width:80px;">' + p_aArgs[1].cfg.getProperty("text") + '</em>');
            for (var i = 0; i < this.layers.length; i++) {
                if (this.layers[i].id == p_aArgs[1].value) {
                    this.selectedLayer = this.layers[i];
                }
            }
           
            //Algorithms
            var algorithmsData = [];
            for(i in this.selectedLayer.algorithms) {
                algorithmsData.push({
                    'value': i,
                    'text': this.selectedLayer.algorithms[i]
                });
            }
            Giscuit.Util.populateYuiMenu(this.algorithmsButton, algorithmsData);
           
            if(this.algorithmsButton.getMenu().getItems().length <= 1) {
                this.algorithmsButton.get('element').style.display = 'none';
            } else {
                this.algorithmsButton.get('element').style.display = '';
            }
           
            //Costs
            var costData = [];
            for(i in this.selectedLayer.costs) {
                costData.push({
                    'value': i,
                    'text': this.selectedLayer.costs[i]
                });
            }
            Giscuit.Util.populateYuiMenu(this.costsButton, costData);
           
            if(this.costsButton.getMenu().getItems().length <= 1) {
                this.costsButton.get('element').style.display = 'none';
            } else {
                this.costsButton.get('element').style.display = '';
            }
               
            //Update autocomplete ds
            this.layerSourceDs.liveData = gt.config.httpToolPath + "/geocode/index/centroid/t" + 
                this.getExtentParam() + '/source/' + this.selectedLayer.source;
                                
            //Refresh datatable
            this.dt.getDataSource().sendRequest('', {
                success: this.dt.onDataReturnInitializeTable,
                scope: this.dt,
                argument: this.dt.getState()
            });
            
            //
            this.getButton.set('disabled', true);
            
            //Select first row
            this.dt.selectRow(this.dt.getTrEl(0)); 
            
            //Clean up
               gt.layer.vector.destroyFeatures(this.pathLines);
               this.pathLines.length = 0;
            
               gt.layer.vector.destroyFeatures(this.pathPoints);
               this.pathPoints.length = 0;
            
        }, null, this);
        
        
        //Algorithms
        this.algorithmsButton = new YAHOO.widget.Button("gtRoutingAlgorithms", {  
            type: "menu",  
            title: gt.translate._('algorithms'),
            menu: "gtRoutingAlgorithmsSelect",
            lazyloadmenu: false
        });
        YAHOO.util.Dom.addClass(this.algorithmsButton.get('element'), 'gt-fixed-width-menu-btn'); 
        
        var algorithmsData = [];
        for(i in this.selectedLayer.algorithms) {
            algorithmsData.push({
                'value': i,
                'text': this.selectedLayer.algorithms[i]
            });
        }
        Giscuit.Util.populateYuiMenu(this.algorithmsButton, algorithmsData, selectedAlgorithmId);
        this.algorithmsButton.set(
            "label",
            '<em style="width:80px;">' + this.algorithmsButton.getMenu().activeItem.cfg.getProperty('text') + '</em>'
        );
        
        if(this.algorithmsButton.getMenu().getItems().length <= 1) {
            this.algorithmsButton.get('element').style.display = 'none';
        } else {
            this.algorithmsButton.get('element').style.display = '';
        }
        
        this.algorithmsButton.getMenu().subscribe("click", function (p_sType, p_aArgs) {
            if (! p_aArgs[1]) {
                return;
            }
            this.algorithmsButton.set("label", '<em style="width:80px;">' + p_aArgs[1].cfg.getProperty("text") + '</em>');
            this.selectedAlgorithm = p_aArgs[1].value;
        }, null, this);
        
        //Costs
        this.costsButton = new YAHOO.widget.Button("gtRoutingCosts", {  
            type: "menu",  
            title: gt.translate._('costs'),
            menu: "gtRoutingCostsSelect",
            lazyloadmenu: false
        });
        YAHOO.util.Dom.addClass(this.costsButton.get('element'), 'gt-fixed-width-menu-btn');
        
        var costsData = [];
        for(i in this.selectedLayer.costs) {
            costsData.push({
                'value': i,
                'text': this.selectedLayer.costs[i]
            });
        }
        Giscuit.Util.populateYuiMenu(this.costsButton, costsData, selectedCostId);
        if(typeof(this.costsButton.getMenu().activeItem) != 'undefined') {
            this.costsButton.set(
                "label",
                '<em style="width:80px;">' + this.costsButton.getMenu().activeItem.cfg.getProperty('text') + '</em>'
            );
        }
        
        if(this.costsButton.getMenu().getItems().length <= 1) {
            this.costsButton.get('element').style.display = 'none';
        } else {
            this.costsButton.get('element').style.display = '';
        }
        
        this.costsButton.getMenu().subscribe("click", function (p_sType, p_aArgs) {
            if (! p_aArgs[1]) {
                return;
            }
            
            this.costsButton.set("label", '<em style="width:80px;">' + p_aArgs[1].cfg.getProperty("text") + '</em>');
            this.selectedCost = p_aArgs[1].value;
        }, null, this);

        //Autocomplete datasource
        var layerSourceDsParams = this.getExtentParam() + '/source/' + this.selectedLayer.source;
        this.layerSourceDs = new YAHOO.util.DataSource(gt.config.httpToolPath + "/geocode/index/centroid/t" + layerSourceDsParams);
        this.layerSourceDs.responseType = YAHOO.util.DataSource.TYPE_JSON;
        this.layerSourceDs.responseSchema = {
            resultsList: "records",
            fields: [
                 {key: "text"},
                 {key: "viewport"},
                 {key: "the_geom"}
            ]
        };
        
        //Initialize dt params
        var ds = new YAHOO.util.DataSource([{id: new Date().getTime(), letter: 0, text: '', the_geom: ''},
            {id: new Date().getTime() + 1, letter: 1, text: '', the_geom: ''}]);
        ds.responseType = YAHOO.util.DataSource.TYPE_JSARRAY;
        ds.responseSchema = {
            fields: ["id", "letter", "text", "the_geom"]
        };

        var columnDefs = [ 
            {key: "letter", resizeable: true, width: 10, formatter: this.formatLetter, label: gt.translate._('point')} ,
            {key: "text", resizeable: true, width: this.getTextColumnWidth(), formatter: Giscuit.Util.bind(this.formatTextbox, this), label: gt.translate._('address')} ,
            {key: "remove", resizeable: true, width: 15, formatter: Giscuit.Util.bind(this.formatRemove, this), label: gt.translate._('remove')} 
        ];
        
        //Create dt
        this.dt = new YAHOO.widget.DataTable("gtRoutingDatatable", columnDefs, ds, {
            scrollable: true,
            selectionMode: "single",
            width: '99.5%',
            MSG_EMPTY: gt.translate._('emptyResult'),
            MSG_LOADING: gt.translate._('loading') + "..."
        });
        
        //Hide X overflow
        YAHOO.util.Dom.getElementsByClassName('yui-dt-bd', 'div', 'gtRoutingDatatable')[0].style.overflowX = 'hidden';
        
        //Hide dt header
        YAHOO.util.Dom.getElementsByClassName('yui-dt-hd', 'div', 'gtRoutingDatatable')[0].style.display = 'none';
                
        //Select column and show autocomplete editor
        this.dt.subscribe("cellClickEvent", this.dt.onEventShowCellEditor);
        this.dt.subscribe("rowClickEvent", this.dt.onEventSelectRow);
        
        //Select first row
        this.dt.selectRow(this.dt.getTrEl(0)); 

        //Resize text column on panel resize
        gt.init.layout.getUnitByPosition('left').on('widthChange', function(args) {
            this.dt.setColumnWidth(this.dt.getColumn('text'), this.getTextColumnWidth());

            for (var i in this.oAc) {
                this.oAc[i].getInputEl().style.width = this.getTextColumnWidth() - 5 + 'px';
            }
        }, null, this);
        
        //Add destination
        this.addButton = new YAHOO.widget.Button('gtRoutingAddButton');
        this.addButton.on("click", function (evt) {
            
            if(this.dt.getRecordSet().getLength() > 25) {
                return;
            }
            
            var newRow = {
                id: new Date().getTime(),
                letter: this.dt.getRecordSet().getLength(),
                text: '', 
                the_geom: ''
            }
            this.dt.addRow(newRow);
        }, null, this);
        
        //Get route based on destinations
        this.getButton = new YAHOO.widget.Button('gtRoutingGetButton');
        this.getButton.on("click", function (evt) {
            if(! this.active) {
                this.activate();
            }
            
            this.getButton.set('disabled', true);
                  
            var postDataObj = {};
            postDataObj.layer = this.selectedLayer.id;
            postDataObj.algorithm = this.selectedAlgorithm;
            
            if(this.selectedCost) {
                postDataObj.cost = this.selectedCost;
            }
             
            var records = this.dt.getRecordSet().getRecords();
              
            //Path points
            var points = [];
            for (var i = 0; i < records.length; i++) {
                var point = records[i].getData().the_geom;
                if (point.length > 0) {
                    
                    //Geometry
                    point = point.substring(6, point.length);
                    point = point.substring(0, point.length - 1);
                    point = point.split(' ');
                    
                    //Name
                    point.push(records[i].getData().text);
                    
                    //
                    points.push(point);
                }
            }
                        
            postDataObj.points = points;
            
            YAHOO.util.Connect.asyncRequest('POST', gt.config.httpToolPath + '/routing/getdirections', {
                success: function (o) {
                    this.getButton.set('disabled', false);
                    
                    try {
                        var responseText = YAHOO.lang.JSON.parse(o.responseText);
                    } catch(e) {
                        Giscuit.Util.parseFailure(o, e);
                        return;
                    }
                    
                    //
                    if (responseText.replyCode != 200) {
                        Giscuit.Util.invalidReplyCode(responseText);
                        
                        return;
                    }
                    
                    this.setPathLine(responseText.paths);
                    
                },
                failure: function (o) {
                    Giscuit.Util.callbackFailure(o);
                },
                scope: this
            }, 'data=' + YAHOO.lang.JSON.stringify(postDataObj));
            
        }, null, this);
        
        this.getButton.set('disabled', true);
    },
    
    getExtentParam: function () {
        var wgsProj = new OpenLayers.Projection("EPSG:4326");
        var extent = gt.map.maxExtent.clone();
        var extentParam = '';
        if (this.selectedLayer.source == 'google') {
            extent.transform(gt.map.getProjectionObject(), wgsProj);
            extentParam = "/extent/" + extent.toBBOX();
        }
        
        return extentParam;
    },
    
    setPathLine: function(records) {
        if (this.pathLines.length != 0) {
            gt.layer.vector.destroyFeatures(this.pathLines);
            this.pathLines.length = 0;
        }
        
        for(var a in records) {
               var geometry = OpenLayers.Geometry.fromWKT(records[a].path);
                
               var feature = new OpenLayers.Feature.Vector(geometry);
               feature.style = this.pathLineStyle;
               
               this.pathLines.push(feature);
        }
        
        if (this.pathLines.length == 0) {
            gt.tooltip.cfg.setProperty('text', gt.translate._('noRouteFound'));
            gt.tooltip.showTooltip();
        } else {
            gt.layer.vector.addFeatures(this.pathLines);
        }
    },
    
    setPathPoint: function(id, the_geom, letter) {
        
        for(var i=0; i < this.pathPoints.length; i++) {
            if(this.pathPoints[i].attributes.id == id) {
                gt.layer.vector.removeFeatures([this.pathPoints[i]]);
                this.pathPoints[i].destroy();
                
                this.pathPoints.splice(i, 1);
            }
        }
        
        var geometry = OpenLayers.Geometry.fromWKT(the_geom);
        var feature = new OpenLayers.Feature.Vector(geometry);
        
        feature.attributes.id = id;
        feature.attributes.letter = String.fromCharCode(65 + letter);
        
        //Style
        var style = new OpenLayers.Style(this.pathPointStyle);
        style = style.clone();
        style.defaultStyle.label = feature.attributes.letter;
        feature.style = style.defaultStyle;
        
        gt.layer.vector.addFeatures([feature]);
        this.pathPoints.push(feature); 
        
        this.setGetButtonStatus();
        
        return true;
    },
    
    deletePathPoint: function (id) {
        this.setGetButtonStatus();
        
        for(var i=0; i < this.pathPoints.length; i++) {
            if(this.pathPoints[i].attributes.id == id) {
                gt.layer.vector.removeFeatures([this.pathPoints[i]]);
                this.pathPoints[i].destroy();
                
                this.pathPoints.splice(i, 1);
                
                return true;
            }
        }
        
        return false;
    },
    
    formatTextbox: function(el, oRecord, oColumn, oData, oDataTable) {
        var id = oRecord.getData('id');
        
        el.innerHTML = '<input id="' + id + '_input" value="' + oData + '">';
        
        if(! YAHOO.util.Dom.get(id + '_container')) {
            var containerNode = document.createElement('div');
            containerNode.id = id + '_container';
        
            var acNode = document.createElement('div');
            acNode.id = id + '_ac';
            acNode.appendChild(containerNode);
                
            YAHOO.util.Dom.get('gtRoutingAcPH').appendChild(acNode);
        }
        
        YAHOO.util.Event.onAvailable(id + '_ac', Giscuit.Util.bind(function () {
            
            //Destroy old ac
            if(typeof(this.oAc[id + '_ac']) != 'undefined') {
                this.oAc[id + '_ac'].destroy();
                delete this.oAc[id + '_ac'];
            }        
                    
            //
            var acPH = YAHOO.util.Dom.get(id + '_ac');
                        
            //FIXME:
            acPH.style.width = this.getTextColumnWidth() - 5 + 'px';
            acPH.style.position = 'absolute';
            acPH.style.marginTop = '-11px';
        
            var input = YAHOO.util.Dom.get(id + '_input');
            input.style.position = 'static';
            
            var container = YAHOO.util.Dom.get(id + '_container');
            container.style.color = 'black';
                         
            var ac = new YAHOO.widget.AutoComplete(id + '_input', id + '_container', this.layerSourceDs);
            //ac.forceSelection = true; 
            ac.useShadow = false;
            ac.resultTypeList = false;
            
            //FIXME:
            //ac.useIFrame = true;
            
            ac.itemSelectEvent.subscribe(function(oType, oArgs) {
                oRecord.setData('the_geom', oArgs[2].the_geom);
                oRecord.setData('text', oArgs[2].text);
                
                this.setPathPoint(
                    oRecord.getData().id, 
                    oRecord.getData().the_geom, 
                    oRecord.getData().letter
                );
            }, null, this);

            ac.containerCollapseEvent.subscribe(function(oType, oArgs) {
                this.forceSelection();
            }, null, this);
            
            ac.textboxBlurEvent.subscribe(function(oType, oArgs) {
                this.forceSelection();
            }, null, this);
            
            this.oAc[id + '_ac'] = ac;
        }, this));
                
    },
    
    forceSelection: function() {
        var records = this.dt.getRecordSet().getRecords();
        
        for (var i = 0; i < records.length; i++) {
            var data = records[i].getData();
            var input = YAHOO.util.Dom.get(data.id + '_input');

            if(data.text != input.value) {
                input.value = '';
                records[i].setData('text', '');
                records[i].setData('the_geom', '');
                
                this.deletePathPoint(data.id);
            }
        }
    },
    
    formatRemove: function(elCell, oRecord, oColumn, oData) {
        var self = this;
        
        //Clear previous content when updating data
        YAHOO.util.Event.purgeElement(elCell, true);
        elCell.innerHTML = '';
        
        var image = document.createElement('img');
        image.style.cursor = 'pointer';
        image.src = gt.config.httpImagePath + '/list_remove.png';
        image.title = gt.translate._('remove');
        
        //Remove
        YAHOO.util.Event.addListener(image, "click", function(oArgs) {
            var id = oRecord.getData('id');
            
            var dt = self.dt;
            if (dt.getRecordSet().getLength() <= 2) {
                return;
            }
            
            //Destoy autocomplete
            self.oAc[id + '_ac'].destroy();
            delete self.oAc[id + '_ac'];
            
            //
            var acPH = YAHOO.util.Dom.get(id + '_ac');
            acPH.parentNode.removeChild(acPH);

            //
            YAHOO.util.Event.purgeElement(this);
            
            //Delete row
            var htmlRow = this.parentNode.parentNode.parentNode;
            dt.deleteRow(htmlRow);
            
            //Delete path point
            Giscuit.Util.bind(self.deletePathPoint(id), self);
            
            //Update letter
            var records = dt.getRecordSet().getRecords();
            for (var i = 0; i < records.length; i++) {
                  dt.getRecordSet().updateRecordValue(i, 'letter', i);
                   dt.updateRow(records[i], records[i].getData());
                   
                   for(var j=0; j < self.pathPoints.length; j++) {
                       var feature = self.pathPoints[j];
                       
                    if(feature.attributes.id == records[i].getData('id')) {
                        feature.attributes.letter = String.fromCharCode(65 + i);
                        feature.style.label = self.pathPoints[j].attributes.letter;
                        feature.layer.redraw();
                    }
                   }
                   
            }
            
            
        });
        
        elCell.style.textAlign = 'center';
        elCell.appendChild(image);
    },
    
    formatLetter: function(elCell, oRecord, oColumn, oData) {
        elCell.innerHTML = String.fromCharCode(65 + oData);
    },
    
    getTextColumnWidth: function () {
        var panelWidth =  parseInt(gt.accord.get('element').style.width);
        
        return panelWidth - 100;
    },
    
    trigger: function (evt) {
        var lonlat = gt.map.lastInfoClick = gt.map.getLonLatFromViewPortPx(evt.xy);
        
        if(this.dt.getSelectedTrEls().length == 0) {
            return;
        }
        
        var requestString = gt.config.httpToolPath + "/geocode/index/centroid/t/limit/1" + 
            this.getExtentParam() + '/source/' + this.selectedLayer.reverse_source +
            '/location/' + lonlat.lon + ',' + lonlat.lat;
        
        var reverseCallback = {
            success: this.reverse,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };
        
        var request = YAHOO.util.Connect.asyncRequest('GET', requestString, reverseCallback);
    },
    
    reverse: function(o) {
        try {
            var responseText = YAHOO.lang.JSON.parse(o.responseText);
        } catch(e) {
            Giscuit.Util.parseFailure(o, e);
            return;
        }
        
        if (responseText.replyCode != 200) {
            Giscuit.Util.invalidReplyCode(responseText);
            return;
        }
        
        var selectedTrEls = this.dt.getSelectedTrEls();
        if(selectedTrEls.length == 0) {
            return;
        }
        
        if(responseText.records.length == 0) {
            //Tooltip
            gt.tooltip.cfg.setProperty('text', gt.translate._('destinationNotFound'));
            gt.tooltip.showTooltip();
            
            return;
        }
        
        var record = this.dt.getRecord(selectedTrEls[0]);
        var result = responseText.records[0];
        var id = record.getData('id');
        
        if(! result.text) {
            result.text = gt.translate._('nameEmpty')
        };
        
        record.setData('the_geom', result.the_geom);
        record.setData('text', result.text);
        
        YAHOO.util.Dom.get(id + '_input').value = result.text;
                
        this.setPathPoint(
            record.getData().id, 
            record.getData().the_geom, 
            record.getData().letter
        );
    },
    
    setGetButtonStatus: function () {
        var counter = 0;
        var records = this.dt.getRecordSet().getRecords();
        for (var i = 0; i < records.length; i++) {
            var data = records[i].getData();
            if (data.the_geom && data.the_geom.length) {
                counter++;
            }
        }
        
        if (counter < 2) {
            this.getButton.set('disabled', true);
        } else {
            this.getButton.set('disabled', false);
        }
    },
    
    clear: function() {
        this.deactivate();
        
        var elements = [
           'dt' ,
           'addButton' ,    
           'getButton',
           'layersButton' ,
           'algorithmsButton',
           'clickControl'
        ];
        for(var i=0; i < elements.length; i++) {
            if(this[elements[i]] != null) {
                this[elements[i]].destroy();
                this[elements[i]] = null;
            }
        }
        
         //
        this.layers.length = 0;
        this.selectedLayer = null;
        this.selectedAlgorithm = null;
        this.layerSourceDs = null;
        
        //
        for (var i in this.oAc) {
            this.oAc[i].destroy();
            delete this.oAc[i];
        }
        this.oAc = {};
        
        Giscuit.Control.prototype.clear.call(this);
    },
    
    CLASS_NAME: "Giscuit.Control.Routing"
});