/**
 * Giscuit

 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Geocode.js 1555 2011-08-02 15:41:37Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
Giscuit.Map.prototype.gtEventListeners.push({
    'draw': function() {
        gt.control.geocode = new Giscuit.Mobile.Control.Geocode();
    }
});

/**
 * Class: Giscuit.Mobile.Control.Geocode
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Mobile.Control.Geocode = OpenLayers.Class(Giscuit.Control, {
    
    selected: null,
    
    scale: null,
    
    queryDelay: 0.5,
    
    currInput: null,
    
    initialize: function (options) {
        //
        var buttonHtml = '<div><div class="ui-bar ui-bar-c" id="gtGeocodeButton">' +
                '<div class="gtMapControlsElImage" style="background-position: -704px 50%;"></div>' +
                '<div class="gtMapControlsElText">' + gt.translate._('geocodeShort').replace(' ', '&nbsp;') + '</div>' +
            '</div></div>';
        $("#gtMapControls").append(buttonHtml);    
        
        //
        var addButtonEvent = Giscuit.Util.bind(function() {
            // 
            $("#gtGeocodeButton").on(gt.config.clickEvt, this, function(evt) {
                evt.stopImmediatePropagation();
                evt.data.activate();
                return false;
            });
        }, this);
        
        //
        addButtonEvent();
                
        //Online
        gt.control.offline.events.register('online', this, function() {
            $("#gtHomeButton").parent().before(buttonHtml);
            addButtonEvent();
        });
            
        //Offline
        gt.control.offline.events.register('offline', this, function() {
            $('#gtGeocodeButton').off(gt.config.clickEvt);
            $('#gtGeocodeButton').parent().remove();
        });
        if(! gt.control.offline.isOnline) {
            $('#gtGeocodeButton').parent().remove();
        }
        
        Giscuit.Control.prototype.initialize.apply(this, [options]);
    },
    
    render: function () {
        var self = this;
        
        var html = '<div data-role="page" id="gtGeocode" data-dom-cache="true" data-url="gtGeocode">' +
            '<div data-role="header">' +
                '<h1>' + gt.translate._('geocode') + '</h1>' +
                '<a href="#gtMap" data-icon="map" data-direction="reverse" class="ui-btn-right jqm-home">' +
                    gt.translate._('map') + 
                '</a>' + 
            '</div>' +
         
            '<div data-role="content">' + 
                '<input type="text" class="gtAcInput">' +
                '<br>' +
                '<div id="gtGeocodeAcResult" style="position: absolute;">' +
                '<div>' +
            '</div>' +        
            
        '</div>';
            
        $.mobile.pageContainer.append(html);
        $('#gtGeocode').page();
        
        $('#gtGeocode .gtAcInput').css('background-image', 'url(' + gt.config.httpImagePath + '/magnify.png)');
        
        //EVENTS:
        
        //Query
        $('#gtGeocode').on('keyup', '.gtAcInput', this, Giscuit.Util.debounce(
            function() {
                Giscuit.Util.bind(self.query(this), self);
            },
            parseInt(this.queryDelay * 1000)
        ));
        
        //Select item
        $('#gtGeocodeAcResult').on(gt.config.clickEvt, 'li', this, self.resultSelect);
        
        //Select none
        $("#gtGeocode").on(gt.config.clickEvt, this, function(evt) {
            var result = $('#gtGeocodeAcResult');
            result.html('');
            self.currInput = null;
            
            //HACK: change page to map if map button is pressed
            if ($(evt.target).parent().hasClass('jqm-home')) {
                $.mobile.changePage($('#gtMap'), {
                    'changeHash': true
                });
            }
            
            return false;
        });
    },
    
    activate: function () {
        var activated = Giscuit.Control.prototype.activate.call(this);
        
        //Hide map controls
        gt.init.layout.hideMapControls();
        
        //
        if(! $('#gtGeocode').length) {
            this.render();
        }
        
        $.mobile.changePage($('#gtGeocode'), {
            'changeHash': true
        });
        
        //Clear inputs
        $('#gtGeocode .gtAcInput').val('');
        
        //Focus
        $('#gtGeocode .gtAcInput').first().focus();
        
        return activated;
    },
    
    deactivate: function () {
        var deactivated = Giscuit.Control.prototype.deactivate.call(this);
                
        //Destroy features
        if(this.selected != null) {
            gt.layer.vector.destroyFeatures([this.selected]);
            this.selected = null;
        }
        
        return deactivated;
    },
    
    query: function (input) {
        //
        var input = this.currInput = $(input);
        var result = $('#gtGeocodeAcResult');
        
        //Extent
        var extent = gt.map.maxExtent;
        if (gt.config.geocode == 'google') {
            var extent = extent.clone();
            var wgsProj = new OpenLayers.Projection("EPSG:4326");
            extent.transform(gt.map.getProjectionObject(), wgsProj);
        }
        
        input.css('background-image', 'url(' + gt.config.httpImagePath 
            + '/spinner.gif)');
        
        //
        var offset = input.offset();
        result.css('left', offset.left);
        result.css('top', offset.top + input.outerHeight());
        result.width(input.outerWidth());
                        
        var url = gt.config.httpToolPath + "/geocode/index/extent/" + 
            extent.toBBOX() + '/text/' + input.val();
        
        $.ajax({
            'type': 'POST',
            'url': url,
            'success': this.resultDraw,
            'error': Giscuit.Util.jqmAjaxError ,
            'dataType': 'json' ,
            'context': this
        });
    },
    
    resultDraw: function (data) {
        //
        var result = $('#gtGeocodeAcResult');
        var input = this.currInput;
        
        //
        
        this.scale = parseFloat(data.scale);
        
        //
        input.css('background-image', 'url(' + gt.config.httpImagePath + '/magnify.png)');
        
        //
        var html = '<ul data-role="listview" style="margin: 0;">';
        for (var i = 0; i < data.records.length; i++) {
            var geom = data.records[i].the_geom;
            var text = data.records[i].text;
            html += '<li data-geometry="' + geom + '"';
            if (gt.config.geocode == 'google') {
                var viewport = data.records[i].viewport.join();
                html += ' data-viewport="' + viewport + '"';
            }
            html += '><a>' + text + '</a></li>';
        }
        html += '</ul>';
        
        result.html(html);
        result.trigger('create');
    } ,
    
    resultSelect: function(evt) {
        //
        var self = evt.data;
        
        //
        var result = $('#gtGeocodeAcResult');
        var li = $(this);
        var input = self.currInput;
        
        //Geometry
        var geomString = li.jqmData('geometry');
        var geometry = OpenLayers.Geometry.fromWKT(geomString);
        if (gt.config.geocode == 'google') {
            var wgsProj = new OpenLayers.Projection("EPSG:4326");
            geometry.transform(wgsProj, gt.map.getProjectionObject());
        }

        //Selected
        if(self.selected != null) {
            gt.layer.vector.destroyFeatures([self.selected]);
            self.selected = null;
        }
        self.selected = new OpenLayers.Feature.Vector(geometry);
        gt.layer.vector.addFeatures([self.selected]);
        
        //Zoom to viewport
        if (gt.config.geocode == 'google') {
            var viewportArr = li.jqmData('viewport').split(',');
            var viewport = new OpenLayers.Bounds(viewportArr[0], viewportArr[1], viewportArr[2], viewportArr[3]);
            viewport.transform(wgsProj, gt.map.getProjectionObject()); 
                    
            gt.map.zoomToExtent(viewport);
        } else {
            gt.map.zoomToExtent(geometry.getBounds());
            
            if(self.scale) {
                gt.map.zoomToScale(self.scale);
            }
        }
        
        //
        result.html('');
        self.currInput = null;
        
        //
        $.mobile.changePage($('#gtMap'), {
            'changeHash': true
        });
        
        return false;
    },
    
    CLASS_NAME: "Giscuit.Mobile.Control.Geocode"
});