Giscuit.Mobile.Map.prototype.gtEventListeners.push({
    'draw': function() {
        gt.control.incidents = new Giscuit.Mobile.Control.Incidents();
    }
});

/**
 * Namespace: Giscuit.Mobile.Control.Incidents
 */
Giscuit.Mobile.Control.Incidents = OpenLayers.Class(Giscuit.Control, {
    
    features: [],
    
    ignoredFields: ['gid', 'image_id', 'image_name', 'image_url', 'category_color', 'geometry'],
    
    currentActiveValue: 1,
    
    currentCategoryValue: 0,
    
    drawControl: null,
    
    selected: null,
    
    newIncident: null,
    
    //allowManageIncidents: false,
    
    lastSelected: null,
    
    initialize: function (options) {
        Giscuit.Control.prototype.initialize.apply(this, [options]);
            
        //Append control button
        var buttonHtml = '<div>'
            + '<div class="ui-bar ui-bar-c" id="gtIncidentsButton">'
            + '<div class="gtMapControlsElImage" style="background-position: -768px 50%;"></div>'
            + '<div class="gtMapControlsElText">' + gt.translate._('reports') + '</div>'
            + '</div></div>';
        $('#gtLegendButton').parent().after(buttonHtml);
        
        var addButtonEvent = Giscuit.Util.bind(function() {
            //Toggle activate/deactivate
            $("#gtIncidentsButton").on(gt.config.clickEvt, this, function(evt) {
                evt.stopImmediatePropagation();
                
                if($("#gtIncidentsButton").hasClass('ui-btn-active')) {
                    evt.data.deactivate();
                } else {
                    $.mobile.showPageLoadingMsg();
                    evt.data.activate();
                }
                
                return false;
            });
        }, this);
        
        //
        addButtonEvent();
        
        //Online
        gt.control.offline.events.register('online', this, function() {
            $("#gtHomeButton").parent().before(buttonHtml);
            addButtonEvent();
        });
            
        //Offline
        gt.control.offline.events.register('offline', this, function() {
            $("#gtIncidentsButton").off(gt.config.clickEvt);
            $('#gtIncidentsButton').parent().remove();
        });
        
        if(! gt.control.offline.isOnline) {
            $('#gtIncidentsButton').parent().remove();
        }
    },
    
    activate: function () {
        var activated = Giscuit.Control.prototype.activate.call(this);
        
        //Hide map controls
        gt.init.layout.hideMapControls();
        
        //Render pages
        if(! $('#gtIncidents').length) {
            this.render();
        }
        
        $.mobile.hidePageLoadingMsg();

        if (! gt.layer.Incidents.visibility) {
            var layerId = gt.control.layers.getLayerIdByClassName('Incidents');
            gt.control.layers.setSettings('view', layerId, true);
        } else {
            this.buildIncidents();
        }
        
        //Show results page
        $.mobile.changePage($('#gtIncidents'), {
            'changeHash': true
        });
        
        return activated;
    },
    
    deactivate: function () {
        var deactivated = Giscuit.Control.prototype.deactivate.call(this);
        
        if (! deactivated) {
            if(this.lastSelected) {
                this.lastSelected.layer.drawFeature(this.lastSelected, 'default');
            }
        }
        
        if ($("#gtIncidentsButton").hasClass('ui-btn-active')) {
            $("#gtIncidentsButton").removeClass('ui-btn-active');
        }

        if (this.drawControl && this.drawControl.active) {
            this.drawControl.deactivate();
        }
        
        return deactivated;
    },
    
    render: function() {
        //Create page for displaying results
        var html = 
        '<div data-role="page" id="gtIncidents" data-dom-cache="true" data-url="gtIncidents">' +
        '<div data-role="header">' +
            '<h1>' + gt.translate._('reports') + '</h1>' +
            '<a href="#gtMap" data-icon="map" data-direction="reverse" class="ui-btn-right jqm-home">' +
                gt.translate._('map') + 
            '</a>' + 
        '</div>' +
         
        '<div data-role="content">' + 
            '<a href="#gtIncidentsAddButton" data-rel="popup" data-role="button">' + gt.translate._('addReport') + '</a>' +
            '<fieldset data-role="controlgroup" data-type="horizontal" class="gtIncidentsHorizontalFieldset">' +
                '<select id="gtIncidentsActiveSelect" name="gtIncidentsActiveSelect">' +
                    '<option value="0">' + gt.translate._('all') + '</option>' +
                    '<option value="1" selected>' + gt.translate._('open') + '</option>' +
                    '<option value="2">' + gt.translate._('closed') + '</option>' +
                '</select>' + 
                '<select id="gtIncidentsCategorySelect" name="gtIncidentsCategorySelect">' +
                    '<option value="0">' + gt.translate._('allCategories') + '</option>' +
                '</select>' + 
            '</fieldset>' +
            '<br>' +
            '<ul id="gtIncidentsList" data-role="listview">' +
            '</ul>' +
        '</div>' +  
        '<div data-role="popup" id="gtIncidentsAddButton">' +
            '<label><input id="gtIncidentsAddCurrentLocation" type="checkbox"/>' + gt.translate._('myCurrentLocation') + '</label>' +
            '<ul data-role="listview" data-inset="true">' +
                '<li><a href="" class="gtIncidentsAddType" data-value="1">' + gt.translate._('point') + '</a></li>' +
                '<li><a href="" class="gtIncidentsAddType" data-value="2">' + gt.translate._('line') + '</a></li>' +
                '<li><a href="" class="gtIncidentsAddType" data-value="3">' + gt.translate._('polygon') + '</a></li>' +
            '</ul>' +
        '</div>' +
        '</div>';
            
        $.mobile.pageContainer.append(html);
        $('#gtIncidents').page();
        
        //Create page for incidents form
        var formPage = 
        '<div data-role="page" id="gtIncidentsAddPage" data-dom-cache="true" data-url="gtIncidentsAddPage">' +
        '<div data-role="header">' +
            '<h1>' + gt.translate._('addReport') + '</h1>' +
            '<a href="#gtMap" data-icon="map" data-direction="reverse" class="ui-btn-right jqm-home">' +
                gt.translate._('map') + 
            '</a>' + 
        '</div>' +
             
        '<div data-role="content">' +                 
        '</div>';
        
        $.mobile.pageContainer.append(formPage);
        $('#gtIncidentsAddPage').page();
        
        //
        $('.gtIncidentsHorizontalFieldset').children().width('100%');
        $('.gtIncidentsHorizontalFieldset').children().children().each(function (index, el) {
            $(el).width('49%');
        });
        
        //Send request for categories
        $.ajax({
            'type': 'GET',
            'url': gt.config.httpToolPath + '/incidents/getcategories',
            'success': this.populateIncidentCategories,
            'error': Giscuit.Util.jqmAjaxError ,
            'dataType': 'json' ,
            'context': this
        });
        
        //EVENTS:
        
        $("#gtIncidentsAddButton").on(gt.config.downClickEvt, '.gtIncidentsAddType', this, function(evt) {
            evt.stopImmediatePropagation();
            
            var withGeolocation = $("#gtIncidentsAddCurrentLocation").prop('checked');
            var option = $(this).jqmData('value');
            
            if (withGeolocation) {
                gt.control.geolocation.start();
            } 
            
            if (evt.data.drawControl != null) {
                evt.data.drawControl.destroy();
                evt.data.drawControl = null;
            }
            
            if (option == 1) {
                evt.data.drawControl = new OpenLayers.Control.DrawFeature(gt.layer.vector, OpenLayers.Handler.Point, {
                    callbacks: {
                        "done": OpenLayers.Function.bind(evt.data.addIncident, evt.data)
                    }
                });
            } else if (option == 2) {
                evt.data.drawControl = new OpenLayers.Control.DrawFeature(gt.layer.vector, OpenLayers.Handler.Path, {
                    callbacks: {
                        "done": OpenLayers.Function.bind(evt.data.addIncident, evt.data)
                    }
                });
            } else if (option == 3) {
                evt.data.drawControl = new OpenLayers.Control.DrawFeature(gt.layer.vector, OpenLayers.Handler.Polygon, {
                    callbacks: {
                        "done": OpenLayers.Function.bind(evt.data.addIncident, evt.data)
                    }
                });
            }
            
            $.mobile.changePage($('#gtMap'));
            
            $("#gtIncidentsButton").addClass('ui-btn-active');
            
            gt.map.addControl(evt.data.drawControl);
            evt.data.drawControl.activate();
            
            return false;
        });
        
        $("#gtIncidentsActiveSelect").on('change', this, function(evt) {
            evt.data.currentActiveValue = this.options[this.selectedIndex].value;

            gt.layer.Incidents.load(null, evt.data.currentActiveValue, evt.data.currentCategoryValue);
        });
        
        $("#gtIncidentsCategorySelect").on('change', this, function(evt) {
            evt.data.currentCategoryValue = this.options[this.selectedIndex].value;

            gt.layer.Incidents.load(null, evt.data.currentActiveValue, evt.data.currentCategoryValue);
        });
        
        //Show object on map
        $(document).on(gt.config.clickEvt, '.gtIncidentsShowOnMapButton', this, function(evt) {
            var geometryStr = $(this.parentNode.parentNode).jqmData('geometry');
            
            if(typeof(geometryStr) != 'undefined') {
                var geometry = OpenLayers.Geometry.fromWKT(geometryStr);
                
                //
                if(evt.data.lastSelected) {
                    evt.data.lastSelected.layer.drawFeature(evt.data.lastSelected, 'default');
                }

                evt.data.lastSelected = gt.layer.Incidents.getFeatureBy('geometry', geometry.toString());
                evt.data.lastSelected.layer.drawFeature(evt.data.lastSelected, 'select');  

                gt.map.zoomToExtent(geometry.getBounds());
                if (geometry instanceof OpenLayers.Geometry.Point) {
                    gt.map.zoomTo(gt.config.resolutions.length - 3);
                }
            }
            
            $.mobile.changePage($('#gtMap'), {
                'changeHash': true
            });
            
            return false;
        });
        
        //Delete object
        $(document).on(gt.config.clickEvt, '.gtIncidentsDeleteButton', this, function(evt) {
            $.mobile.showPageLoadingMsg();
            this.disabled = true;
            
            var gid = $(this.parentNode.parentNode.parentNode).jqmData('gid');

            url = gt.config.httpToolPath + '/incidents/delete/gid/' + gid;

            $.ajax({
                'type': 'POST',
                'url': url,
                'success': function(data) {
                    $.mobile.hidePageLoadingMsg();
                    $.mobile.changePage($('#gtMap'), {
                        'changeHash': true
                    });
                    gt.layer.Incidents.load(null, evt.data.currentActiveValue, evt.data.currentCategoryValue);
                },
                'error': Giscuit.Util.jqmAjaxError ,
                'dataType': 'json' ,
                'context': this
            });

            return false;
        });
        
        //Close/open object
        $(document).on(gt.config.clickEvt, '.gtIncidentsCloseButton', this, function(evt) {
            $.mobile.showPageLoadingMsg();
            this.disabled = true;
            
            var gid = $(this.parentNode.parentNode.parentNode).jqmData('gid');
            
            url = gt.config.httpToolPath + '/incidents/setstatus/gid/' + gid;

            $.ajax({
                'type': 'POST',
                'url': url,
                'success': function(data) {
                    $.mobile.hidePageLoadingMsg();
                    $.mobile.changePage($('#gtMap'), {
                        'changeHash': true
                    });
                    gt.layer.Incidents.load(null, evt.data.currentActiveValue, evt.data.currentCategoryValue);
                },
                'error': Giscuit.Util.jqmAjaxError ,
                'dataType': 'json' ,
                'context': this
            });

            return false;
        });
        
        //Incidents form events
        var submited = false;
        //Submit incident
        $('#gtIncidentsAddPage').on(gt.config.clickEvt, '#gtIncidentsSubmit', this, function(evt) {
            $(this).button('disable');
            submited = true;
            evt.data.incidentSubmit();
           
            return false;
        });
        
        //Remove geometry if incident is not submited
        $('#gtIncidentsAddPage').on('pagehide', this, function(evt, ui) {
            if (! submited) {
                gt.layer.Incidents.removeFeatures([evt.data.newIncident]);
                evt.data.newIncident = null;
            }
            submited = false;
        });
        
        //Cancel incident
        $('#gtIncidentsAddPage').on(gt.config.clickEvt, '#gtIncidentsBack', this, function(evt) {
            if (submited) {
                gt.layer.Incidents.removeFeatures([evt.data.newIncident]);
                evt.data.newIncident = null;
            }
            
            $.mobile.changePage($('#gtMap'), {
                'changeHash': true
            });
            
            return false;
        });
    },

    buildIncidents: function () {
        //Locally update features
        this.features.length = 0;

        for (var i = 0; i < gt.layer.Incidents.features.length; i++) {
            var tempObj = jQuery.extend(true, {}, gt.layer.Incidents.features[i].attributes);
            tempObj.geometry = gt.layer.Incidents.features[i].geometry;

            this.features.push(tempObj);
        }
        
        var html = '';
        
        for (var a = 0; a < this.features.length; a++) {
            var item = this.features[a];
            html += '<li>' + item.title + '<ul>';
            
            for (var attr in item) {
                //Filter the unnecessary fields
                if (jQuery.inArray(attr, this.ignoredFields) == -1) {
                    //Format "active" field
                    if (attr == 'active') {
                        //Show or hide "active" field
                        if (this.currentActiveValue != 0) {
                            continue;
                        } else if (item[attr] == 't') {
                            html += '<li>' + gt.translate._('status') + ': ' + gt.translate._('open') + '</li>';
                        } else {
                            html += '<li>' + gt.translate._('status') + ': ' + gt.translate._('closed') + '</li>';
                        }
                    //Format "image" field
                    } else if (attr == 'image') {
                        html += '<li>' + item[attr] + '</li>';
                    } else {
                        html += '<li>' + gt.translate._(attr) + ': ' + item[attr] + '</li>';
                    }
                }
            }
            html += '<li data-geometry="' + item.geometry + '"><button type="button" class="gtIncidentsShowOnMapButton" style="z-index: 1100;">' + gt.translate._('showResultsOnMap') + '</button></li>';
            
            if (gt.pluginParams.incidentsSetstatusIsAllowed) {
                if (item['active'] == 't') {
                    var closeOpenText = gt.translate._('close');
                } else {
                    var closeOpenText = gt.translate._('open');
                }
                html += '<fieldset class="gtIncidentsHorizontalFieldset" data-role="controlgroup" data-type="horizontal" data-gid="' + item.gid + '" style="padding-left: 15px; padding-right: 15px;">';
                    html += '<button type="button" class="gtIncidentsCloseButton">' + closeOpenText + '</button>';
                    html += '<button type="button" class="gtIncidentsDeleteButton">' + gt.translate._('delete') + '</button>';
                html += '</fieldset>';
            }
            
            html += '</ul></li>';
        }
        
        Giscuit.Util.jqmCleanPage('gtIncidents');
        
        $("#gtIncidentsList").html(html);
        $("#gtIncidentsList").listview('refresh');
        
        //
        $('.gtIncidentsHorizontalFieldset').children().width('100%');
        $('.gtIncidentsHorizontalFieldset').children().children().each(function (index, el) {
            $(el).width('49%');
        });
        
    },
    
    addIncident: function (geom) {
        //Check if valid Polygon
        if(geom instanceof OpenLayers.Geometry.Polygon && geom.getVertices().length < 3) {
            return;
        }
        
        //Check if valid Line
        if(geom instanceof OpenLayers.Geometry.LineString && geom.getVertices().length == 2
            && geom.components[0].distanceTo(geom.components[1]) == 0) {
            return;
        }
        
        $.mobile.showPageLoadingMsg();

        if (geom instanceof OpenLayers.Geometry.Point && gt.control.geolocation.point != null) {
            var pixels = gt.map.getLayerPxFromLonLat(new OpenLayers.LonLat(geom.x, geom.y));
            var topLeft = gt.map.getLonLatFromViewPortPx({
                'x': pixels.x - gt.map.snapTolerance,
                'y': pixels.y - gt.map.snapTolerance
            });
            var bottomRight = gt.map.getLonLatFromViewPortPx({
                'x': pixels.x + gt.map.snapTolerance,
                'y': pixels.y + gt.map.snapTolerance
            });
            
            var bounds = new OpenLayers.Bounds();
            bounds.extend(topLeft);
            bounds.extend(bottomRight);
            
            var boundsGeometry = bounds.toGeometry();
    
            if (boundsGeometry.intersects(gt.control.geolocation.point.geometry)) {
                this.newIncident = new OpenLayers.Feature.Vector(gt.control.geolocation.point.geometry.clone());
            } else {
                this.newIncident = new OpenLayers.Feature.Vector(geom);
            }
        } else {
            this.newIncident = new OpenLayers.Feature.Vector(geom);
        }

        gt.layer.Incidents.addFeatures([this.newIncident]);
        
        this.drawControl.deactivate();
        $("#gtIncidentsButton").removeClass('ui-btn-active');
        
        var url = gt.config.httpToolPath + '/incidents/add';
        
        $.ajax({
            'type': 'GET',
            'url': url,
            'success': this.getIncidentFormSuccess,
            'error': Giscuit.Util.jqmAjaxError ,
            'dataType': 'json' ,
            'context': this
        });
    },
    
    getIncidentFormSuccess: function(data) {
        var content = $("#gtIncidentsAddPage > div[data-role='content']");
        content.html(data.form).trigger('create');
        
        $.mobile.changePage($('#gtIncidentsAddPage'), {
            'changeHash': true
        });
        
        $('.ui-loader').each(function (evt) {
            $(this).css('display', 'none');
        });
        
        $.mobile.hidePageLoadingMsg();
    }, 
    
    incidentSubmit: function() {
        var url = gt.config.httpToolPath + '/incidents/add';
        var postData = {'geometry': this.newIncident.geometry.toString()};
        
        $('#gtIncidentsForm').ajaxSubmit({
            'type': 'POST',
            'url': url,
            'data': postData,
            'success': this.incidentSubmitSuccess,
            'error': Giscuit.Util.jqmAjaxError ,
            'dataType': 'json',
            'context': this
        }); 
    },
    
    incidentSubmitSuccess: function(data) {
        if (data.form) {
            this.getIncidentFormSuccess(data);
        } else {
            $.mobile.changePage($('#gtMap'), {
                'changeHash': true
            });
            if (this.selected != null ) {
                gt.map.globalSelectControl.unselect(this.selected);
                this.selected = null;
            }
            gt.layer.Incidents.load(null, this.currentActiveValue, this.currentCategoryValue);
        }
    },
    
    populateIncidentCategories: function (data) {
        var options = $("#gtIncidentsCategorySelect");
        
        for (var category in data.records) {
            options.append($("<option />").val(data.records[category].id).text(data.records[category][data.currentKey]));
        }

        $("#gtIncidentsCategorySelect").selectmenu("refresh", true);
    }, 

    CLASS_NAME: "Giscuit.Mobile.Control.Incidents"
});
