/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Privatelayers.js 1555 2011-08-02 15:41:37Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * Class: Giscuit.Mobile.Control.Privatelayers
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Mobile.Control.Privatelayers = OpenLayers.Class(Giscuit.Control, {
    
    records: [],
    
    settingsCurrLayerId: null,

    initialize: function (options) {
        Giscuit.Control.prototype.initialize.apply(this, [options]);
        
        debug.info('Initializing layers');
        
        //
        this.records = Giscuit.Util.getStorageObject('gtLayers');
        
        //Add
        this.add();
        
        //Get records
        if(gt.control.offline.isOnline) {
            $.getJSON(gt.config.httpMobilePath + '/map/settings', Giscuit.Util.bind(function(data) {
                if(data.guest) {
                    window.applicationCache.removeEventListener('updateready', gtUpdateready);
                                    
                    window.applicationCache.addEventListener('obsolete', function(evt) {
                        debug.info('Swaping cache');
                        try { window.applicationCache.swapCache(); } catch(e) { }
                        
                        window.location.reload(true);
                    }, false);
                    
                    window.applicationCache.update();
                    
                    return;
                }
                
                //Layers
                this.records = gt.map.orderRecordsAsc(data.records);
                Giscuit.Util.setStorageObject('gtLayers', this.records);
                
                //User
                gt.config.snap = data.user.snap;
                gt.config.touchOffset = data.user.touch_offset;
                gt.config.tooltip = data.user.tooltip;
                gt.pluginParams = data.user.pluginParams;
                Giscuit.Util.setStorageObject('gtSettings', data.user);
                
                //
                this.addLazyLayers();
            }, this))
            .error(function(evt) {
                if(gt.control.offline.isOnline) {
                    gt.control.offline.setStatus(false);
                }
            });
        }
        
        //
        $("#gtMapControls").append('<div>'
            + '<div class="ui-bar ui-bar-c" id="gtLayersButton">'
            + '<div class="gtMapControlsElImage" style="background-position: 0 50%;"></div>'
            + '<div class="gtMapControlsElText">' + gt.translate._('layers') + '</div>'
            + '</div></div>');
        
        //Activate
        $("#gtLayersButton").on(gt.config.downClickEvt, this, function(evt) {
            //
            evt.stopImmediatePropagation();
            
            //Hide map controls
            gt.init.layout.hideMapControls();
            
            //Render
            var page = '#gtLayers';
            var content = $(page + " > div[data-role='content']");
            if(! content.children().length) {
                $.mobile.showPageLoadingMsg();
                
                setTimeout(Giscuit.Util.bind(function() {
                    evt.data.render();
                    $.mobile.changePage($(page));
                    
                    $.mobile.hidePageLoadingMsg();
                }, evt.data), 5);
            } else {
                $.mobile.changePage($(page));
            }
            
            return false;
        });
    },
    
    render: function() {
        //
        var records = this.records.slice(0);
        
        //Show only offline layers in offline mode
        if(! gt.control.offline.isOnline) {
            var i = records.length;
            while(i--) {
                var rec = records[i];
                
                if(! rec['class']) {
                    continue;
                }
            
                if(rec.offline) {
                    continue;
                }
                
                records.splice(i, 1);
            }
        }
        
        //Build HTML
        var notInOrder = true;
        var rootNode = $('<ul data-role="listview" data-split-icon="gear" id="gtLayersMenu">');
        
        while(notInOrder) {
            notInOrder = false;
        
            for(var i = 0; i < records.length; i++) {
                var liClass = '', 
                    nameClass = '',
                    settingsClass = '',
                    parentNode = null;
                    
                var rec = records[i];
                
                
                //Skip already added records
                if(rootNode.find('#gtLayersMenu-' + rec.id).length) {
                    continue;
                }
                
                //Root
                if(! rec.parent_id) {
                    parentNode = rootNode;
                    
                //Non root
                } else {
                    parentNode = rootNode.find('#gtLayersMenu-' + rec.parent_id);
                    
                    //Skip records if their parent was not added yet
                    if(! parentNode.length) {
                        notInOrder = true;
                        continue;
                    }
                }
                
                //Visibility
                if(rec.view) {
                    liClass += 'ui-btn-active ';
                    settingsClass += 'ui-btn-active ';
                }
                                                                
                //Container
                if(! rec['class']) {
                    liClass += 'gtLayersMenuContainer ';
                    
                    parentNode.append('<li class="' + liClass + '">' +
                        '<div>' + rec.name + '</div>' + 
                        '<ul id="gtLayersMenu-' + rec.id + '" data-split-icon="gear"></ul></li>');
                
                //Layer
                } else {
                    settingsClass += 'gtLayersMenuSettings ';
                    liClass += 'gtLayersMenuName ';
                    
                    parentNode.append('<li class="' + liClass + '" id="gtLayersMenu-' + rec.id + '" data-layer-id="' + rec.id + '">' +
                        '<a class="' + nameClass + '">' +
                        rec.name + 
                        '<a class="' + settingsClass + '"></a>' +
                        '</a></li>');
                }
            }
            
        }
        
        //Remove empty containers in offline mode
        if(! gt.control.offline.isOnline) {
            emptyLength = true;
            while(emptyLength) {
                var emptyNodes = rootNode.find('ul:empty');
                emptyLength = emptyNodes.length;
                
                for(var i=0; i < emptyLength; i++) {
                    $(emptyNodes[i]).parent().remove();
                }
            }
        }
        
        //
        if(! gt.control.offline.isOnline) {
            var orderButtonHtml = '';
        } else {
            var orderButtonHtml = '<a id="gtLayersOrderButton" data-icon="gear" class="ui-btn-left">' + 
                gt.translate._('reorder') + 
            '</a>';
        }
        
        var html = '<div data-role="page" id="gtLayers" data-dom-cache="true" data-url="gtLayers">' +
            '<div data-role="header">' +
                '<h1>' + gt.translate._('layers') + '</h1>' +
                '<a href="#gtMap" data-icon="map" data-direction="reverse" class="ui-btn-right jqm-home">' +
                    gt.translate._('map') + 
                '</a>' +
                orderButtonHtml +            
            '</div>' +
         
            '<div data-role="content">' + 
            '</div>' +
            
        '</div>';
            
        $.mobile.pageContainer.append(html);
        
        var content = $("#gtLayers > div[data-role='content']");
        content.html(rootNode);
        
        $('#gtLayers').page();
        
        //EVENTS:

        //Activate order
        $("#gtLayersOrderButton").on(gt.config.clickEvt, this, function(evt) {
            var content = $("#gtLayersOrder > div[data-role='content']");
            
            if(! $('#gtLayersOrder').length) {
                //Render
                evt.data.renderOrder();
            }
            
            $.mobile.changePage($('#gtLayersOrder'));
            
            evt.data.resizeOrder();
            
            return false;
        });
        
        //Redraw map on page hide 
        $('#gtLayers').on('pagehide', this, function(evt){
            gt.map.refresh();
            gt.map.fixGoogleLayers();
            
            if(gt.control.editgeom && gt.control.editgeom.active) {
                gt.control.editgeom.controlPanel.redraw();
            }
        });
        
        //Toggle layer visibility
        var menuNameTaphold = false;
        $(document).off(gt.config.clickEvt, '.gtLayersMenuName');
        $(document).on(gt.config.clickEvt, '.gtLayersMenuName', this, function(evt) {
            if(menuNameTaphold == false) {
                evt.data.nameClick($(this).data('layer-id'), this);
            }
            
            //
            menuNameTaphold = false;
            
            return false;
        });
        
        //Metadata
        $(document).off('taphold', '.gtLayersMenuName');
        $(document).on('taphold', '.gtLayersMenuName', this, function(evt) {
            var layer = evt.data.getLayerRecordById($(this).jqmData('layer-id'));

            gt.map.showMetadata(layer);

            menuNameTaphold = true;
            
            return false;
        });
        
        $(document).off('pagehide', '#gtMetadata');
        $(document).on('pagehide', '#gtMetadata', this, function(evt) {
            menuNameTaphold = false;
        });
        
        //Layer settings
        $(document).off(gt.config.clickEvt, '.gtLayersMenuSettings');
        $(document).on(gt.config.clickEvt, '.gtLayersMenuSettings', this, function(evt) {
            var layerId = $(this).parent().jqmData('layer-id');
            evt.data.settingsClick(layerId, this);
             
            return false;
        });
    },
    
    renderOrder: function() {
        //Content
        var contentHtml = '';
        for(var i = 0; i < this.records.length; i++) {
            var rec = this.records[i];
            
            if(! rec['class']) {
                continue;
            }
        
            contentHtml += '<li><div class="gtLayersOrderControlGroup" data-layer-id="' + rec.id + '" data-role="controlgroup" data-type="horizontal">' +
                '<a data-direction="down" data-role="button" data-icon="arrow-u" class="gtLayersOrderUpButton" style="width: 40px">&nbsp;</a>' +
                '<input type="button" value="' + rec.name + '" disabled>' +
                '<a data-direction="up" data-role="button" data-icon="arrow-d" class="gtLayersOrderDownButton" style="width: 40px">&nbsp;</a>' +
            '</div></li>';
        }
        
        //Page
        var pageHtml = '<div data-role="page" id="gtLayersOrder" data-dom-cache="true" data-url="gtLayersOrder">' +
            '<div data-role="header" data-position="fixed" data-position="inline">' +
                '<h1>' + gt.translate._('orderLayers') + '</h1>' +
                '<a id="gtLayersOrderSave" data-icon="check" class="ui-btn-right">' + 
                    gt.translate._('save') + 
                '</a>' +
            '</div>' +
 
            '<div data-role="content" style="padding: 0;">' +
                '<ul id="gtLayersOrderList">' +
                    contentHtml +
                '</ul>' +
            '</div>' +        
        '</div>';
        
        $.mobile.pageContainer.append(pageHtml);
        $('#gtLayersOrder').page();
                
        //Disable first and last button
        $('.gtLayersOrderUpButton').first().addClass('ui-disabled');
        $('.gtLayersOrderDownButton').last().addClass('ui-disabled');
        
        //Set text opacity to 1
        $('.gtLayersOrderControlGroup').each(function() {
            $(this).children().each(function(index) {
                $(this).css('margin', 0);
                if(index == 1) {
                    $(this).css('opacity', 1);
                }
            });
        });
        
        //EVENTS:
        
        //Move layer up
        $('#gtLayersOrderList').on(gt.config.clickEvt, '.gtLayersOrderUpButton', this, function(evt) {
            var from = $(this).parent().parent().parent();
            var to = from.prev();
            
            from.detach();
            to.before(from);
            
            return false;
        });
        
        //Move layer down
        $('#gtLayersOrderList').on(gt.config.clickEvt, '.gtLayersOrderDownButton', this, function(evt) {
            var from = $(this).parent().parent().parent();
            var to = from.next();
            
            from.detach();
            to.after(from);
            
            return false;
        });
        
        //Save current layers order
        $('#gtLayersOrderSave').on(gt.config.clickEvt, this, function(evt) {
            var self = evt.data;
            
            var layerIds = [];
            $('.gtLayersOrderControlGroup').each(function() {
                layerIds.push($(this).jqmData('layer-id'));
            });
            
            $.mobile.showPageLoadingMsg();
            
            var url = gt.config.httpMobilePath + '/privatelayers/order/ids/' + layerIds.join();
            
            $.getJSON(url, function(data) {
                for(var i=0; i < layerIds.length; i++) {
                    var rec = self.getLayerRecordById(layerIds[i]);
                    rec.order = i + 1;
                }
                Giscuit.Util.setStorageObject('gtLayers', self.records);
                
                $.mobile.hidePageLoadingMsg();
                $.mobile.changePage($('#gtLayers'));
            })
            .error(Giscuit.Util.jqmAjaxError);
            
            return false;
        });
        
        //Resize
        $(window).on('orientationchange throttledresize', this.resizeOrder);
    },
    
    resizeOrder: function() {
        var nameWidthOffset = -6;
        var nameWidth = $(window).width() + nameWidthOffset  - 
            $('.gtLayersOrderUpButton').first().width() -
            $('.gtLayersOrderDownButton').first().width();
        
        $('.gtLayersOrderControlGroup input').each(function() {
               $(this).parent().width(nameWidth);
        });
    },
    
    add: function () {
        if(gt.control.offline.isOnline) {
            //Main
            gt.layer.main = new OpenLayers.Layer.MapServer("main", gt.config.httpPath + "/draw", {
                layers: "basic"
            }, {
                transitionEffect: "resize",
                ratio: 1.1,
                singleTile: true
            });
            gt.map.addLayer(gt.layer.main);
            
            //Edit
            gt.map.addEditLayer();
            
            //Vector
            gt.map.addVectorLayer();
            
            //
            this.events.triggerEvent('add');
        } else {
            Giscuit.Util.Db.getMetadata(Giscuit.Util.bind(this.addOffline, this));
        }
        
        
    },
    
    addLazyLayers: function() {
        //
        var snapLayers = [];
        var records = gt.map.orderRecordsDesc(this.records.slice(0));
        
        //JavaScript
        for(var i=0; i < records.length; i++) {
            var mIndex = gt.map.getLayerIndex(gt.layer.main);
            records[i].offline = false;
            
            if(! records[i]['js_layer']) {
                continue;
            }
            
            //Add
            var layerClass = records[i]['class'];
            gt.layer[layerClass] = new Giscuit.Mobile.Layer[layerClass]();
            gt.map.addLayer(gt.layer[layerClass]);
            
            //Order
            var currIndex = gt.map.getLayerIndex(gt.layer[layerClass]);
            
            if(records[i]['js_isbase']) {
                gt.map.raiseLayer(gt.layer[layerClass], (currIndex - mIndex + 1) * -1)
            } else {
                gt.map.raiseLayer(gt.layer[layerClass], (currIndex - mIndex - 1) * -1);
            }
            
            //View
            gt.layer[layerClass].setVisibility(records[i]['view']);
            
            //Snap
            if(records[i]['select'] === true && 
                typeof(gt.layer[layerClass].features) != 'undefined') {
                
                snapLayers.push(gt.layer[layerClass]);
            }
            
        }
        
        //Snap
        if (gt.config.snap == 't') {
            snapLayers.push(gt.layer.vector);
            snapLayers.push(gt.layer.edit);
            gt.map.addSnapLayer(snapLayers, true);
        }
        
    },
    
    addOffline: function(metadata) {
        //
        var layers = [];
        var snapLayers = [];
        
        //Order
        var records = gt.map.orderRecordsDesc(this.records.slice(0));

        //SLD
        var layerType = Giscuit.Util.getStorageObject('gtLayerType');
        var mapSLD = Giscuit.Util.getStorageObject('gtMapSLD');
                
        //Default styles
        var defaultStyle = OpenLayers.Util.applyDefaults({
            pointRadius: gt.map.pointRadius ,
            strokeWidth: 3
        }, OpenLayers.Feature.Vector.style["default"]);
        var selectStyle = OpenLayers.Util.applyDefaults({
            pointRadius: gt.map.pointRadius ,
            strokeWidth: 3
        }, OpenLayers.Feature.Vector.style["select"]);
        
        //
        for(var i=0; i < this.records.length; i++) {
            if(! metadata[this.records[i].id]) {
                this.records[i].offline = false;
            } else {
                this.records[i].offline = true;
            }
        }
        
        //
        for(var i=0; i < records.length; i++) {
            var layer = null;
            var record = records[i];
            
            //Skip
            if(! metadata[record.id]) {
                continue;
            }
            
            if(layerType['layer_' + record.id] == 'vector') {
                //
                var styleMap = new OpenLayers.StyleMap( {
                    'default': defaultStyle,
                    'select': selectStyle
                });
                
                styleMap.styles['default'].context = {
                    '__getLabel': Giscuit.Util.bind(function (feature) {
                        //
                        if(! this.label) {
                            return '';
                        };
                                                                                                
                        var labelDefinition = '';
                        for(var j=0; j < this.label_fields.length; j++) {
                            var labelField = feature.attributes[this.label_fields[j]];
                            if(labelField !== null && typeof(labelField) != 'undefined') {
                                labelDefinition += labelField + "\n";
                            }
                        }
                        
                        return labelDefinition;
                    }, record)
                };

                layer = new OpenLayers.Layer.Vector(record['class'], {
                    strategies: [new OpenLayers.Strategy.Fixed()],
                    protocol: new Giscuit.Util.Db.Vector({tableName: 'layer_' + record.id}),
                    'visibility': record.view ,
                    'order': record.order,
                    'layername': record['class'],
                    'maxscaledenom': parseFloat(record.maxscaledenom),
                    'maxScale': parseFloat(record.minscaledenom),
                    'minscaledenom': parseFloat(record.minscaledenom),
                    'minScale': parseFloat(record.maxscaledenom),
                    'styleMap': styleMap
                });

                //Style
                if(typeof(mapSLD['layer_' + record.id]) != 'undefined') {
                    var styleRules = gt.map.getStyleRules(mapSLD['layer_' + record.id]);
                    
                    //Add labels
                    this.setLabel(styleRules, record);
                    
                    layer.styleMap.styles['default'].rules = [];
                    layer.styleMap.styles['default'].addRules(styleRules);
                } else {
                    debug.warn('SLD for layer "' + className + '" was not found');
                }
                                
                snapLayers.push(layer);
                
            } else if (layerType['layer_' + record.id] == 'raster') {
                layer = new OpenLayers.Layer.TMS(record['class'], gt.config.staticserverUrls, {
                    'serviceVersion': '' ,
                    'type': 'jpeg' ,
                    'visibility': record.view ,
                    'order': record.order,
                    'layername': record.id ,
                    'maxscaledenom': parseFloat(record.maxscaledenom),
                    'maxScale': parseFloat(record.minscaledenom),
                    'minscaledenom': parseFloat(record.minscaledenom),
                    'minScale': parseFloat(record.maxscaledenom),
                    'buffer': 1,
                    'isBaseLayer': false,
                    'async': true
                });
                
                //
                layer.protocol = new Giscuit.Util.Db.Raster({tableName: 'layer_' + record.id});
                layer.getURLasync = function(bounds, callback, scope) {
                    //
                    var sortURL = scope.layer.getURL(bounds).split('/');
                    sortURL.splice(0, 3);
                    sortURL = sortURL.join('/');
                    
                    //
                    this.protocol.read({
                        url: sortURL,
                        callback: function(result) {
                            if(result.images.length) {
                                var url = result.images[0].image;
                            } else {
                                var url = gt.config.httpImagePath + '/blank.gif';
                            }
                            callback.call(scope, url);
                        }
                    });
                };
            }
            
            if(layer) {
                layers.push(layer);
                gt.layer[record['class']] = layer;
            }
            
        }
        
        gt.map.addLayers(layers);
        
        //Vector
        gt.map.addVectorLayer();
        
        //Snap
        snapLayers.push(gt.layer.vector);
        gt.map.addSnapLayer(snapLayers, false);

        //
        this.events.triggerEvent('add');

        //
        gt.control.offline.events.triggerEvent('offline');
        gt.init.layout.renderMapControls();
    },
    
    setLabel: function(styleRules, record) {
        //Parse rules
        for(var i=0; i < styleRules.length; i++) {
            for (var type in styleRules[i].symbolizer) break;

            if(type == 'Line') {
                continue;
            }
            
            var symbolizer = styleRules[i].symbolizer[type];
            
            symbolizer.label = '${__getLabel}';
            symbolizer.fontFamily = 'sans-serif'
            symbolizer.fontSize = record.label_size + 'pt';
            symbolizer.fontColor = record.label_color;
            
            if(type == 'Point') {
                symbolizer.labelAlign = 'cb';
            } else {
                symbolizer.labelAlign = 'cm';
            }
        }
        
    },
    
    nameClick: function(id, el) {
        var self = this;
        
        //Offline
        if(! gt.control.offline.isOnline) {
               this.toggleVisibility(id, el);
            return;
        }
    
        //Online
        if($('.ui-loading').length > 0) {
            return;
        }
        $.mobile.showPageLoadingMsg();
        
        var record = this.getLayerRecordById(id);
        var url = gt.config.httpMobilePath + '/privatelayers/edit/id/' + id + '/column/view/value/' + (! record.view);
        
        $.getJSON(url, function(data) {
            self.toggleVisibility(id, el);
            
            $.mobile.hidePageLoadingMsg();
            
            self.events.triggerEvent('changelayer');
        })
        .error(Giscuit.Util.jqmAjaxError);
        
    } ,
    
    toggleVisibility: function(id, el) {
        var record = this.getLayerRecordById(id);
    
        //
        record.view = ! record.view;
        
        //
        if(el) {
            optionsEl = el.children[1];
    
            $(el).removeClass('ui-btn-active');
            $(optionsEl).removeClass('ui-btn-active');
            
            if(record.view) {
                $(el).addClass('ui-btn-active');
                $(optionsEl).addClass('ui-btn-active');
            }
        }
        
        //
        if(! gt.control.offline.isOnline) {
            record.changed = true;
        }
        Giscuit.Util.setStorageObject('gtLayers', this.records);
        
        //Javascript layer
        if(record.js_layer || ! gt.control.offline.isOnline) {
            var layer = gt.layer[record['class']];
            if(typeof(layer) != "undefined") {
                layer.setVisibility(record.view);
            }
        }
    } ,
    
    settingsClick: function(id, el) {
        //
        var maxFields = gt.config.maxFieldsAmount;
        this.settingsCurrLayerId = id;
        var submitDisabled = '';
        
        //
        var record = this.getLayerRecordById(id);
                
        //Content
        var html = '<form id="gtPrivatelayersSettingsForm" enctype="application/x-www-form-urlencoded" method="post"><dl class="zend_form">';
        
        //Information
        if(record.fields.length || record.query) {
            html += '<dt id="select-label"><label for="select" class="required">' + gt.translate._('information') + '</label></dt>' +
            '<dd id="select-element"><input type="checkbox" name="select" id="select" value="t"' + ((record.select) ? ' checked="checked"' : '');
            if (record.select == 'none') {
                html += ' disabled></dd>';
                submitDisabled = ' disabled';
            } else {
                html += '></dd>';
            }
        }

        if(record.fields.length) {
            //Label
            html += '<dt id="label-label"><label for="label" class="required">' + gt.translate._('label') + '</label></dt>' +
            '<dd id="label-element"><input type="checkbox" name="label" id="label" value="t"' + ((record.label) ? ' checked="checked"' : '') + '></dd>';
        }

        //Edit
        if(record.edit != 'none') {
            html += '<dt id="edit-label"><label for="edit" class="required">' + gt.translate._('edit') + '</label></dt>' +
            '<dd id="edit-element"><input type="checkbox" name="edit" id="edit" value="t"' + ((record.edit) ? ' checked="checked"' : '') + '></dd>';
        }

        //Label settings
        if(record.fields.length && (! record.classlabel_active || ! gt.control.offline.isOnline)) {
            html += '<dt id="label_size-label"><label for="label_size" class="required">' + gt.translate._('size') + '</label></dt>' +
            '<dd id="label_size-element"><input type="text" name="label_size" id="label_size" value="' + record.label_size + '"></dd>' +
            
            '<dt id="label_color-label"><label for="label_color" class="required">' + gt.translate._('color') + '</label></dt>' +
            '<dd id="label_color-element"><input data-role="none" type="text" name="label_color" id="label_color" value="' + record.label_color + '"></dd>' +
    
            '<dt id="label_fields-label"><label for="label_fields" class="optional">' + gt.translate._('fields') + '</label></dt>' +
            '<fieldsset data-role="controlgroup">';
                    
            for(var i=0; i < record.fields.length; i++) {
                if(i >= maxFields) {
                    debug.warn('Maximum number of fields (' + maxFields + ') reached');
                    var firstNFieldsDisplayed = gt.translate._('firstNFieldsDisplayed').replace("%1$s", gt.config.maxFieldsAmount);
                    Giscuit.Util.jqmMessage(firstNFieldsDisplayed);
                    break;
                } 
                
                var checked = '';
                if(Giscuit.Util.inArray(record.label_fields, record.fields[i])) {
                    var checked = ' checked="checked"';
                }
                
                html += '<label for="label_fields-' + record.fields[i] + '">' +
                '<input type="checkbox" name="label_fields[]" id="label_fields-' + record.fields[i] + '" value="' + record.fields[i] + '"' + checked + '>' + gt.translate._(record.fields[i]) + '</label>';
            }
    
            html += '</fieldsset>';
        }
   
        html += '<button type="button" name="submit_button" id="gtPrivatelayersLabelSubmit">' + gt.translate._('save') + '</button>' +
        '</dl></form>';
        
        //Page
        var pageHtml = '<div data-role="page" id="gtLayersSettings" data-dom-cache="true" data-url="gtLayersSettings">' +
            '<div data-role="header">' +
                '<h1>' + record.name + '</h1>' +
                '<a href="#gtLayers" data-icon="arrow-l" data-iconpos="notext" data-direction="reverse" class="ui-btn-right jqm-home">' +
                    gt.translate._('back') +
                '</a>' + 
            '</div>' +
            '<div data-role="content">' + html + '</div>' +        
        '</div>';
        
        //Render
        $('#gtLayersSettings').off();
        $('#gtLayersSettings').remove();
        
        $.mobile.pageContainer.append(pageHtml);
        
        $('#gtLayersSettings').page();
        
        //
        $('#label_color').colorPicker();
        
        //EVENTS:
        
        //Settings submit
        $('#gtPrivatelayersLabelSubmit').on(gt.config.clickEvt, this, function(evt) {
            $(this).button('disable');
                        
            //Size
            var form = $('#gtPrivatelayersSettingsForm');
            var size = form.find('input[name="label_size"]').attr('value');
            var color = form.find('input[name="label_color"]').attr('value');
            
            if (!isNaN(parseInt(size)) && isFinite(size)) {
                if (parseInt(size) < 4) {
                    size = 4;
                } else if (parseInt(size) > 32) {
                    size = 32;
                }
            } else {
                size = 10;
            }
            form.find('input[name="label_size"]').val(size);
                        
            //Color
            if(! /^#[0-9A-F]{6}$/i.test(color)) {
                color = '#000000';
            }
            record.label_color = color;
            
            if(gt.control.offline.isOnline) {
                var url =  gt.config.httpMobilePath + 
                    '/privatelayers/settings/id/' + record.id;
                                        
                $.ajax({
                    'type': 'POST',
                    'url': url,
                    'data': form.serialize(),
                    'success': evt.data.settingsSubmit ,
                    'error': Giscuit.Util.jqmAjaxError ,
                    'dataType': 'json' ,
                    'context': evt.data
                });
            } else {
                evt.data.settingsSubmit();
            }
            
            return false;
        });
        
        //
        $.mobile.changePage($('#gtLayersSettings'));
    } ,
    
    settingsSubmit: function (data) {

        var record = this.getLayerRecordById(this.settingsCurrLayerId);
        var layer = gt.layer[record['class']];
        
        var form = $('#gtPrivatelayersSettingsForm');
            
        //Select
        record.select = form.find('input[name="select"]').is(':checked');
        
        //Activate deactivate javascript layer snap
        if(record.js_layer && typeof(gt.snap.vector) != 'undefined') {
            var newTargets = [];
            for(var i=0; i < gt.snap.vector.targets.length; i++) {
                var target = gt.snap.vector.targets[i];
                
                if(target.layer != layer) {
                    newTargets.push(target);
                }
            }
                        
            gt.snap.vector.targets = newTargets;
            gt.snap.edit.targets = newTargets;
            if (record.select) {
                var defaults = gt.snap.vector.defaults;
                var newTarget = {
                    'edge': defaults.edge,
                    'edgeTolerance': defaults.tolerance,
                    'node': defaults.node,
                    'nodeTolerance': defaults.tolerance,
                    'tolerance': defaults.tolerance,
                    'vertex': defaults.vertex,
                    'vertexTolerance': defaults.tolerance,
                    'layer': layer
                };
                
                if(typeof(layer.features) != 'undefined') {
                    gt.snap.vector.targets.push(newTarget);
                    gt.snap.edit.targets.push(newTarget);
                }
            }
        }
            
        //Label
        record.label = form.find('input[name="label"]').is(':checked');
                                
        //Edit
        var edit = form.find('input[name="edit"]');
        if(edit.length) {
            if(edit.is(':checked')) {
                for(var i = 0; i < this.records.length; i++) {
                    if(this.records[i]['edit'] === true) {
                        this.records[i]['edit'] = false;
                    }
                }
                record.edit = true;
                   
                if(gt.control.editgeom.active) {
                    gt.control.editgeom.deactivate();
                    gt.control.editgeom.activate();
                }
            } else {
                record.edit = false;
                gt.control.editgeom.deactivate();
            }
        }
            
        //Label size
        var labelSize = form.find('input[name="label_size"]').val();
        if(labelSize) {
            record.label_size = labelSize;
        }
            
        //Label color
        var labelColor = form.find('input[name="label_color"]').val();
        if(labelColor) {
            record.label_color = labelColor;
        }
            
        //Label fields
        record.label_fields = [];
        form.find("input[name='label_fields[]']:checked").each(function() {
            record.label_fields.push($(this).val());
        });
            
        //
        if(! gt.control.offline.isOnline) {
            record.changed = true;
            
            //
            var mapSLD = Giscuit.Util.getStorageObject('gtMapSLD');
            if(typeof(mapSLD['layer_' + record.id]) != 'undefined') {
                var styleRules = gt.map.getStyleRules(mapSLD['layer_' + record.id]);
                this.setLabel(styleRules, record);
                layer.styleMap.styles['default'].rules = [];
                layer.styleMap.styles['default'].addRules(styleRules);
                layer.redraw();
            }
        }
        
        //
        Giscuit.Util.setStorageObject('gtLayers', this.records);
            
        //
        $('#gtPrivatelayersLabelSubmit').button('enable');
               
        //
        $.mobile.changePage($('#gtLayers'));
    } ,
       
    getLayerRecordById: function(val) {
        for(var i = 0; i < this.records.length; i++) {
            if(this.records[i].id == val) {
                return this.records[i];
            }
        }
        
        return false;
    } ,
    
    getEditLayer: function() {
        for(var i = 0; i < this.records.length; i++) {
            if(this.records[i]['edit'] === true && this.records[i]['view'] === true) {
                return this.records[i];
            }
        }
        
        return false;
    },
    
    setSettings: function(settingsName, layerId, action) {
        //
        if(settingsName != 'view') {
            return false;
        }

        //
        var el = null;
        $('.gtLayersMenuName').each(function() {
            if($(this).data('layer-id') == layerId) {
                el = this;
            }
        });
        
        //
        var record = this.getLayerRecordById(layerId);
          if(record.view != action) {
              this.nameClick(layerId, el);
        }
                  
        return true;
    },
    
    getSettings: function(settingsName, layerId, action) {
        return this.getLayerRecordById(layerId)[settingsName]; 
    },
    
    getLayerIdByClassName: function(val) {
        for(var i = 0; i < this.records.length; i++) {
            if(this.records[i]['class'] == val) {
                return this.records[i].id;
            }
        }
        
        return false;
    },
    
    getVisible: function() {
        return [];
    },
          
    CLASS_NAME: "Giscuit.Mobile.Control.Privatelayers"
});