/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Publiclayers.js 1555 2011-08-02 15:41:37Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * Class: Giscuit.Mobile.Control.Publiclayers
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Mobile.Control.Publiclayers = OpenLayers.Class(Giscuit.Control, {
    records : [],
    
    currSettingsLayerId: null,

    initialize: function (options) {
        Giscuit.Control.prototype.initialize.apply(this, [options]);
        
        debug.info('Initializing layers');        
        
        //Get records
        $.getJSON(gt.config.httpMobilePath + '/map/settings', Giscuit.Util.bind(function(data) {
            //
            this.records = gt.map.orderRecordsAsc(data.records);
                
            //Add layers
            this.add();
        }, this))
        .error(Giscuit.Util.jqmAjaxError);
            
        $("#gtMapControls").append('<div>'
            + '<div class="ui-bar ui-bar-c" id="gtLayersButton">'
            + '<div class="gtMapControlsElImage" style="background-position: 0 50%;"></div>'
            + '<div class="gtMapControlsElText">' + gt.translate._('layers') + '</div>'
            + '</div></div>');
            
        $("#gtLayersButton").on(gt.config.clickEvt, this, function(evt) {
            //
            evt.stopImmediatePropagation();
        
            //Hide map controls
            gt.init.layout.hideMapControls();
                
            //Render
            var page = '#gtLayers';
            var content = $(page + " > div[data-role='content']");
            if(! content.children().length) {
                $.mobile.showPageLoadingMsg();
                
                setTimeout(Giscuit.Util.bind(function() {
                    evt.data.render();
                    $.mobile.changePage($(page));
                    
                    $.mobile.hidePageLoadingMsg();
                }, evt.data), 5);
            } else {
                $.mobile.changePage($(page));
            }
                
            return false;
        });

    } ,
    
    add: function () {
        //Order
        var records = gt.map.orderRecordsDesc(this.records.slice(0));
        
        //Url layers
        var urlLayers = [];
        var pathName = window.location.pathname.split('/');
        for(var i=0; i < pathName.length; i++) {
            if(pathName[i] == 'layers') {
                var urlLayers = pathName[i + 1].split(','); 
            }
        }
        //Cookie layer
        var savedLayers = Giscuit.Util.getStorageObject('gtPublicLayers');

        for(var i in records) {
            if(! records[i].layerClass || records[i].merged) {
                continue;
            }
            
            var layerId = records[i].id;
            
            //Visibility
            var visibility = false;
            //Url
            if(urlLayers.length) {
                if(Giscuit.Util.inArray(urlLayers, records[i].id)) {
                    visibility = true;
                }
            //Saved
            } else if(savedLayers.length) {
                if(Giscuit.Util.inArray(savedLayers, records[i].id)) {
                    visibility = true;
                }
            //Default
            } else {
                if(records[i].selected == 't') {
                    visibility = true;
                }
            }
            
            //Javascript
            if(records[i].js_layer == true) {
                gt.layer[records[i].layerClass] = new Giscuit.Mobile.Layer[records[i].layerClass]();
                gt.map.addLayer(gt.layer[records[i].layerClass]);
                gt.layer[records[i].layerClass].setVisibility(visibility);
                
            //Tms
            } else {
                var extension = records[i].format;
                if(extension == 'png24') {
                    extension = 'png';
                }
                
                var layer = new OpenLayers.Layer.TMS(records[i].layerClass, gt.config.staticserverUrls, {
                    'order': records[i].order ,
                    'layername': records[i].layerClass ,
                    'transitionEffect': records[i].transitionEffect ,
                    'serviceVersion': '' ,
                    'type': extension ,
                    'visibility': visibility ,
                    'maxscaledenom': parseFloat(records[i].maxscaledenom),
                    'maxScale': parseFloat(records[i].minscaledenom),
                    'minscaledenom': parseFloat(records[i].minscaledenom),
                    'minScale': parseFloat(records[i].maxscaledenom),
                    'buffer': 1,
                    'isBaseLayer': false
                });
                
                gt.map.addLayer(layer);
                gt.layer[records[i].layerClass] = layer;
            }
        }
        
        //Vector
        gt.map.addVectorLayer();
        
        //
        this.events.triggerEvent('add');
    },
    
    render: function() {
        //
        var notInOrder = true;
        var baseSeparatorDrawn = false;
        var rootNode = $('<ul data-role="listview" data-split-icon="gear" id="gtLayersMenu">');
        
        //
        var visibleLayers = [];
        var visibleRecords = gt.control.layers.getVisible();
        for (var i = 0; i < visibleRecords.length; i++) {
            visibleLayers.push(visibleRecords[i].id);
        }
        
        while(notInOrder) {
            notInOrder = false;
        
            for(var i = 0; i < this.records.length; i++) {
                var liClass = '', 
                    nameClass = '',
                    parentNode = null;
                    
                var record = this.records[i];
                
                //Skip merged records
                if(record.merged) {
                    continue;
                }
                
                //Skip already added records
                if(rootNode.find('#gtLayersMenu-' + record.id).length) {
                    continue;
                }
                
                //Root
                if(! record.parent_id || record.parent_id == '-1') {
                    if (record.parent_id == '-1') {
                        liClass += 'gtLayersMenuBase ';
                    }
                    
                    parentNode = rootNode;
                    
                //Non root
                } else {
                    parentNode = rootNode.find('#gtLayersMenu-' + record.parent_id);
                    
                    //Skip records if their parent was not added yet
                    if(! parentNode.length) {
                        notInOrder = true;
                        continue;
                    }
                }
                
                //Visibility
                if(Giscuit.Util.inArray(visibleLayers, record.id)) {
                    liClass += 'ui-btn-active ';
                }
                                                                
                //Container
                if(! record['class']) {
                    liClass += 'gtLayersMenuContainer ';
                    
                    parentNode.append('<li class="' + liClass + '">' +
                        '<div>' + record.name + '</div>' + 
                        '<ul id="gtLayersMenu-' + record.id + '" data-split-icon="gear"></ul></li>');
                
                //Layer
                } else {
                    liClass += 'gtLayersMenuName ';
                    
                    if (record.parent_id == '-1' && ! baseSeparatorDrawn) {
                        parentNode.append('<li data-role="list-divider">' + gt.translate._('baseLayers') + '</li>');
                        baseSeparatorDrawn = true;
                    }
                    
                    parentNode.append('<li class="' + liClass + '" id="gtLayersMenu-' + record.id + '" data-layer-id="' + record.id + '" data-icon="false">' +
                        '<a class="' + nameClass + '">' +
                        record.name + 
                        '</a></li>');
                }
            }
        }
        
        //Remove empty containers
        var emptyLength = true; 
         while(emptyLength) { 
             var emptyNodes = rootNode.find('ul:empty'); 
             emptyLength = emptyNodes.length; 
             
             for(var i=0; i < emptyLength; i++) { 
                 $(emptyNodes[i]).parent().remove(); 
             } 
        }

        //Render hierarchic layer structure
        var html = '<div data-role="page" id="gtLayers" data-dom-cache="true" data-url="gtLayers">' +
            '<div data-role="header">' +
                '<h1>' + gt.translate._('layers') + '</h1>' +
                '<a href="#gtMap" data-icon="map" data-direction="reverse" class="ui-btn-right jqm-home">' +
                    gt.translate._('map') + 
                '</a>' +       
            '</div>' +
         
            '<div data-role="content">' + 
            '</div>' +
            
        '</div>';
            
        $.mobile.pageContainer.append(html);
        
        var content = $("#gtLayers > div[data-role='content']");
        content.html(rootNode);
        
        $('#gtLayers').page();
        
        //EVENTS:
        
        //Redraw google layers
        $('#gtLayers').on('pagehide', function(evt, ui){
            gt.map.fixGoogleLayers();
        });
            
        //Toggle layer visibility
        var menuNameTaphold = false;
        $(document).on(gt.config.clickEvt, '.gtLayersMenuName', this, function(evt) {
            if(menuNameTaphold == false) {
                evt.data.nameClick($(this).jqmData('layer-id'), this);
                    
                //Select only one base layer at a time
                if($(this).hasClass('gtLayersMenuBase') == true) {
                    var curr = this;
                        
                    $('.gtLayersMenuBase').each(function(index) {
                        if(this == curr) {
                            return;
                        }
                            
                        var el = this;
                        if($(el).hasClass('ui-btn-active')) {
                            var record = evt.data.getLayerRecordById($(this).jqmData('layer-id'));
                            var layer = gt.layer[record.layerClass];
                                
                            $(el).removeClass('ui-btn-active');
                            layer.setVisibility(false);
                        }
                            
                    });
                }
                    
                evt.data.saveLayersState();
            }
                
            //
            menuNameTaphold = false;
                
            return false;
        });
            
        //Metadata
        $(document).on('taphold', '.gtLayersMenuName', this, function(evt) {
            var layer = evt.data.getLayerRecordById($(this).jqmData('layer-id'));

            gt.map.showMetadata(layer);

            menuNameTaphold = true;
                
            return false;
        });
        
        $(document).on('pagehide', '#gtMetadata', this, function(evt) {
            menuNameTaphold = false;
        });
    },
    
    nameClick: function(id, el) {
        var record = this.getLayerRecordById(id);
        var layer = gt.layer[record.layerClass];
        
        if(layer.visibility) {
            if(el) {
                $(el).removeClass('ui-btn-active');
            }
            layer.setVisibility(false);
        } else {
            if(el) {
                $(el).addClass('ui-btn-active');
            }
            layer.setVisibility(true);
        }
        
        this.events.triggerEvent('changelayer');
    } ,

    getLayerRecordById: function(id) {
        for(var i = 0; i < this.records.length; i++) {
            if(this.records[i].layerClass && this.records[i].id == id) {
                return this.records[i];
            }
        }
        
        return false;
    } ,
    
    saveLayersState: function() {
        //
        var visibleLayers = [];
        var visibleRecords = gt.control.layers.getVisible();
        for (var i = 0; i < visibleRecords.length; i++) {
            visibleLayers.push(visibleRecords[i].id);
        }
        
        Giscuit.Util.setStorageObject('gtPublicLayers', visibleLayers);
    },
    
    setSettings: function(settingsName, layerId, action) {
        //
        if(settingsName != 'view') {
            return false;
        }

        //
        var el = null;
        $('.gtLayersMenuName').each(function() {
            if($(this).data('layer-id') == layerId) {
                el = this;
            }
        });
        
        //
        var record = this.getLayerRecordById(layerId);
        var layer = gt.layer[record['class']];
          if(layer.visibility != action) {
              this.nameClick(layerId, el);
        }
        
        return true;
    },
    
    getSettings: function(settingsName, layerId) {
        //
        if(settingsName != 'view') {
            return undefined;
        }

        //
        var record = this.getLayerRecordById(layerId);
        var layer = gt.layer[record['class']];
        
        return layer.visibility;
    },
    
    getLayerIdByClassName: function(val) {
        for(var i = 0; i < this.records.length; i++) {
            if(this.records[i].layerClass == val) {
                return this.records[i].id;
            }
        }
        
        return false;
    } ,
    
    getVisible: function() {
        var data = [];
        
        for(var a=0; a < gt.map.layers.length; a++) {
            var layer = gt.map.layers[a];

            if(! layer.visibility) {
                continue;
            }

            for(var b in this.records) {
                var record = this.records[b];
                
                if(record.layerClass == layer.name) {
                    data.push(record);
                    break;
                }
            }

        }
        
        return data;
    },
    
    CLASS_NAME: "Giscuit.Mobile.Control.Publiclayers"

});