Giscuit.Mobile.Map.prototype.gtEventListeners.push({
    'draw': function() {
        gt.control.routing = new Giscuit.Mobile.Control.Routing();
    }
});

/**
 * Namespace: Giscuit.Mobile.Control.Routing
 */
Giscuit.Mobile.Control.Routing = OpenLayers.Class(Giscuit.Control, {
    
    layers: [],
    
    selectedLayer: null,
    
    selectedInput: null,
    
    currInput: null, 
    
    queryDelay: 0.5,
    
    pathLines: [],
    
    pathLineStyle: null,

    pathPoints: [],
    
    pathPointStyle: null,
    
    clickControl: null,
    
    initialize: function (options) {
        Giscuit.Control.prototype.initialize.apply(this, [options]);
        
        //Set layer style
        this.pathLineStyle = {
            'strokeWidth': 6,
            'strokeColor': '#007FFF',
            'strokeOpacity': 0.5
        };
            
        var pathPointRadius = 20;
        this.pathPointStyle = {
            'pointRadius': pathPointRadius,
            'externalGraphic': gt.config.httpImagePath + '/marker/blank.png',
            'graphicXOffset': pathPointRadius * -1,
            'graphicYOffset': pathPointRadius * -2,
                
            'labelYOffset': pathPointRadius * 2 - 12,
            'fontFamily': 'sans-serif',
            'fontSize': '10pt',
            'fontColor': '#000000',
            'fontWeight': 'bold'
        };

        //Append control button
        var buttonHtml = '<div>'
            + '<div class="ui-bar ui-bar-c" id="gtRoutingButton">'
            + '<div class="gtMapControlsElImage" style="background-position: -832px 50%;"></div>'
            + '<div class="gtMapControlsElText">' + gt.translate._('routing') + '</div>'
            + '</div></div>';
        $("#gtMapControls").append(buttonHtml);
            
        var addButtonEvent = Giscuit.Util.bind(function() {
            //Toggle activate/deactivate
            $("#gtRoutingButton").on(gt.config.clickEvt, this, function(evt) {
                evt.stopImmediatePropagation();
                
                if($("#gtRoutingButton").hasClass('ui-btn-active')) {
                    evt.data.deactivate();
                } else {
                    $.mobile.showPageLoadingMsg();
                    $.ajax({
                        'type': 'GET',
                        'url': gt.config.httpToolPath + '/routing/index',
                        'success': evt.data.activate,
                        'error': Giscuit.Util.jqmAjaxError ,
                        'dataType': 'json' ,
                        'context': evt.data
                    });
                }
                    
                return false;
            });
        }, this);
        
        //
        addButtonEvent();
            
        //Click control
        this.clickControl = new Giscuit.Control.Click({
            'clickCallback': Giscuit.Util.bind(this.trigger, this)
        });
        gt.map.addControl(this.clickControl);
            
        //Online
        gt.control.offline.events.register('online', this, function() {
            $("#gtHomeButton").parent().before(buttonHtml);
            addButtonEvent();
        });
        
        //Offline
        gt.control.offline.events.register('offline', this, function() {
            $('#gtRoutingButton').off(gt.config.clickEvt);
            $('#gtRoutingButton').parent().remove();
        });
        
        if(! gt.control.offline.isOnline) {
            $('#gtRoutingButton').parent().remove();
        }

    },
    
    activate: function (data) {
        var activated = Giscuit.Control.prototype.activate.call(this);
        
        //
        if(data.layers.length == 0) {
            Giscuit.Util.jqmMessage(gt.translate._('noLayersAvailableForRouting'));
            return;
        }
        
        //Hide map controls
        gt.init.layout.hideMapControls();
        
        //Create control page and populate algorithms dropdown
        if(! $('#gtRouting').length) {
            this.render(data);
        }
        
        if (this.pathPoints.length != 0) {
            gt.layer.vector.addFeatures(this.pathPoints);
        }
        
        $.mobile.hidePageLoadingMsg();
        
        //Show results page
        $.mobile.changePage($('#gtRouting'), {
            'changeHash': true
        });
        
        return activated;
    },
    
    deactivate: function () {
        var deactivated = Giscuit.Control.prototype.deactivate.call(this);
        
        if (deactivated == true) {
            if(this.clickControl.active) {
                this.clickControl.deactivate();
            }
            
            gt.layer.vector.removeFeatures(this.pathLines);
            gt.layer.vector.removeFeatures(this.pathPoints);
        }

        return deactivated;
    },
    
    render: function(data) {
        var html = 
        '<div data-role="page" id="gtRouting" data-dom-cache="true" data-url="gtRouting">' +
        '<div data-role="header">' +
            '<h1>' + gt.translate._('routing') + '</h1>' +
            '<a href="#gtMap" data-icon="map" data-direction="reverse" class="ui-btn-right jqm-home">' +
                gt.translate._('map') + 
            '</a>' + 
        '</div>' +
                 
        '<div data-role="content">' + 
            '<div id="gtRoutingHorizontalSelect" data-role="controlgroup">' +
                '<select id="gtRoutingLayersSelect">' +
                '</select>' + 
                '<select id="gtRoutingAlgorithmsSelect">' +
                '</select>' + 
                '<select id="gtRoutingCostsSelect">' +
                '</select>' + 
            '</div>' +
            '<br>' +
            '<ul id="gtRoutingTable" data-role="listview" data-split-icon="delete">' +
            '</ul>' +
            '<br>' +
            '<div data-role="controlgroup">' +
                '<button type="button" id="gtRoutingAddDestination">' + gt.translate._('addDestination') + '</button>' +
                '<button type="button" id="gtRoutingGetDirections">' + gt.translate._('getDirections') + '</button>' +
            '</div>' +
            '<div id="gtRoutingAcResult" style="position: absolute; z-index: 9999;"></div>' +
        '</div>' +        
                
        '</div>';
                    
        $.mobile.pageContainer.append(html);
        $('#gtRouting').page();
        
        //Populate initial values
        this.layers = data.layers;
        this.selectedLayer = this.layers[0];
        var layersSelect = $("#gtRoutingLayersSelect");
        
        for (var i in data.layers) {
            var option = $("<option />").val(data.layers[i].id)
                .text(data.layers[i].name);
            layersSelect.append(option);
        }
        
        //Update select items
        this.updateAlgorithms();
        this.updateCosts();
        
        layersSelect.selectmenu("refresh", true);
        
        //Add first 2 destinations
        this.addDestination();
        this.addDestination();
        
        //Hide select menus with single value
        if (layersSelect.children().length < 2) {
            layersSelect.parent().parent().css('display', 'none');
        }
        
        $('#gtRouting .gtAcInput').css('background-image', 'url(' + gt.config.httpImagePath + '/magnify.png)');
        $('#gtRoutingGetDirections').button('disable');
        
        //EVENTS:
        
        //Add new destination
        $("#gtRoutingAddDestination").on(gt.config.clickEvt, this, function(evt) {
            evt.data.addDestination();
                
            return false;
        });
            
        //Source layer changed
        $("#gtRoutingLayersSelect").on('change', this, function(evt) {
            //Update selected layer
            for (var i = 0; i < evt.data.layers.length; i++) {
                if (evt.data.layers[i].id == $(this).val()) {
                    evt.data.selectedLayer = evt.data.layers[i];
                }
            }
    
            //Reset the list items
            $('#gtRoutingTable').children().remove();
            evt.data.addDestination();
            evt.data.addDestination();
            
            //Update select items
            evt.data.updateAlgorithms();
            evt.data.updateCosts();
                
            gt.layer.vector.removeFeatures(evt.data.pathLines);
            gt.layer.vector.removeFeatures(evt.data.pathPoints);
            evt.data.pathLines.length = 0;
            evt.data.pathPoints.length = 0;
                
        });
            
        $("#gtRoutingGetDirections").on(gt.config.clickEvt, this, function(evt) {
            $(this).button('disable');
                
            var postDataObj = {};
            postDataObj.layer = $('#gtRoutingLayersSelect').val();
            postDataObj.algorithm = $('#gtRoutingAlgorithmsSelect').val();
                
            if($('#gtRoutingCostsSelect').val()) {
                postDataObj.cost = $('#gtRoutingCostsSelect').val();
            }
                
            //Path points
            var points = [];
            $('#gtRoutingTable').children().each(function (index, el) {
                var geom = $(el).jqmData('geometry');
                if (typeof geom != 'undefined' && geom != '') {
                    geom = geom.substring(6, geom.length);
                    geom = geom.substring(0, geom.length - 1);
                    geom = geom.split(' ');
                    points.push(geom);
                }
            });
            
            postDataObj.points = points;
            var jsonFormat = new OpenLayers.Format.JSON();
                
            $.ajax({
                'type': 'POST',
                'url': gt.config.httpToolPath + '/routing/getdirections',
                'data': {data: jsonFormat.write(postDataObj)},
                'success': evt.data.setPathLine,
                'error': Giscuit.Util.jqmAjaxError ,
                'dataType': 'json' ,
                'context': evt.data
            });
                
            return false;
        });
        
        //Autocomplete
        var self = this;
        $('#gtRouting').on('keyup', '.gtAcInput', this, Giscuit.Util.debounce(
            function() {
                Giscuit.Util.bind(self.query(this), self);
            },
            parseInt(this.queryDelay * 1000)
        ));
        
        //Select item 
        $('#gtRoutingAcResult').on(gt.config.clickEvt, 'li', this, self.resultSelect);
        
        //Select none
        $('#gtRouting').on(gt.config.clickEvt, this, function(evt) {
            if ($(evt.target).hasClass('gtAcInput')) {
                self.query(evt.target);
            } else {
                var result = $('#gtRoutingAcResult');
                result.html('');
                self.currInput = null; 
            }
            
            return false;
        });
        
        //Delete a route point
        $('#gtRoutingTable').on(gt.config.clickEvt, '.gtRoutingDeletePoint', this, function (evt) {
            var list = $("#gtRoutingTable");
            
            if (list.children().length <= 2) {
                return;
            }
            
            var listEl = $(evt.currentTarget).parent();
            var id = listEl.jqmData('id');    
            listEl.remove();
            Giscuit.Util.bind(self.deletePathPoint(id), self);  

            //Update letter
            $('.gtRoutingLetter').each(function (index, el) {
                el.innerHTML = self.formatLetter(index);
                
               for(var i = 0; i < self.pathPoints.length; i++) {
                   var feature = self.pathPoints[i];

                   if (feature.attributes.id == $(el).closest('li').jqmData('id')) {
                       feature.attributes.letter = self.formatLetter(index);
                       feature.style.label = self.pathPoints[i].attributes.letter;
                       feature.layer.redraw();
                   }
               }
            });
        });

    },
    
    updateAlgorithms: function() {
        //Update layer algorithms
        var algorithmsSelect = $("#gtRoutingAlgorithmsSelect");
        algorithmsSelect.children().remove();
        for (var i in this.selectedLayer.algorithms) {
            algorithmsSelect.append($("<option />").val(i)
                .text(this.selectedLayer.algorithms[i]));
        }
        
        //Show/hide algorithms select
        if (algorithmsSelect.children().length < 2) {
            algorithmsSelect.parent().parent().css('display', 'none');
        } else {
            algorithmsSelect.parent().parent().css('display', 'block');
        }
        
        algorithmsSelect.selectmenu("refresh", true);
    },
    
    updateCosts: function() {
        //Update layer costs
        var costsSelect = $("#gtRoutingCostsSelect");
        costsSelect.children().remove();
        for (var i in this.selectedLayer.costs) {
            costsSelect.append($("<option />").val(i)
                .text(this.selectedLayer.costs[i]));
        }
        
        //Show/hide costs select
        if (costsSelect.children().length < 2) {
            costsSelect.parent().parent().css('display', 'none');
        } else {
            costsSelect.parent().parent().css('display', 'block');
        }
        
        costsSelect.selectmenu("refresh", true);
    },

    addDestination: function() {
        var list = $("#gtRoutingTable");
        if (list.children().length > 25) {
            return;
        }
        
        var list = $("#gtRoutingTable");
        var letter = this.formatLetter(list.children().length);
        var id = new Date().getTime() + 1;
        var newItemHtml = '<li data-id=' + id + '>' +
            '<a><span class="gtRoutingLetter" style="padding-right: 1em;">' + letter + '</span>' + 
            '<input type="text" class="gtAcInput" style="line-height: 1.4; display:inline-block; width: 100%;"></a>' +
            '<a class="gtRoutingDeletePoint" href=""></a></li>';

        list.append(newItemHtml);
        
        var newItem = list.find('li').last();
        list.listview('refresh');
        
        newItem.trigger('create');

        //CSS
        newItem.find('.ui-link-inherit')
            .css('overflow', 'visible')
            .css('padding-right', '83px');
    },
    
    formatLetter: function (number) {
        return String.fromCharCode(65 + number);
    },
    
    setPathLine: function (data) {
        $('#gtRoutingGetDirections').button('enable');
        
        var records = data.paths;
        if (this.pathLines.length != 0) {
            gt.layer.vector.destroyFeatures(this.pathLines);
            this.pathLines.length = 0;
        }
        
        var bounds = new OpenLayers.Bounds();
        
        for(var a in records) {
            var geometry = OpenLayers.Geometry.fromWKT(records[a].path);
            bounds.extend(geometry.getBounds());
            
            var feature = new OpenLayers.Feature.Vector(geometry);
            feature.style = this.pathLineStyle;
            
            this.pathLines.push(feature);
        }
        
        if (this.pathLines.length == 0) {
            Giscuit.Util.jqmMessage(gt.translate._('noRouteFound'));
        } else {
            gt.layer.vector.addFeatures(this.pathLines);
            gt.map.zoomToExtent(bounds);
            $.mobile.changePage($('#gtMap'), {
                'changeHash': true
            });
        }
    },
    
    setPathPoint: function(id, the_geom, letter) {
        for(var i=0; i < this.pathPoints.length; i++) {
            if(this.pathPoints[i].attributes.id == id) {
                gt.layer.vector.removeFeatures([this.pathPoints[i]]);
                this.pathPoints[i].destroy();
                
                this.pathPoints.splice(i, 1);
            }
        }
        
        var geometry = OpenLayers.Geometry.fromWKT(the_geom);
        var feature = new OpenLayers.Feature.Vector(geometry);
        
        feature.attributes.id = id;
        feature.attributes.letter = String.fromCharCode(65 + letter);
        
        //Style
        var style = new OpenLayers.Style(this.pathPointStyle);
        style = style.clone();
        style.defaultStyle.label = feature.attributes.letter;
        feature.style = style.defaultStyle;
        
        gt.layer.vector.addFeatures([feature]);
        this.pathPoints.push(feature); 
        
        this.setGetButtonStatus();
        
        return true;
    },
    
    deletePathPoint: function (id) {
        this.setGetButtonStatus();
        
        for(var i=0; i < this.pathPoints.length; i++) {
            if(this.pathPoints[i].attributes.id == id) {
                gt.layer.vector.removeFeatures([this.pathPoints[i]]);
                this.pathPoints[i].destroy();
                
                this.pathPoints.splice(i, 1);
                
                return true;
            }
        }
        
        return false;
    },
    
    query: function (input) {
        var input = this.currInput = $(input); 
        var result = $('#gtRoutingAcResult');
        
        input.css('background-image', 'url(' + gt.config.httpImagePath 
            + '/spinner.gif)');
        
        //
        var offset = input.offset();
        result.css('left', offset.left);
        result.css('top', offset.top + input.outerHeight());
        result.width(input.outerWidth());

        var url = gt.config.httpToolPath + "/geocode/index/centroid/t" + 
            this.getExtentParam() + '/source/' + this.selectedLayer.source +
            '/text/' + input.val();
        
        $.ajax({
            'type': 'POST',
            'url': url,
            'success': this.resultDraw,
            'error': Giscuit.Util.jqmAjaxError ,
            'dataType': 'json' ,
            'context': this
        });
    },
    
    resultDraw: function (data) {
        //
        var result = $('#gtRoutingAcResult');
        var input = this.currInput;
        
        //
        this.scale = parseFloat(data.scale);
        
        //
        input.css('background-image', 'url(' + gt.config.httpImagePath + '/magnify.png)');
        
        //
        var html = '<ul data-role="listview" style="margin: 0;">';
        if (data.records.length == 0) {
            //Add default on map options
            html += '<li data-onmap="noGeolocation"><a>' + gt.translate._('getPositionFromMap') + '</a></li>';
            html += '<li data-onmap="geolocation"><a>' + gt.translate._('getPositionFromMyLocation') + '</a></li>';
        } else {
            for (var i = 0; i < data.records.length; i++) {
                var geom = data.records[i].the_geom;
                var text = data.records[i].text;
                html += '<li data-geometry="' + geom + '"';
                if (gt.config.geocode == 'google') {
                    var viewport = data.records[i].viewport.join();
                    html += ' data-viewport="' + viewport + '"';
                }
                html += '><a>' + text + '</a></li>';
            }
        }
        html += '</ul>';
        result.html(html);
        result.trigger('create');
    },

    resultSelect: function(evt) {
        //
        var self = evt.data; 
         var result = $('#gtRoutingAcResult'); 
         var li = $(this); 
         var input = self.currInput; 

        if (typeof (li.jqmData('onmap')) != 'undefined' && li.jqmData('onmap') == 'noGeolocation') {
            self.selectedInput = input;
            self.clickControl.activate();
            
            $.mobile.changePage($('#gtMap'), {
                'changeHash': true
            });
        } else if (typeof li.jqmData('onmap') != 'undefined' && li.jqmData('onmap') == 'geolocation') {
            self.selectedInput = input;
            self.clickControl.activate();
            gt.control.geolocation.start();
            
            $.mobile.changePage($('#gtMap'), {
                'changeHash': true
            });
        } else {
            //
            var geometry = li.jqmData('geometry');
            var text = li.find('a').html();
            var inputLi = $(input).closest('li');
            var id = inputLi.jqmData('id');
            var letter = inputLi.index();
            
            //Save
            $(input).val(text);
            inputLi.jqmData('geometry', geometry);
            inputLi.jqmData('text', text);
            
            //
            self.setPathPoint(id, geometry, letter);
        }       
        
        //Clear
        result.html('');
        self.currInput = null; 

        return false;
    },
    
    forceSelection: function() {
        var self = this;
        $('#gtRoutingTable').children().each(function (index, el) {
            if ($(el).jqmData('text') != $(el).find('input').val()) {
                $(el).find('input').val('');
                $(el).jqmData('text', '');
                $(el).jqmData('geometry', '');
                
                self.deletePathPoint($(el).jqmData('id'));
            }
        });
    },
    
    getExtentParam: function () {
        var wgsProj = new OpenLayers.Projection("EPSG:4326");
        var extent = gt.map.maxExtent.clone();
        var extentParam = '';
        if (this.selectedLayer.source == 'google') {
            extent.transform(gt.map.getProjectionObject(), wgsProj);
            extentParam = "/extent/" + extent.toBBOX();
        }
        
        return extentParam;
    },
    
    trigger: function (evt) {
        this.clickControl.deactivate();
        gt.control.geolocation.stop();
        
        var lonlat = gt.map.lastInfoClick = gt.map.getLonLatFromViewPortPx(evt.xy);
        
        var requestString = gt.config.httpToolPath + "/geocode/index/centroid/t/limit/1" + 
            this.getExtentParam() + '/source/' + this.selectedLayer.reverse_source +
            '/location/' + lonlat.lon + ',' + lonlat.lat;
        
        $.mobile.showPageLoadingMsg();
        $.ajax({
            'type': 'GET',
            'url': requestString,
            'success': this.reverse,
            'error': Giscuit.Util.jqmAjaxError ,
            'dataType': 'json' ,
            'context': this
        });
    },
    
    reverse: function(data) {
        
        if(this.selectedInput == null) {
            return;
        }
        
        if(data.records.length == 0) {
            $.mobile.changePage($('#gtRouting'), {
                'changeHash': true
            });
            
            Giscuit.Util.jqmMessage(gt.translate._('destinationNotFound'));
            
            return;
        }
        
        var inputLi = $(this.selectedInput).closest('li');
        var result = data.records[0];
        var id = inputLi.jqmData('id');
        var letter = 0;
        
        $('#gtRoutingTable').children().each(function (index, el) {
            if ($(el).jqmData('id') == inputLi.jqmData('id')) {
                letter = index;
            }
        });
        
        if(! result.text) {
            result.text = gt.translate._('nameEmpty');
        };
        
        inputLi.jqmData('geometry', result.the_geom);
        inputLi.jqmData('text', result.text);
        $(this.selectedInput).val(result.text);

        this.setPathPoint(id, result.the_geom, letter);
        
        $.mobile.hidePageLoadingMsg();

        $.mobile.changePage($('#gtRouting'), {
            'changeHash': true
        });
    },
    
    setGetButtonStatus: function () {
        var counter = 0;
        $('#gtRoutingTable').children().each(function (index, el) {
            if ($(el).jqmData('geometry') && $(el).jqmData('geometry').length) {
                counter++;
            }
        });

        if (counter < 2) {
            $('#gtRoutingGetDirections').button('disable');
        } else {
            $('#gtRoutingGetDirections').button('enable');
        }
    },
    
    CLASS_NAME: "Giscuit.Mobile.Control.Routing"
});