/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Serial.js 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * Class: Giscuit.Mobile.Control.Serial
 */
Giscuit.Mobile.Control.Serial = OpenLayers.Class(Giscuit.Control, {
    
    initialize: function (options) {
        //
        var self = this;
        
        //
        var addButton = function() {
            $("#gtMapControls").append('<div>'
                + '<div class="ui-bar ui-bar-c" id="gtSerialButton">'
                + '<div class="gtMapControlsElImage" style="background-position: -960px 50%;"></div>'
                + '<div class="gtMapControlsElText">' + gt.translate._('devices') + '</div>'
                + '</div></div>');
            
            $("#gtSerialButton").on(gt.config.clickEvt, self, function(evt) {
                //
                evt.stopImmediatePropagation();
                
                evt.data.activate();
                
                return false;
            });
        }
        
        //
        Giscuit.Map.prototype.gtEventListeners.push({
            'draw': function() {
                if(gt.config.guest == 'f') {
                    addButton();
                }
            }
        });
        
        //
        Giscuit.Control.prototype.initialize.apply(this, [options]);
    },
    
    activate: function () {
        var activated = Giscuit.Control.prototype.activate.call(this);
        
        //Hide map controls
        gt.init.layout.hideMapControls();
        
        //
        if(! $('#gtSerial').length) {
            this.initializePlugin();
        } else {
            $.mobile.changePage($('#gtSerial'));
        }

        return activated;
    },
    
    render: function() {
        var baudrates = [300, 600, 1200, 2400, 4800, 9600, 19200, 38400, 57600, 115200, 230400, 460800, 921600];
        
        var html = '<div data-role="page" id="gtSerial" data-dom-cache="true" data-url="gtSerial">' +
            '<div data-role="header">' +
                '<h1>' + gt.translate._('devices') + '</h1>' +
                '<a href="#gtMap" data-icon="map" data-direction="reverse" class="ui-btn-right jqm-home">' +
                    gt.translate._('map') + 
                '</a>' + 
            '</div>' +
            '<div data-role="content">' +
                '<button id="gtSerialGetDevicesButton">' + gt.translate._('refresh') +  '</button>' + 
                '<ul style="margin-top: 0;" data-role="listview" id="gtSerialDevices"></ul>' +
            '</div>' +    
                '<div data-role="popup" id="gtSerialBaudrate">' +
                '<form style="margin: 1em;">' +
                    '<label>' + gt.translate._('baudrate') + ':</label>' +
                    '<select name="baudrate" id="gtSerialBaudrateSelect"></select>' +
                    '<a data-role="button" data-theme="b" id="gtSerialConnectButton">' +  
                        gt.translate._('connect') +  
                    '</a>' +
                '</form>' +
            '</div>' +
        '</div>';
    
        //
        $.mobile.pageContainer.append(html);
        
        //
        for(var i=0; i < baudrates.length; i++) {
            $('#gtSerialBaudrateSelect').append('<option value="' + baudrates[i] + '">' + baudrates[i] + '</option>');
        }
        $('#gtSerialBaudrateSelect').val(4800);
        
        //
        $('#gtSerial').page();
        
        //
        $('#gtSerialGetDevicesButton').button('disable');
        
        //EVENTS:
        var list = $('#gtSerialDevices');
        var currDeviceAddress , currDeviceLi;
        
        //Get devices response
        this.events.register("getdevices", this, function(data) {
            list.html('');
            
            for(var i=0; i < data.length; i++) {
                var liClass = '';
                if(currDeviceAddress == data[i].address) {
                    var liClass = ' ui-btn-active';
                }
                
                list.append('<li class="' + liClass + '" data-address="' +  data[i].address + '"><a>' +  data[i].label + '</a></li>');
            }
            
            list.listview('refresh');
            
            $('#gtSerialGetDevicesButton').button('enable');
        });
        
        //Open response
        this.events.register("open", this, function(evt) {
            $.mobile.hidePageLoadingMsg();
            $('#gtSerialGetDevicesButton').button('enable');
            
            if(evt === true) {
                Giscuit.Util.jqmMessage('connectedSuccessfully');
            } else {
                //
                list.find('li.ui-btn-active').each(function() {
                    $(this).removeClass('ui-btn-active');
                });
                
                //
                Giscuit.Util.jqmMessage(evt);
            }
        });
        
        //Open
        list.on(gt.config.clickEvt, 'li', this, function(evt) {
            //
            if($(this).hasClass('ui-btn-active')) {
                //Close
                currDeviceAddress = currDeviceLi = undefined;
                evt.data.close();
                
                //
                $(this).removeClass('ui-btn-active');
                
            } else {
                //
                list.find('li.ui-btn-active').each(function() {
                    if(currDeviceLi == $(this)) {
                        //Close
                        currDeviceAddress = currDeviceLi = undefined;
                        evt.data.close();
                        
                    }
                    
                    //
                    $(this).removeClass('ui-btn-active');
                });
                
                //
                currDeviceAddress = $(this).jqmData('address');
                currDeviceLi = $(this);
                
                //
                $(this).addClass('ui-btn-active');
                $('#gtSerialGetDevicesButton').button('disable');
                
                //Open
                if(Giscuit.Util.isMobile) {
                    $.mobile.showPageLoadingMsg();
                    evt.data.open(currDeviceAddress, null);
                } else {
                    $('#gtSerialBaudrate').popup('open');
                }
            }
        });
        
        //Select baudrate
        $('#gtSerialConnectButton').on(gt.config.clickEvt, this, function(evt) {
            $('#gtSerialBaudrate').popup('close');
            
            var baudrate = $('#gtSerialBaudrateSelect').val();
            
            $.mobile.showPageLoadingMsg();
            evt.data.open(currDeviceAddress, baudrate);
        });
        
        //Refresh
        $('#gtSerialGetDevicesButton').on(gt.config.clickEvt, this, function(evt) {
            $(this).button('disable');
                        
            evt.data.getDevices();
        });
          
        //
        $.mobile.hidePageLoadingMsg();
        $.mobile.changePage($('#gtSerial'));
        
        //
       this.getDevices();
    },

    CLASS_NAME: "Giscuit.Mobile.Control.Serial"
});

//Initialize
gt.control.serial = new Giscuit.Mobile.Control.Serial();

if(Giscuit.Util.isMobile) {
    window.addEventListener("load", function() {
        document.addEventListener("deviceready", function() {
            gt.control.serial = $.extend(gt.control.serial, Giscuit.Util.Serialphonegap);
        }, false);
    }, false);
} else {
    gt.control.serial = $.extend(gt.control.serial, Giscuit.Util.Serialjava);
}