<?php

class NewsControllerTest extends Giscuit_Test_PHPUnit_ControllerTestCase
{

    protected $_newsArray = array(
        'title_en' => 'Test Title' , 
        'tags_en' => 'test tag' , 
        'desc_en' => 'test descript' , 
        'text_en' => 'test content' ,
        'active' => 't'
    );

    protected function _retrieveDbEntry ($whereTerm, array $fields = array())
    {
        $table = new Giscuit_Db_Table_News();
        $config = Zend_Registry::get('config');
        
        $select = $table->select();
        $select->setIntegrityCheck(false);
        $select->from($config->database->schema . '.news', $fields);
        $table->joinMultilangField($select, 'title');
        $select->where('text_title.en = ?', $whereTerm);
        
        return $table->fetchAll($select)->current();
    }

    public function testDefaultAction ()
    {
        $this->dispatch('/');
        $this->assertResponseCode(200);
        $this->assertModule('default');
        $this->assertController('news');
        $this->assertAction('index');
    }

    public function testIndexAction ()
    {
        //Auth
        $this->_testLoginActionSuccess();
        
        $this->dispatch('/' . $this->_language . '/default/news');
        $this->assertModule('default');
        $this->assertController('news');
        $this->assertAction('index');
    
    }
    
    public function testIfExistsTestEntryDelete()
    {
        $this->_testLoginActionSuccess();
        
        $result = $this->_retrieveDbEntry($this->_newsArray['title_en'], array('id'));
        if ( $result != null ) {
            $this->dispatch('/' . $this->_language . '/admin/news/delete/ids/' . $result->id);
            $response = $this->getResponse()->getBody();
            $response = Zend_Json::decode($response);
            $this->assertEquals($response['replyCode'], 200);
        }
    }

    public function testAddNewNewsEntry ()
    {
        $this->_testLoginActionSuccess();
        $this->request->setMethod('POST')->setPost($this->_newsArray);
        $this->dispatch('/' . $this->_language . '/admin/news/add/browse/submit');
        $this->assertRedirectTo('/' . $this->_language . '/admin/news');
    }

    public function testFetchAfterAddingAction ()
    {
        $result = $this->_retrieveDbEntry('Test Title');
        $this->assertEquals($result->title, 'Test Title');
    }

    public function testEditNewsEntryAction ()
    {
        $this->_testLoginActionSuccess();
        $result = $this->_retrieveDbEntry('Test Title', array('id'));
        
        $editArr = $this->_newsArray;
        $editArr['title_en'] = 'Edited Title';
        $this->request->setMethod('POST')->setPost($editArr);
        $this->dispatch('/' . $this->_language . '/admin/news/edit/id/' . $result->id . '/browse/submit');
        $this->assertRedirectTo('/' . $this->_language . '/admin/news');
    }

    public function testFetchAfterEditAction ()
    {
        $result = $this->_retrieveDbEntry('Edited Title');
        $this->assertEquals($result->title, 'Edited Title');
    }

    public function testRssAction()
    {   
        $this->dispatch('/' . $this->_language . '/default/feed/rss');

        $response = $this->getResponse()->getBody();
        
        $this->assertContains('<title><![CDATA[Edited Title]]></title>', $response);
    }
    
    public function testDeleteAction ()
    {
        $this->_testLoginActionSuccess();
        $result = $this->_retrieveDbEntry('Edited Title', array('id'));
        
        $this->dispatch('/' . $this->_language . '/admin/news/delete/ids/' . $result->id);
        $response = $this->getResponse()->getBody();
        $response = Zend_Json::decode($response);
        $this->assertEquals($response['replyCode'], 200);
    }

    public function testFetchAfterDeleteAction ()
    {
        $result = $this->_retrieveDbEntry('Edited Title');
        $this->assertNull($result);
    }

}