<?php

class SettingsControllerTest extends Giscuit_Test_PHPUnit_ControllerTestCase
{
    protected $_nonDefArr = array (
        'giscuit_language' => 'en',
        'snap' => 't',
        'timezone' => 'Europe/Chisinau' ,
        'tooltip' => 'f',
    	'touch_offset' => 1
    );
    
    protected $_defaultArr = array (
        'giscuit_language' => 'en',
        'snap' => 'f',
        'timezone' => 'Europe/Chisinau' , 
        'tooltip' => 't',
    	'touch_offset' => 3
    );
    
    public function testSettingsIndexAction ()
    {
        $this->_testLoginActionSuccess();
        
        $this->dispatch('/' . $this->_language . '/default/settings');
        $this->assertModule('default');
        $this->assertController('settings');
        $this->assertAction('index');
        $this->assertQuery('form#gtSettingsForm');
    }
    
    public function testNonDefaultSettingsSubmitAction()
    {
        $translate = Zend_Registry::get('Zend_Translate');
        
        $this->_testLoginActionSuccess();
        
        $this->request->setMethod('POST')->setPost($this->_nonDefArr);
        $this->dispatch('/' . $this->_language . '/default/settings');
        
        $response = Zend_Json::decode($this->getResponse()->getBody());
        $this->assertEquals($response['replyCode'], 200);
        $this->assertEquals($response['result'], $translate->_('changesSaved'));
        
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->userId();
        $users = new Giscuit_Db_Table_Users();
        $userData = $users->getItems(array(
            'whereId' => $userId
        ))->current()->toArray();
        $config = Zend_Registry::get('config');
        
        $this->assertEquals($userData['language'],$this->_nonDefArr['giscuit_language']);
        $this->assertEquals($userData['snap'],$this->_nonDefArr['snap']);
        $this->assertEquals($userData['tooltip'],$this->_nonDefArr['tooltip']);
    }
    
    public function testDefaultSettingsSubmitAction()
    {
        $translate = Zend_Registry::get('Zend_Translate');
        
        $this->_testLoginActionSuccess();
        
        $this->request->setMethod('POST')->setPost($this->_defaultArr);
        $this->dispatch('/' . $this->_language . '/default/settings');
                
        $response = Zend_Json::decode($this->getResponse()->getBody());
        $this->assertEquals($response['replyCode'], 200);
        $this->assertEquals($response['result'], $translate->_('changesSaved'));
        
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->userId();
        $users = new Giscuit_Db_Table_Users();
        $userData = $users->getItems(array(
            'whereId' => $userId
        ))->current()->toArray();
        
        $this->assertEquals($userData['language'],$this->_defaultArr['giscuit_language']);
        $this->assertEquals($userData['snap'],$this->_defaultArr['snap']);
        $this->assertEquals($userData['tooltip'],$this->_defaultArr['tooltip']);
    }
    
    public function testLanguageAction()
    {
        $config = Zend_Registry::get('config');
        
        $lang['key'] = 'en';
        
        $this->request->setMethod('POST')->setPost($lang);
        $this->dispatch('/' . $this->_language . '/default/settings/changelang');
        
        $this->assertRedirect('/' . $lang['key'] . '/' . implode('/', $config->defaultRoute->toArray()));
    }
    
}