<?php

class WmsControllerTest extends Giscuit_Test_PHPUnit_ControllerTestCase
{
    protected $_wmsParams = array(
        'LAYERS' => 'Worldboundaries' , 
        'SERVICE' => 'WMS' , 
        'VERSION' => '1.1.1' , 
        'REQUEST' => 'GetMap' , 
        'WIDTH' => '904' , 
        'HEIGHT' => '785' , 
        'SRS' => 'EPSG%3A4326' , 
        'BBOX' => '28.069405078882,46.975994527363,28.07425451278,46.980205595517',
        'STYLES' => 'default' ,
        'FORMAT' => 'image/png' ,
        'map_size' => '904+785' 
    );
    
    protected $_validLogin = array(
        'account' => 'test@giscuit.com' , 
        'password' => '123456'
    );
    
    public function testGetCapabilities()
    {
        $this->dispatch('/' . $this->_language . '/default/wms/public?SERVICE=WMS&REQUEST=GetCapabilities');
        
        $response = $this->getResponse()->getBody();
        
        $this->assertContains('<Name>Worldboundaries</Name>', $response);
    }
    
    public function testPublicAction()
    {
        $queryString = '';
        foreach ($this->_wmsParams as $key => $value) {
            $queryString .= "{$key}={$value}&";  
        }
        
        $url = '/' . $this->_language . '/default/wms/public?' . $queryString;        
        $this->dispatch($url);
        
        $response = $this->getResponse()->getBody();
        
        //
        $mapSize = explode('+', $this->_wmsParams['map_size']);

        //Get image size
        file_put_contents(GT_TMP_PATH . DIR_SEP . 'WmsControllerTest', $response);
        $size = getimagesize(GT_TMP_PATH . DIR_SEP . 'WmsControllerTest');
        unlink(GT_TMP_PATH . DIR_SEP . 'WmsControllerTest');
        
        //Assert size
        $this->assertEquals($size[0], $mapSize[0]);
        $this->assertEquals($size[1], $mapSize[1]);
        
    }
    
    public function testPrivateAction()
    {
        $queryString = '';
        foreach ($this->_wmsParams as $key => $value) {
            $queryString .= "{$key}={$value}&";  
        }
        $this->_validLogin['password'] = md5($this->_validLogin['password']);
        foreach ($this->_validLogin as $key => $value) {
            $queryString .= "{$key}={$value}&";
        }
        
        $url = '/' . $this->_language . '/default/wms/private?' . $queryString;        
        $this->dispatch($url);
        
        $response = $this->getResponse()->getBody();
        
        //
        $mapSize = explode('+', $this->_wmsParams['map_size']);
        
        //Get image size
        file_put_contents(GT_TMP_PATH . DIR_SEP . 'WmsControllerTest', $response);
        $size = getimagesize(GT_TMP_PATH . DIR_SEP . 'WmsControllerTest');
        unlink(GT_TMP_PATH . DIR_SEP . 'WmsControllerTest');
        
        //Assert size
        $this->assertEquals($size[0], $mapSize[0]);
        $this->assertEquals($size[1], $mapSize[1]);
    }
    
    public function testGetFeatureInfo()
    {
        $queryString = '';
        foreach ($this->_wmsParams as $key => $value) {
        	if ($key == 'REQUEST') {
        		$queryString .= "{$key}=GetFeatureInfo&"; 
        	} else {
        		$queryString .= "{$key}={$value}&"; 
        	}
        }
        $queryString .= "QUERY_LAYERS=Worldboundaries&INFO_FORMAT=text/plain&X=353&Y=190";
        
        $url = '/' . $this->_language . '/default/wms/public?' . $queryString;        
        $this->dispatch($url);
        
        $response = $this->getResponse()->getBody();
        
        $this->assertContains('Moldova', $response);
    }
    
    public function testDescribeLayer()
    {
        $queryString = '';
        foreach ($this->_wmsParams as $key => $value) {
        	if ($key == 'REQUEST') {
        		$queryString .= "{$key}=DescribeLayer&"; 
        	} else {
        		$queryString .= "{$key}={$value}&"; 
        	}
        }
        
        $url = '/' . $this->_language . '/default/wms/public?' . $queryString;        
        $this->dispatch($url);
        
        $response = $this->getResponse()->getBody();
        
        $this->assertContains('typeName="Worldboundaries"', $response);
    }
    
    public function testGetLegendGraphic()
    {
        $queryString = '';
        foreach ($this->_wmsParams as $key => $value) {
        	if ($key == 'REQUEST') {
        		$queryString .= "{$key}=GetLegendGraphic&"; 
        	} else {
        		$queryString .= "{$key}={$value}&"; 
        	}
        }
        $queryString .= "LAYER=Worldboundaries";
        
        $url = '/' . $this->_language . '/default/wms/public?' . $queryString;        
        $this->dispatch($url);
        
        $response = $this->getResponse()->getBody();
        $legend = imagecreatefromstring($response);
        
        $requestSuccess = true;
        if ($legend === false) {
        	$requestSuccess = false;
        }
        imagedestroy($legend);

        $this->assertTrue($requestSuccess);
    }
}